
reader_type={
    "count",--short
    "byte",
    "short",
    "int",
    "datetime",--int
    "string",--(short+UTF8 byte[])
    "bytes",-- (int + byte[])
    "int64",
    "float",
}
reader_type=table.enum(reader_type)

GameDataReader = {}

function GameDataReader.readByte(packetContentBytes,skip)
    return string.unpack(packetContentBytes, "b", skip)
end

function GameDataReader.readShort(packetContentBytes,skip)
    return string.unpack(packetContentBytes, "h", skip)
end

function GameDataReader.readInt(packetContentBytes,skip)
    return string.unpack(packetContentBytes, "i", skip)
end

function GameDataReader.readFloat(packetContentBytes,skip)
    local skip,value=string.unpack(packetContentBytes, "f", skip)
    return skip,tonumber(string.format("%.2f",value))
end

function GameDataReader.readString(packetContentBytes,skip)
    local length
    skip, length = GameDataReader.readShort(packetContentBytes,skip)
    local key = string.sub(packetContentBytes,skip,skip+length-1)
    return (skip+length),key
end

function GameDataReader.readBytes(packetContentBytes,skip)
    local length
    skip, length = GameDataReader.readInt(packetContentBytes,skip)
    local data = string.sub(packetContentBytes,skip,skip+length-1)
    return (skip+length),data
end

function GameDataReader.readLong(packetContentBytes,skip)
	return string.unpack(packetContentBytes, "L", skip)
end

function GameDataReader.readTypeValue(packetContentBytes,skip,type)

    if type==reader_type.byte then
        return GameDataReader.readByte(packetContentBytes,skip)
    elseif type==reader_type.short then
        return GameDataReader.readShort(packetContentBytes,skip)
    elseif type==reader_type.int then
        return GameDataReader.readInt(packetContentBytes,skip)
    elseif type==reader_type.string then
		return GameDataReader.readString(packetContentBytes,skip)
	elseif type==reader_type.float then
		return GameDataReader.readFloat(packetContentBytes,skip)
	elseif type==reader_type.datetime then
		return GameDataReader.readInt(packetContentBytes,skip)
	elseif type==reader_type.int64 then
		return GameDataReader.readLong(packetContentBytes,skip)
	elseif type==reader_type.bytes then
		return GameDataReader.readBytes(packetContentBytes,skip)
    end
end

function GameDataReader.removeFile(filename)

	local file_path = GameDataReader.getWritablePath(filename)
	
	cc.FileUtils:getInstance():removeFile(file_path)
end

function GameDataReader.removePatchFile(filename)

	local storagePath = GameAssetsManager.GetStoragePath()
	
	local file_path = storagePath..filename

	cc.FileUtils:getInstance():removeFile(file_path)
end

function GameDataReader.writeToFile(packetContentBytes,filename)

    local file_path = GameDataReader.getWritablePath(filename)

    local file=io.open(file_path,"wb")
    file:write(packetContentBytes)
    file:flush()
    file:close()
end

function GameDataReader.readFile(file_path)

	if not cc.FileUtils:getInstance():isFileExist(file_path) then
		if PINGTEST then screenString(file_path.." not exist") end
	return end

	local file=io.open(file_path,"rb")
	local packetContentBytes=file:read("*a")
	file:close()
    
	return packetContentBytes
end

function GameDataReader.readFromFile(filename)

    local file_path = GameDataReader.getWritablePath(filename)
    
	return GameDataReader.readFile(file_path)
end

function GameDataReader.readFromLocalFile(filename)

	local file_path = cc.FileUtils:getInstance():fullPathForFilename(filename)

	return GameDataReader.readFile(file_path)
end

function GameDataReader.readFromAppFile(filename)

	local file_path = cc.FileUtils:getInstance():fullPathForFilename(filename)
	if not cc.FileUtils:getInstance():isFileExist(file_path) then return end
	local filedata = cc.FileUtils:getInstance():getStringFromFile(file_path)
	return filedata
end

function GameDataReader.readDataFromAppFile(filename)

    local file_path = cc.FileUtils:getInstance():fullPathForFilename(filename)
    if not cc.FileUtils:getInstance():isFileExist(file_path) then return end
    local filedata = cc.FileUtils:getInstance():getDataFromFile(file_path)
    return filedata
end

function GameDataReader.getWritablePath(filename)

    local file_path = cc.FileUtils:getInstance():getWritablePath()
    
    file_path = file_path..filename
    
    return file_path
end