ArenaScene = {}

ArenaScene = class("ArenaScene",function()
    return cc.Scene:create()
end)

local children={}
local total_rival=5

function ArenaScene:sceneEnter()
    outputString("ArenaScene:sceneEnter()")
end

function ArenaScene:sceneExit()
    ArenaScene.RemoveAllChildren(self)
end

function ArenaScene.RemoveAllChildren(scene)
    table.clear(children)
    GameAssistor.RemoveUnusedSceneResource(scene)
end

function ArenaScene.create()
    local scene = ArenaScene.new()

    Sence_Init(scene)
    -- require('mobdebug').start('localhost')
    local layerFarm = cc.Layer:create()

    scene.getMainLayer=function()
        return layerFarm
    end
    children.mainLayerFarm = layerFarm

    local background = GameAssistor.CreateBackMaskLayout(180)
    layerFarm:addChild(background)

    scene.AddMainChildren()

    scene:addChild(layerFarm)
    user_name = cc.UserDefault:getInstance():getStringForKey("user_name", "")
    -- CommonAssistor.ShowWinBox(1)        
    if user_name == "" then
        CommonAssistor.ShowQuMingBox()
    end

    -- PlayerDataList.Roles[2].unlock = 0
    -- PlayerDataList.Roles[3].unlock = 0
    -- GameRecorder.SaveBasicData()

    return scene
end

local function SetSpine(roleID,nd_spine,scale)
    if nd_spine.roleID==roleID then
        return 
    end
    nd_spine:removeAllChildren() 
    nd_spine.roleID=roleID
    local gData=GameDataTransfer.GetRoleData(roleID)
    local spine = CreateSpineWithModelID(gData.model_id,scale)
    spine:setAnimation(0,"A",true)
    nd_spine:addChild(spine)
end
ArenaScene.SetRoleSpine=SetSpine

function ArenaScene.Refresh()
    ArenaScene.RefreshMyData()
end

function ArenaScene.RefreshMyData()
    children.my_name:setString(user_name)
    children.my_rank:setString(ArenaScene.GetUserData("rank"))
    children.my_power:setString(GameRecorder.GetAmountCombatPower())
end

function ArenaScene.RefreshRivalList()
    -- local names=ArenaScene.GetRandomNames(total_rival)
    local ranks=ArenaScene.GetNextRivalRanks(ArenaScene.GetUserData("rank"))

    for key,nd_main in pairs(children.nd_rivalList) do
        local rank=ranks[key]
        nd_main.my_rank:setString(rank)
        -- nd_main.my_name:setString(names[key])
        local data=ArenaScene.GetRivalData(rank)
        nd_main.data=data 
        data.name=data.name
        nd_main.my_name:setString(data.name)
        nd_main.my_power:setString(data.combat_power)
        SetSpine(data.role_show,nd_main.my_avatar)
    end
end

function ArenaScene.AddMainChildren()

    local layerFarm = children.mainLayerFarm

    local page_layer = cc.CSLoader:createNode("PK.csb")

    layerFarm:addChild(page_layer)

    local nd_mine=page_layer:getChildByTag(1)
    
    local nd_top=nd_mine:getChildByTag(1)
    children.my_rank=nd_top:getChildByTag(2)
    children.my_name=nd_top:getChildByTag(3)
    children.my_avatar=nd_mine:getChildByTag(2)
    children.my_power=nd_mine:getChildByTag(3):getChildByTag(2)
    SetSpine(1,children.my_avatar)

    local bt_rankAward=nd_mine:getChildByTag(4)
    CommonAssistor.AddSimpleTouchEvent(bt_rankAward,function()
        return AchievementScene.QuickOpenTypeStore(AchievementScene.type_task.PVP_RANK)
    end)
    
    children.nd_rivalList={}
    local nd_rivals=page_layer:getChildByTag(2)
    for i=1,total_rival do
        local nd_main=nd_rivals:getChildByTag(i)
        local nd_top=nd_main:getChildByTag(1)
        nd_main.my_rank=nd_top:getChildByTag(2)
        nd_main.my_name=nd_top:getChildByTag(3)
        nd_main.my_avatar=nd_main:getChildByTag(2)
        nd_main.my_power=nd_main:getChildByTag(3):getChildByTag(2)

        children.nd_rivalList[i]=nd_main

        local bt_challenge=nd_main:getChildByTag(4)
        CommonAssistor.AddSimpleTouchEvent(bt_challenge,function()
            if nd_main.data then
                -- table.print(nd_main.data,"bt_challenge")
                if PINGTEST then return Ping_TestAreaChallenge(nd_main.data.rank) end
                return GameManager.StartPvPFight(nd_main.data)
            end
        end)
    end
    
    return ArenaScene.RefreshRivalList()
end

function ArenaScene.GetRandomNames(total)
    local result={}
    while (#result<total) do
        local lastName=Random(str_system.STRING_GAME_LASTNAMEBEGIN,str_system.STRING_GAME_LASTNAMEEND)
        local firstName=Random(str_system.STRING_GAME_FIRSTNAMEBEGIN,str_system.STRING_GAME_FIRSTNAMEEND)
        local fullName=GameDataTransfer.GetGameString(lastName)..GameDataTransfer.GetGameString(firstName)
        local isExist=false
        for i,name in pairs(result) do
            if fullName==name then isExist=true break end
        end
        if not isExist then table.insert(result,fullName) end
    end      
    return result
end

function ArenaScene.GetNextRank(rank)
    if rank<100 then return rank-1 end
    if rank<200 then return rank-Random(3,5) end
    if rank<500 then return rank-Random(7,10) end
    if rank<2000 then return rank-Random(15,20) end
    if rank<5000 then return rank-Random(95,100) end
    return rank-Random(195,200)
end

function ArenaScene.GetNextRivalRanks(rank)
    local result={}
    if rank<=5 then
        for i=6,1,-1 do
            if rank~=i then table.insert(result,i) end
        end
    else
        for i=1,5 do
            rank=ArenaScene.GetNextRank(rank)
            table.insert(result,rank)
        end
    end
    return result
end

function ArenaScene.GetUserData(key)
    return PlayerDataList.Arenas[key] or 0
end

function ArenaScene.SetUserData(key,value)
    PlayerDataList.Arenas[key]=value
end

function ArenaScene.GetRankData(rank)
    for i=rank,1,-1 do
        local data=GameDataTransfer.GetArenaRankData(i)
        if data then return data end
    end
end

function ArenaScene.GetRivalData(rank)
    local result={rank=rank}
    local rankData = ArenaScene.GetRankData(rank)
    table.copy(rankData,result)
    
    local roleIDs={}
    for i=1,3 do
        if result["role_"..i]>0 then table.insert(roleIDs,i) end
    end

    result.combat_power = GameRecorder.GetAttributeCombatPower(result.attack,result.defence,result.life)*(#roleIDs)
    return result
end

function ArenaScene.ChallengeWinner(rank)
    ArenaScene.SetUserData("rank",math.min(rank,ArenaScene.GetUserData("rank")))
    ArenaScene.SetUserData("rankMax",math.min(rank,ArenaScene.GetUserData("rankMax")))
    ArenaScene.SetUserData("lastTime",os.time())
    return GameRecorder.SaveBasicData()
end



