GameMainScene = {}

GameMainScene = class("GameMainScene",function()
    return cc.Scene:create()
end)

local pageZOrder=enum{
    "PAGE_MAIN=0",
    "PAGE_SUBFUNC",
    "PAGE_TOPBAR",
    "PAGE_TOPSUBPAGE",
    "PAGE_WARNING",
    "PAGE_PROTECT",

    "SUBPAGE_ROLE=1",
    "SUBPAGE_SKILLS",
    "SUBPAGE_INTENSIFY",
    "SUBPAGE_PETS",
    "SUBPAGE_RUINEXPLORERS",
    "SUBPAGE_ACHIEVEMENT",
    "SUBPAGE_ARENA",
    "SUBPAGE_MAINFB",
    
    "SUBPAGE_STORE",
    "SUBPAGE_FIRSTCHARGE",
    "SUBPAGE_CHARGE",
}
GameMainScene.pageZOrder=pageZOrder

main_buttons=enum{
    "RETURN=0",
    "FB",
    "ROLE",
    "SKILL",
    "INTENSIFY",
    "PETS",
    "EXPLORERS",
    "TASK",
    "ARENA",
}

local children={}
local page_buttons={}

GameMainScene.AutoCallFunc=nil
function GameMainScene.SetEnterAutoFunc(Func)
    GameMainScene.AutoCallFunc=Func
end

function GameMainScene.AutoCallFunc_SceneEnter()
    if GameMainScene.AutoCallFunc then GameMainScene.AutoCallFunc() end
    GameMainScene.AutoCallFunc=nil
end

function GameMainScene:sceneEnter()
    cclog("GameMainScene sceneEnter")

    CombatPowerUnit.BackupCombatPowers()

    GameMainScene.AutoCallFunc_SceneEnter()
end

function GameMainScene:sceneExit()
    cclog("GameMainScene sceneExit")

    GameMainScene.RemoveAllChildren(self)
end

function GameMainScene.RemoveAllChildren(scene)

    table.clear(children)
    table.clear(page_buttons)

    GameAssistor.RemoveUnusedSceneResource(scene)
end

function GameMainScene.create()

    local scene = GameMainScene.new()

    Sence_Init(scene)

    GameMainScene.AddSubFuncPageChildren(scene)

    GameMainScene.AddMainChildren(scene)
    
    GameMainScene.AddMaskLayout(scene)
    
    if PINGTEST then Ping_AddTestButtons(scene) end

    return scene
end

local function CheckAble(child)
    return checkVisible(child)
end

function GameMainScene.Return()
end

function GameMainScene.Refresh()
end

function GameMainScene.AddMaskLayout(layerFarm)
    local layout = GameAssistor.CreateBackMaskLayout(0)
    layout:setVisible(false)
    layerFarm:addChild(layout,pageZOrder.PAGE_PROTECT,pageZOrder.PAGE_PROTECT)
    children.maskLayout = layout
    return layout
end

function GameMainScene.SetProtectMaskTimer(time)
    local layout=children.maskLayout
    layout:setVisible(true)
    local function PlayAction()
        layout:setVisible(false)
    end
    local actionTo = cc.Sequence:create(cc.DelayTime:create(time),cc.CallFunc:create(PlayAction))
    layout:runAction(actionTo)
end

function GameMainScene.AddSubFuncPageChildren(layerFarm)

    local background = GameAssistor.CreateBackMaskLayout()
    layerFarm:addChild(background,pageZOrder.PAGE_SUBFUNC,pageZOrder.PAGE_SUBFUNC)
    children.subLayerFarm = background
    background:setTouchEnabled(false)
end

function GameMainScene.GetSubFuncPage()
    return children.subLayerFarm 
end

function GameMainScene.RefreshPlayerStateBar(checkVIP)

    if not children.tx_player_diamond then return end

    if checkVIP then StoreScene.RefreshVIPID() end

    local player = PlayerDataList.Basic
    
    local vipString="青铜"
    if player.vip>0 then 
        local data=GameDataTransfer.GetStoreData(player.vip)
        if data then vipString=GameDataTransfer.GetGameString(data.name) end
    end
    children.tx_player_vip:setString(vipString)
    children.tx_player_diamond:setString(player.diamond)
    children.tx_player_coins:setString(player.coin)
    children.tx_player_lv:setString(player.level)
    
    children.bt_firstChargePage:setVisible(player.first==0)
end

function GameMainScene.RefreshPlayerRoleList()
    local nd_roleList=children.nd_roleList
    if not nd_roleList then return end
    
    for roleID=1,3 do
        local nd_role=nd_roleList:getChildByTag(roleID)
        if nd_role then
            local gData=GameDataTransfer.GetRoleData(roleID)
            
            local nd_info=nd_role:getChildByTag(2)
            if not nd_info.spine and not PINGTEST1 then
                local tx_name=nd_info:getChildByTag(1)
                GameAssistor.SetBlackOutlineNoSpace(tx_name)
                tx_name:setString(GameDataTransfer.GetGameString(gData.RoleName))

                local nd_spine=nd_info:getChildByTag(4)
                local spine = CreateSpineWithModelID(gData.model_id,1.2)
                spine:setAnimation(0,"A",true)
                nd_spine:addChild(spine) nd_info.spine=spine
            end

            local role=GameRecorder.GetRole(roleID)
            local nd_state=nd_role:getChildByTag(1)
            local bt_get=nd_state:getChildByTag(1)
            bt_get:setVisible(role.unlock==0)

            local bg_name=nd_info:getChildByTag(0)
            local bg_halo=nd_info:getChildByTag(3)
            bg_halo:setVisible(role.unlock==1)            
            for i=1,3 do
                bg_name:getChildByTag(i):setVisible(roleID==i)
                bg_halo:getChildByTag(i):setVisible(roleID==i)
            end
            
            bt_get:addTouchEventListener(
                function(sender,eventType)
                    if eventType==ccui.TouchEventType.ended then
                        StoreScene.CharageRMB(GameData_List.Game_Shop[roleID - 1])
                    end
                end)
        end
    end
end

function GameMainScene.AddPageButton(key,button,building)
    page_buttons[key] = {key=key,button=button,touch=false,building=building or false}
end

function GameMainScene.GetPageButtonData(key)
    return page_buttons[key] or {}
end

function GameMainScene.SetPageButtonTouchEvent(eventType,key)
    if eventType == ccui.TouchEventType.ended or eventType == ccui.TouchEventType.canceled then
        GameMainScene.SetPageButtonTouch(key,false)
    elseif eventType== ccui.TouchEventType.began then
        GameMainScene.SetPageButtonTouch(key,true)
    end
end

function GameMainScene.SetPageButtonTouch(key,touch)
    GameMainScene.GetPageButtonData(key).touch = touch
    -- if touch then
    --     if key == main_buttons.RETURN then
    --         GameSound.PlayEffect(SoundEffect.UI_CLICK)
    --     else
    --         GameSound.PlayEffect(SoundEffect.UI_CLICK)
    --     end
    -- end
end

function GameMainScene.GetPageButtonTouch(key)
    return GameMainScene.GetPageButtonData(key).touch
end

function GameMainScene.GetPageButton(key)
    return GameMainScene.GetPageButtonData(key).button
end

function GameMainScene.IsBuildingButton(key)
    return GameMainScene.GetPageButtonData(key).building or false
end


function GameMainScene.SetSubMenuActive(pageID)
    if not node_bottom then
        return
    end

    for _,nd in pairs(node_bottom:getChildren()) do
        nd:getChildByTag(0):setVisible(pageID==nd:getTag())
    end

end

function GameMainScene:ShowMianze()

    local layerFarm = GameAssistor.GetRunningScene()
    local background = GameAssistor.CreateBackMaskLayout(180)

    -- local zOrder=GameMainScene.pageZOrder.PAGE_WARNING
    layerFarm:addChild(background,10000,10000)

    local obstructLayout = GameAssistor.CreateBackMaskLayout()
    background:addChild(obstructLayout)
    background.obstructLayout=obstructLayout

    obstructLayout:setAnchorPoint(cc.p(0.5,0.5))
    GameAssistor.SetToSceneCenter(obstructLayout)

   
    
    local function HidePage()
        GameMainScene.HidePageScaleToAction(obstructLayout,RemovePage)
    end

    local page_layer = cc.CSLoader:createNode("mianze.csb")
    GameAssistor.SetToSceneCenter(page_layer)
    obstructLayout:addChild(page_layer)

    local closeBox = function()
        GameMainScene.HidePageScaleToAction(obstructLayout, function()
            layerFarm:removeChild(background)
        end)
    end

    local mianze_txt = [[本游戏免责声明：
1.中国电信爱游戏仅提供此游戏免费下载，在下载后运行此游戏过程中所产生的任何费用与中国电信爱游戏无关；
2.此游戏为通用适配版，不保证每款手机在下载后都能正常运行和显示，部分手机可能因为分辨率或性能原因无法运行或存在问题,中国电信爱游戏对此不承担任何责任；
3.此游戏版权和游戏内所涉及的图片、音乐、文字等内容均归游戏开发商负责，与中国电信爱游戏无关；
4.下载后在运行游戏过程中所产生的疑问，请直接联系该游戏开发商的客服，中国电信爱游戏不承担此游戏的客服服务；
5.此免责声明最终解释权归中国电信股份有限公司游戏运营中心。]]
        local txtMianze = page_layer:getChildByName("txt_mianze")
        txtMianze:setString(mianze_txt)

    GameMainScene.SetObstructLayoutTouchEvent(obstructLayout,closeBox)        
    GameMainScene.PlayPageScaleToAction(obstructLayout)

end


function GameMainScene.AddMainChildren(layerFarm)
    -- GameRecorder.SetBasicData("tutorial1",mian_t_index)
    local page_layer = cc.CSLoader:createNode("UI-RoleChoose.csb")
    layerFarm:addChild(page_layer,pageZOrder.PAGE_MAIN,pageZOrder.PAGE_MAIN)
    children.nd_roleList=page_layer:getChildByTag(1)
    GameMainScene.RefreshPlayerRoleList()
    local page_layer = cc.CSLoader:createNode("UI.csb")

    layerFarm:addChild(page_layer,pageZOrder.PAGE_TOPBAR,pageZOrder.PAGE_TOPBAR)
    
    local bt_hideAll=page_layer:getChildByTag(1)
    bt_hideAll:addTouchEventListener(
        function(sender,eventType)
            GameMainScene.SetPageButtonTouchEvent(eventType,main_buttons.RETURN)
            if eventType==ccui.TouchEventType.ended then
                GameSound.PlayEffect(SoundEffect.UI_CLICK)
                return GameMainScene.HideAllSceneFromSubFuncPage()
            end
        end)
    GameMainScene.AddPageButton(main_buttons.RETURN,bt_hideAll)
    bt_hideAll:setPressedActionEnabled(true)
    
    children.bt_exitgame = page_layer:getChildByTag(5)
    
    children.bt_exitgame:addTouchEventListener(
        function(sender,eventType)
            --GameMainScene.SetPageButtonTouchEvent(eventType,main_buttons.RETURN)
            if eventType==ccui.TouchEventType.ended then
                --cc.PlatformAdapter:openPay("exit","","","")
                cc.PlatformAdapter:openPay("exit","1111","22222","33333",function(p1,p2,p3,p4) end)
            end
        end)
    children.bt_exitgame:setPressedActionEnabled(true)

    local function SetEventButton(node,tag,pageID,key)
        -- cclog("   tag:" .. tag .. "  pageID:" .. pageID )
        local nd_button=node:getChildByTag(tag)
        local button=nd_button:getChildByTag(1)
        button:addTouchEventListener(
            function(sender,eventType)
                if key then GameMainScene.SetPageButtonTouchEvent(eventType,key) end
                if eventType==ccui.TouchEventType.ended then
                    GameSound.PlayEffect(SoundEffect.UI_CLICK,false)
                    for _,nd in pairs(node:getChildren()) do
                        nd:getChildByTag(0):setVisible(tag==nd:getTag())
                    end
                    -- cclog("AddSceneToSubFuncPage  :" .. pageID)
                    return GameMainScene.AddSceneToSubFuncPage(pageID)
                end
            end)
        button:setPressedActionEnabled(true)
        if key then 
            GameMainScene.AddPageButton(key,button,false) 
        end
        return nd_button
    end
    
    node_bottom = page_layer:getChildByTag(2)
    children.nd_bottom=node_bottom
    
    SetEventButton(node_bottom,1,pageZOrder.SUBPAGE_MAINFB,main_buttons.FB)
    SetEventButton(node_bottom,2,pageZOrder.SUBPAGE_ROLE,main_buttons.ROLE)
    SetEventButton(node_bottom,3,pageZOrder.SUBPAGE_SKILLS,main_buttons.SKILL)
    SetEventButton(node_bottom,4,pageZOrder.SUBPAGE_INTENSIFY,main_buttons.INTENSIFY)
    SetEventButton(node_bottom,5,pageZOrder.SUBPAGE_PETS,main_buttons.PETS)
    SetEventButton(node_bottom,6,pageZOrder.SUBPAGE_ACHIEVEMENT,main_buttons.TASK)
    SetEventButton(node_bottom,7,pageZOrder.SUBPAGE_ARENA,main_buttons.ARENA)
    SetEventButton(node_bottom,8,pageZOrder.SUBPAGE_RUINEXPLORERS,main_buttons.EXPLORERS)

    if AchievementScene.CangetReward() then
        local taskBtn = page_layer:getChildByTag(2):getChildByTag(6)
        red_point = cc.Sprite:create("icon_hongdian.png")
        taskBtn:addChild(red_point)
        red_point:setPosition(25,20)
    end

    local nd_top=page_layer:getChildByTag(3)

    local function SetTopButtonEvent(node,tag,pageID)
        local nd_button=node:getChildByTag(tag)
        local button=nd_button:getChildByTag(1)
        button:addTouchEventListener(
            function(sender,eventType)
                if eventType==ccui.TouchEventType.ended then
                    return GameMainScene.AddSceneToSubFuncPage(pageID)
                end
            end)
        button:setPressedActionEnabled(true)
        return button
    end

    local function SetChargeButtonEvent(node,tag,cTag,index)
        local nd_button=node:getChildByTag(tag)
        local button=nd_button:getChildByTag(1)
        button:addTouchEventListener(
            function(sender,eventType)
                if eventType==ccui.TouchEventType.ended then
                    GameSound.PlayEffect(SoundEffect.UI_CLICK,false)
                    return StoreScene.QuickOpenTypeStore(cTag,index)
                end
            end)
        button:setPressedActionEnabled(true)
        return button
    end
    
    --[[for tag=1,7 do
        local button=nd_top:getChildByTag(tag):getChildByTag(1)
        button:setPressedActionEnabled(true)
    end--]]

    children.bt_firstChargePage=SetEventButton(nd_top,1,pageZOrder.SUBPAGE_FIRSTCHARGE)
    SetEventButton(nd_top,3,pageZOrder.SUBPAGE_CHARGE)
    SetEventButton(nd_top,7,pageZOrder.SUBPAGE_STORE)

    SetChargeButtonEvent(nd_top,2,res_store_type.VIP)
    for i,tag in ipairs({4,5,6}) do
        SetChargeButtonEvent(nd_top,tag,res_store_type.EQUIP_SET,i)
    end
    
    local nd_stateBar=page_layer:getChildByTag(4)
    children.ic_player=nd_stateBar:getChildByTag(1):getChildByTag(37)
    children.tx_player_lv=nd_stateBar:getChildByTag(1):getChildByTag(43)
    children.tx_player_vip=nd_stateBar:getChildByTag(2):getChildByTag(1)
    
    children.ic_player:loadTexture(GameDataTransfer.GetIconPath(1))
    
    local nd_currencyBar=nd_stateBar:getChildByTag(3)
    children.tx_player_diamond=nd_currencyBar:getChildByTag(1):getChildByTag(2)
    children.tx_player_coins=nd_currencyBar:getChildByTag(2):getChildByTag(2)
    
    --GameAssistor.SetBlackOutlineNoSpaces{children.tx_player_vip}
    
    ---[[

    local btnGuanyu = nd_stateBar:getChildByName("guanyu")
    CommonAssistor.AddSimpleTouchEvent(btnGuanyu,function()
       GameMainScene.ShowMianze()
    end)

    local btnMore = nd_stateBar:getChildByName("more_game")
    CommonAssistor.AddSimpleTouchEvent(btnMore,function()
        cc.PlatformAdapter:openPay("more","1111","22222","33333",function(p1,p2,p3,p4) end)
    end)

    local bt_soundOn,bt_soundOff=nd_stateBar:getChildByTag(4),nd_stateBar:getChildByTag(5)
    local function SetSoundOn(ON)
        local cUserDefault=cc.UserDefault:getInstance()
        cUserDefault:setBoolForKey("SoundOn",ON)
        GameSound.SetEnable(ON,ON) outputString("SoundOn",ON)
        GameMainScene.RefreshSoundOnButtons()
    end
    CommonAssistor.AddSimpleTouchEvent(bt_soundOn,function()
        SetSoundOn(false)
    end)
    CommonAssistor.AddSimpleTouchEvent(bt_soundOff,function()
        SetSoundOn(true)
    end)
    GameMainScene.RefreshSoundOnButtons=function()
        local musicEnable,effectEnable=GameSound.GetEnable()
        bt_soundOn:setVisible(musicEnable)
        bt_soundOff:setVisible(not musicEnable)
    end
    GameMainScene.RefreshSoundOnButtons()
    ------------------------------------------------------]]
    
    GameMainScene.RefreshPlayerStateBar(true)
    
    local bt_addCoin=nd_currencyBar:getChildByTag(2):getChildByTag(3)
    bt_addCoin:addTouchEventListener(
        function(sender,eventType)
            if eventType==ccui.TouchEventType.ended then 
                GameSound.PlayEffect(SoundEffect.UI_CLICK,false)
                if PINGTEST then return Ping_AddRandomMoney("coin") end
                return StoreScene.QuickOpenTypeStore(res_store_type.COINS)
            end
        end)
    local bt_addGem=nd_currencyBar:getChildByTag(1):getChildByTag(3)
    bt_addGem:addTouchEventListener(
        function(sender,eventType)
            if eventType==ccui.TouchEventType.ended then 
                GameSound.PlayEffect(SoundEffect.UI_CLICK,false)
                if PINGTEST then return Ping_AddRandomMoney("diamond") end
                return GameMainScene.AddSceneToSubFuncPage(pageZOrder.SUBPAGE_CHARGE)
            end
        end)

    -- local btn_jjc = page_layer:getChildByTag(2):getChildByTag(7)
    -- btn_jjc:setVisible(false)
end

function GameMainScene.AddSubPageEventBackground(layerFarm,zOrder,returnEvent)

    local background = GameAssistor.CreateBackMaskLayout(180)
    layerFarm:addChild(background,zOrder or 0)

    local obstructLayout = GameAssistor.CreateBackMaskLayout()
    background:addChild(obstructLayout)
    background.obstructLayout=obstructLayout

    obstructLayout:setAnchorPoint(cc.p(0.5,0.5))
    GameAssistor.SetToSceneCenter(obstructLayout)

    if returnEvent then
        local function RemovePage()
            returnEvent()
        end
        GameMainScene.SetObstructLayoutTouchEvent(obstructLayout,RemovePage)
    end

    return background,obstructLayout
end

function GameMainScene.AddBasicSubPageBackgroundPage()
    local layerFarm = GameAssistor.GetRunningScene()
    local background,obstructLayout = GameMainScene.AddSubPageEventBackground(layerFarm,pageZOrder.PAGE_TOPSUBPAGE)
    background:setVisible(false)
    local function RemovePage()
        layerFarm:removeChild(background)
    end
    GameMainScene.SetObstructLayoutTouchEvent(obstructLayout,RemovePage)
    return background,obstructLayout
end

function GameMainScene.SetObstructLayoutTouchEvent(obstructLayout,callback)
    obstructLayout:addTouchEventListener(
        function(sender,eventType)
            if not obstructLayout.isDone then return end
            if eventType== ccui.TouchEventType.began then
                --GameSound.PlayEffect(SoundEffect.UI_CLICK)
            elseif eventType== ccui.TouchEventType.ended then
                GameMainScene.HidePageScaleToAction(obstructLayout,callback)
            end
        end)
end

function GameMainScene.HidePageScaleToAction(child,callback)
    local ease = cc.EaseBackIn:create(cc.ScaleTo:create(0.3,0))
    local action_hide = cc.Sequence:create(ease,cc.CallFunc:create(callback))
    child:runAction(action_hide)
    child.isDone=false
end

function GameMainScene.PlayPageScaleToAction(child)
    child:setScale(0)
    child.isDone=false
    local function ShowPage()
        child.isDone=true
    end
    local ease = cc.EaseBackOut:create(cc.ScaleTo:create(0.3,1))
    local actionTo = cc.Sequence:create(ease,cc.CallFunc:create(ShowPage))
    child:runAction(actionTo)
end

function GameMainScene.HideAllSceneFromSubFuncPage()
    local layerFarm = GameMainScene.GetSubFuncPage()
    children.bt_exitgame:setVisible(true)
    for key, subPage in pairs(layerFarm:getChildren()) do
        subPage:setVisible(false)
    end
    GameMainScene.nowShowingPageID=nil

    local nd_bottom=children.nd_bottom
    if nd_bottom then
        for _,nd in pairs(nd_bottom:getChildren()) do
            nd:getChildByTag(0):setVisible(false)
        end
    end
end

function GameMainScene.GetSceneToSubFuncPage(pageID)
    local subSceneList=GameMainScene.subSceneList
    if not subSceneList then
        subSceneList={}
        subSceneList[pageZOrder.SUBPAGE_MAINFB]=StageScene
        subSceneList[pageZOrder.SUBPAGE_ROLE]=RoleScene
        subSceneList[pageZOrder.SUBPAGE_SKILLS]=SkillScene
        subSceneList[pageZOrder.SUBPAGE_PETS]=PetScene
        subSceneList[pageZOrder.SUBPAGE_INTENSIFY]=IntensifyScene
        subSceneList[pageZOrder.SUBPAGE_ACHIEVEMENT]=AchievementScene
        subSceneList[pageZOrder.SUBPAGE_ARENA]=ArenaScene
        subSceneList[pageZOrder.SUBPAGE_RUINEXPLORERS]=RuinExplorers
        
        subSceneList[pageZOrder.SUBPAGE_FIRSTCHARGE]=FirstCharge
        subSceneList[pageZOrder.SUBPAGE_STORE]=StoreScene
        subSceneList[pageZOrder.SUBPAGE_CHARGE]=ChargeScene
        
        GameMainScene.subSceneList=subSceneList
    end
    return subSceneList[pageID]
end

function GameMainScene.CheckSubFuncPageShowing(pageID)
    if children.maskLayout:isVisible() then return false end
    local layerFarm = GameMainScene.GetSubFuncPage()
    if not layerFarm then return false end
    local subPage = layerFarm:getChildByTag(pageID)
    if subPage then return subPage:isVisible() end
    return false
end

function GameMainScene.AddSceneToSubFuncPage(pageID)

    children.bt_exitgame:setVisible(false)
    if not pageID then return end
    -- GameMainScene.lastPageID
    --if GameMainScene.CheckSubFuncPageShowing(pageID) then return end

    local layerFarm = GameMainScene.GetSubFuncPage()

    local subPage = layerFarm:getChildByTag(pageID)

    if subPage then
        return GameMainScene.ShowSceneFromSubFuncPage(subPage)
    end

    local nextScene=GameMainScene.GetSceneToSubFuncPage(pageID)

    if nextScene then
        subPage = nextScene.create()
    end

    if subPage then
        layerFarm:addChild(subPage,pageID,pageID)
        subPage.pageID=pageID
        return GameMainScene.ShowSceneFromSubFuncPage(subPage)
    end
end

function GameMainScene.ShowSceneFromSubFuncPage(subPage)

    local lastPageID=GameMainScene.nowShowingPageID or 0
    GameMainScene.nowShowingPageID=subPage.pageID
    
    if subPage.Refresh then subPage.Refresh() end
    
    subPage:setVisible(true)

    if GameMainScene.nowShowingPageID==lastPageID then return end

    local time,width=0.4,1280
    
    local target_x=(subPage.pageID>lastPageID) and width or -width
    if subPage.getMainLayer then
        local layer=subPage.getMainLayer()
        layer:setPositionX(target_x)
        children.maskLayout:setVisible(true)
        layer:runAction(cc.Sequence:create(cc.MoveTo:create(time,cc.p(0,0)),cc.CallFunc:create(function()
            children.maskLayout:setVisible(false)
        end)))
    end

    local layerFarm = GameMainScene.GetSubFuncPage()
    local lastPage = layerFarm:getChildByTag(lastPageID)
    if lastPage and lastPage.getMainLayer then 
        local layer=lastPage.getMainLayer()
        local action_hide = cc.Sequence:create(cc.MoveTo:create(time,cc.p(-target_x,0)),cc.CallFunc:create(function()
            lastPage:setVisible(false)
        end))
        layer:runAction(action_hide)
    end
end

function GameMainScene.ReturnToSubPage(pageID)
    
    if GameAssistor.SceneIsRunning("GameMainScene") then
        return GameMainScene.AddSceneToSubFuncPage(pageID)
    end
    
    GameMainScene.SetEnterAutoFunc(function()
        GameMainScene.AddSceneToSubFuncPage(pageID)
    end)
    
    LoadingScene.setNextScene(GameMainScene)    
    return NowSceneReplaceChecker(LoadingScene)
end

function GameMainScene.AddGoToConfirmBox(stringID)
    local bt_ok=CommonAssistor.AddConfirmBox(GameDataTransfer.GetSysStrings(stringID),function()
        if str_system.STRING_TIPS_COIN_NO_ENOUGH==stringID then
            return StoreScene.QuickOpenTypeStore(res_store_type.COINS)
        elseif str_system.STRING_TIPS_DIAMOND_NO_ENOUGH==stringID then
            return GameMainScene.AddSceneToSubFuncPage(pageZOrder.SUBPAGE_CHARGE)
        else
            return StoreScene.QuickOpenTypeStore(res_store_type.OTHERS)
        end
    end)
    bt_ok:setTitleText(GameDataTransfer.GetSysStrings(str_system.STRING_TIPS_GOTO))
end








