IntensifyScene = {}

IntensifyScene = class("IntensifyScene",function()
    return cc.Scene:create()
end)

local children={}
local select_role_id=1
local select_position_id=1

function IntensifyScene:sceneEnter()
    outputString("IntensifyScene:sceneEnter()")
end

function IntensifyScene:sceneExit()
    IntensifyScene.RemoveAllChildren(self)
end

function IntensifyScene.RemoveAllChildren(scene)
    table.clear(children)
    GameAssistor.RemoveUnusedSceneResource(scene)
end

function IntensifyScene.create()

    local scene = IntensifyScene.new()

    Sence_Init(scene)

    local layerFarm = cc.Layer:create()

    scene.getMainLayer=function()
        return layerFarm
    end
    children.mainLayerFarm = layerFarm

    local background = GameAssistor.CreateBackMaskLayout(180)
    layerFarm:addChild(background)

    scene.AddMainChildren()

    scene:addChild(layerFarm)

    return scene
end

function IntensifyScene.Refresh()
    IntensifyScene.ChooseRoleButton(select_role_id)
end

function IntensifyScene.RefreshPage()
    local dataList=IntensifyScene.RefreshPositionLvs()
    --table.print(dataList,"IntensifyScene.RefreshPage")

    for key,data in ipairs(dataList) do
        local button=children.bt_equipments[key]
        if data.ID>0 then
            button.setLevel(data.lv)
        end
    end

    IntensifyScene.ChoosePositionButton(select_position_id)
end

function IntensifyScene.RefreshPositionLvs(index)
    local dataList=GameRecorder.GetRoleEquipments(select_role_id)
    children.positionLvs={}
    for key,data in ipairs(dataList) do
        children.positionLvs[key]=data.lv
    end
    if index then return dataList[index] end
    return dataList
end

function IntensifyScene.AddSuccessEffectNode(page_layer)

    local main_node = cc.Node:create()
    page_layer:addChild(main_node)
    
    local function HidePage() 
        main_node:setVisible(false) 
    end
    HidePage()
    
    --[[
    local effect,timeline=CommonAssistor.AddEffectLayer("QH-1.csb")
    page_layer:addChild(effect)--]]
    
    ---[[
    local effect=cc.CSLoader:createNode("QH-1.csb")
    local filenames={"e-QH-jt.csb","e-QH.csb","e-QH-cz.csb"}
    for i=1,3 do
        local node=effect:getChildByTag(i)
        node:removeFromParent() main_node:addChild(node)
        node.playEff=function()
            CommonAssistor.PlayTimelineEffect(filenames[i],node)
        end
    end--]]
    
    IntensifyScene.PlaySuccessEffects=function()
        local duration=1.3
        main_node:setVisible(true) main_node:stopAllActions()
        for key,node in pairs(main_node:getChildren()) do
            node.playEff()
        end
        local actionTo = cc.Sequence:create(cc.DelayTime:create(duration),cc.CallFunc:create(HidePage))
        main_node:runAction(actionTo)
        GameMainScene.SetProtectMaskTimer(duration)
    end
end

local function SetLevel(tx_qty,lv)
    tx_qty:setPosition(105,83) 
    tx_qty:setString("+"..(lv or 0))
    tx_qty:setVisible(true)
end

function IntensifyScene.AddMainChildren()

    local layerFarm = children.mainLayerFarm

    local page_layer = cc.CSLoader:createNode("QH.csb")
    GameAssistor.SetToSceneCenter(page_layer)
    layerFarm:addChild(page_layer)
    
    IntensifyScene.AddSuccessEffectNode(page_layer)

    local nd_main=page_layer:getChildByTag(1)
    
    IntensifyScene.SetIntensifyIcon=function(ID,lv)
        local node=nd_main:getChildByTag(1)
        RoleScene.SetEquipItem(node:getChildByTag(1),ID)
        RoleScene.SetEquipItem(node:getChildByTag(3),ID)
    end
    
    local nd_upgrade_main=nd_main:getChildByTag(3)
    children.nd_upgrade_main=nd_upgrade_main

    local nd_attribute1=nd_upgrade_main:getChildByTag(1)
    local nd_attribute2=nd_upgrade_main:getChildByTag(2)
    local nd_cost=nd_upgrade_main:getChildByTag(3)

    local function setValue(node,value)
        node:getChildByTag(3):setString("+"..value)
    end
    local function setValue2(node,value1,value2)
        setValue(node,value2)
        node:getChildByTag(4):setVisible(value2>value1)
    end
    
    IntensifyScene.SetIntensifyMainAttribute=function(ID)
        if ID <= 0 then
            return 
        end
        local iData=GameDataTransfer.GetItemData(ID)
        local value,skip=IntensifyScene.GetMainAttribute(iData)
        local strType,strValue = GameDataTransfer.GetSysString(str_system.STRING_ATTRIBUTE+skip),value
        local node=nd_attribute1:getChildByTag(1)
        node:getChildByTag(2):setString(strType) 
        setValue(node,value)
        local node=nd_attribute2:getChildByTag(1)
        node:getChildByTag(2):setString(strType)
        setValue(node,value)
    end

    IntensifyScene.SetIntensifyAttributes=function(equipID,equipLv)
        local node=nd_main:getChildByTag(1)
        local lvLeft  = node:getChildByTag(1):getChildByTag(1):getChildByTag(3)
        local lvRight = node:getChildByTag(3):getChildByTag(1):getChildByTag(3)
        if equipID == 0 then
            lvLeft:setVisible(false)
            lvRight:setVisible(false)
            do return end
        else
            lvLeft:setVisible(true)
            lvRight:setVisible(true)
        end
        SetLevel(node:getChildByTag(1):getChildByTag(1):getChildByTag(3),equipLv)
        SetLevel(node:getChildByTag(3):getChildByTag(1):getChildByTag(3),equipLv+1)
        
        local lvData1=GameDataTransfer.GetItemLevelData(equipID,equipLv)
        local power,defence,blood=0,0,0
        if lvData1 then
            power,defence,blood=lvData1.power,lvData1.defence,lvData1.blood
        end
        --outputString("-----Now power:"..power.." defence:"..defence.." blood:"..blood)
        setValue(nd_attribute1:getChildByTag(2),power)
        setValue(nd_attribute1:getChildByTag(3),defence)
        setValue(nd_attribute1:getChildByTag(4),blood)
        
        local canUpgrade=true
        local lvData2=GameDataTransfer.GetItemLevelData(equipID,equipLv+1)
        if lvData2 then
            --outputString("----Next power:"..lvData2.power.." defence:"..lvData2.defence.." blood:"..lvData2.blood)
            setValue2(nd_attribute2:getChildByTag(2),power,lvData2.power)
            setValue2(nd_attribute2:getChildByTag(3),defence,lvData2.defence)
            setValue2(nd_attribute2:getChildByTag(4),blood,lvData2.blood)
            
            local bt_upgrade=nd_cost:getChildByTag(2)
            CommonAssistor.AddSimpleTouchEvent(bt_upgrade,function()
                return IntensifyScene.UpgradePosition()
            end)
            children.bt_upgrade=bt_upgrade
            local tx_cost=bt_upgrade:getChildByTag(1)
            --outputString("Cost:"..lvData2.Cost)
            tx_cost:setString(lvData2.Cost)
        else
            canUpgrade=false outputString("Max")
        end
        nd_attribute2:setVisible(canUpgrade)
        nd_cost:setVisible(canUpgrade)
    end

    local nd_equipments=page_layer:getChildByTag(6)
    children.bt_equipments={}
    for i=1,3 do
        local button=nd_equipments:getChildByTag(i):getChildByTag(0)
        RoleScene.SetStorageItemEvent(button)
        local tx_qty=button:getChildByTag(3)
        button.setLevel=function(lv)
            SetLevel(tx_qty,lv)
        end
        button.setItem2=function(ID,lv)
            local iData=GameDataTransfer.GetItemData(ID)
            button.setItem(iData)
            button.setLevel(lv)
        end
        button:addTouchEventListener(
            function(sender,eventType)
                if eventType== ccui.TouchEventType.ended then
                    return IntensifyScene.ChoosePositionButton(i)
                end
            end)
        children.bt_equipments[i]=button
    end
    
    local nd_button=page_layer:getChildByTag(7)
    children.nd_role_buttons=RoleScene.AddChooseRoleButtons(nd_button,IntensifyScene.ChooseRoleButton)
end

function IntensifyScene.GetMainAttribute(data)
    local skip=1
    if data.power>0 then return data.power,skip end skip=skip+1
    if data.defence>0 then return data.defence,skip end skip=skip+1
    if data.blood>0 then return data.blood,skip end skip=skip+1
    if data.crit>0 then return data.crit,skip end skip=0
    return 0,skip
end

function IntensifyScene.ChooseRoleButton(roleID)
    
    select_role_id=roleID
    children.nd_role_buttons.refreshStates(roleID)
    
    local dataList=IntensifyScene.RefreshPositionLvs()
    --table.print(dataList,"IntensifyScene.ChooseRoleButton")

    for key,data in ipairs(dataList) do
        local button=children.bt_equipments[key]
        local equipID,equipLv=data.ID,data.lv
        if equipID>0 then
            local gData=GameDataTransfer.GetItemData(equipID)
            --outputString("name:"..GameDataTransfer.GetGameString(gData.Equip_Name).." lv:"..equipLv)
            button.setItem2(equipID,equipLv)
        else 
            --outputString("position:"..key.." no equipment lv:"..equipLv)
            button.setEmpty()
        end
    end
    
    return IntensifyScene.ChoosePositionButton(1)
end

function IntensifyScene.ChoosePositionButton(posid)
    
    select_position_id=posid

    for i=1,3 do
        local button=children.bt_equipments[i]
        GameAssistor.SetButtonBright(button,i~=posid)
    end    

    local dataList=IntensifyScene.RefreshPositionLvs()
    --table.print(dataList,"IntensifyScene.ChoosePositionButton")

    local equipID,equipLv=dataList[posid].ID,dataList[posid].lv
    -- cclog("equipID:" .. equipID .. " equipLv:" .. equipLv)
    IntensifyScene.SetIntensifyIcon(equipID)
    
    IntensifyScene.SetIntensifyMainAttribute(equipID)        
    IntensifyScene.SetIntensifyAttributes(equipID,equipLv)
    children.nd_upgrade_main:setVisible(equipID>0)
end

function IntensifyScene.UpgradePosition()
    
    local dataList=IntensifyScene.RefreshPositionLvs(select_position_id)
    local equipID,equipLv=dataList.ID,dataList.lv+1
    if equipID==0 then return end
    
    local lvData=GameDataTransfer.GetItemLevelData(equipID,equipLv)
    if not lvData then return outputString("Max") end
    
    if equipLv>GameRecorder.GetBasicData("level") then
        return CommonAssistor.AddWarningBox(GameDataTransfer.GetSysString(str_system.STRING_TIPS_ERROR_UPGRADELV))
    end
    
    --outputString("UpgradePosition Cost:"..lvData.Cost)
    local cost=lvData.Cost
    if GameRecorder.GetBasicData("coin")<cost then 
        return GameMainScene.AddGoToConfirmBox(str_system.STRING_TIPS_COIN_NO_ENOUGH)
    end
    GameRecorder.AddQuantity("coin",-cost)
    
    IntensifyScene.PlaySuccessEffects()
    children.positionLvs[select_position_id]=equipLv    
    local role=GameRecorder.GetRole(select_role_id)
    for i=1,3 do
        role.equip_lvs[i]=children.positionLvs[i]
    end
    GameRecorder.SaveBasicData()
    
    GameMainScene.RefreshPlayerStateBar()

    CombatPowerUnit.CheckRoleCombatPowers()

    return IntensifyScene.RefreshPage()
end



------Toutorial-----------------
function IntensifyScene.GetUpgradeButton()
    return children.bt_upgrade
end

function IntensifyScene.GetPositionValidity()
    return CommonAssistor.GetPageVisible(children.nd_upgrade_main)
end
