SkillScene = {}

SkillScene = class("SkillScene",function()
    return cc.Scene:create()
end)

local children={}
local select_role_id=1
local select_skill_pos=1
local select_skill_id=1

function SkillScene:sceneEnter()
    outputString("SkillScene:sceneEnter()")
end

function SkillScene:sceneExit()
    SkillScene.RemoveAllChildren(self)
end

function SkillScene.RemoveAllChildren(scene)
    table.clear(children)
    GameAssistor.RemoveUnusedSceneResource(scene)
end

function SkillScene.create()

    local scene = SkillScene.new()

    Sence_Init(scene)

    local layerFarm = cc.Layer:create()

    scene.getMainLayer=function()
        return layerFarm
    end
    children.mainLayerFarm = layerFarm

    local background = GameAssistor.CreateBackMaskLayout(180)
    layerFarm:addChild(background)

    scene.AddMainChildren()

    scene:addChild(layerFarm)

    return scene
end

function SkillScene.Refresh()
    SkillScene.ChooseRoleButton(select_role_id,true)
end

function SkillScene.RefreshSkillLvs()
    local skillList=GameRecorder.GetRoleSkills(select_role_id)
    children.skillLvs={}
    for key,data in ipairs(skillList) do
        children.skillLvs[data.ID]=data.lv
    end
    return skillList
end

function SkillScene.AddMainChildren()

    local layerFarm = children.mainLayerFarm

    local page_layer = cc.CSLoader:createNode("WG.csb")
    GameAssistor.SetToSceneCenter(page_layer)
    layerFarm:addChild(page_layer)

    children.nd_skill_units=page_layer:getChildByTag(1)

    local nd_info=page_layer:getChildByTag(4)
    children.tx_skill_name=nd_info:getChildByTag(0):getChildByTag(1)
    children.tx_skill_description=nd_info:getChildByTag(1):getChildByTag(1)

    local nd_info=page_layer:getChildByTag(5)
    children.tx_skill_effect1=nd_info:getChildByTag(1):getChildByTag(1)
    children.tx_skill_effect2=nd_info:getChildByTag(2):getChildByTag(1)
    
    local nd_skills=page_layer:getChildByTag(6)
    children.bt_skills={nd_skills:getChildByTag(1),nd_skills:getChildByTag(2),nd_skills:getChildByTag(3)}

    local nd_button=page_layer:getChildByTag(7)
    children.nd_role_buttons=RoleScene.AddChooseRoleButtons(nd_button,SkillScene.ChooseRoleButton)
end

function SkillScene.GetCircleClipping()

    local mask = cc.Sprite:create(GameDataTransfer.GetIconPath("icon-z"))
    local clippingNode = cc.ClippingNode:create(mask)
    clippingNode:setAlphaThreshold(0)
    local drawIcon = cc.Sprite:create()
    clippingNode:addChild(drawIcon)

    return drawIcon,clippingNode
end

function SkillScene.SetCircleClipping(nd_icon,scale)

    local drawIcon,clippingNode = SkillScene.GetCircleClipping()

    local windowSize=nd_icon:getCustomSize()
    clippingNode:setPosition(cc.p(windowSize.width/2,windowSize.height/2))
    nd_icon:addChild(clippingNode)
    nd_icon:setOpacity(0)
    if scale then clippingNode:setScale(scale) end

    return drawIcon,clippingNode
end

function SkillScene.ChooseRoleButton(roleID,isRefresh)
    
    select_role_id=roleID
    children.nd_role_buttons.refreshStates(roleID)
    
    local skillList=SkillScene.RefreshSkillLvs()
    --table.print(skillList,"SkillScene.ChooseRoleButton skillList:")
    
    for key,data in ipairs(skillList) do
        local gData=GameDataTransfer.GetActiveSkillData(data.ID,data.lv)
        --outputString("name:"..gData.skill_name,"lv:"..data.lv)
        local button=children.bt_skills[key]:getChildByTag(1)
        button:getChildByTag(2):setString(GameDataTransfer.GetGameString(gData.skill_name))
        if not button.skillIcon then
            button.skillIcon=SkillScene.SetCircleClipping(button:getChildByTag(1))
            GameAssistor.SetBlackOutlineNoSpace(button:getChildByTag(3))
        end
        button.skillIcon:setTexture(GameDataTransfer.GetIconPath(gData.Icon,true))
        button:getChildByTag(3):setString("等级."..data.lv)
        button:addTouchEventListener(
            function(sender,eventType)
                if eventType== ccui.TouchEventType.ended then
                    return SkillScene.ChooseSkillButton(data.ID,key)
                end
            end)
    end
    
    local position = isRefresh and select_skill_pos or 1
    SkillScene.ChooseSkillButton(skillList[position].ID,position)
end

function SkillScene.ChooseSkillButton(skillID,position)
    
    select_skill_pos=position
    select_skill_id=skillID
    
    for i=1,3 do
        local button=children.bt_skills[i]:getChildByTag(1)
        GameAssistor.SetButtonBright(button,i~=position)
    end    
    outputString("select_skill_id:"..select_skill_id)

    local level=children.skillLvs[skillID] or 0
    outputString("select level:"..level)
    
    local gData=GameDataTransfer.GetActiveSkillData(skillID,level)
    --outputString("skill_name:"..GameDataTransfer.GetGameString(gData.skill_name))
    --outputString("skill_Description:"..GameDataTransfer.GetGameString(gData.skill_Description))
    
    children.tx_skill_name:setString(GameDataTransfer.GetGameString(gData.skill_name))
    children.tx_skill_description:setString(GameDataTransfer.GetGameString(gData.skill_Description))

    local attackRate,attackAdd=0,0
    local activeSkillValue,skillValue=GameDataTransfer.GetActiveSkillValue(skillID,level)
    if activeSkillValue then skillValue=GameDataTransfer.GetSkillValue(activeSkillValue.skillrefid,level) end
    if skillValue then attackRate,attackAdd=skillValue.Attack_rate,skillValue.Attack2 end
    --[[local info = string.gsub(GameDataTransfer.GetSysString(str_system.STRING_TIPS_SKILLINFO1),"?",attackRate)
        .."--"..string.gsub(GameDataTransfer.GetSysString(str_system.STRING_TIPS_SKILLINFO2),"?",attackAdd)
    outputString(info)--]]
    children.tx_skill_effect1:setString((string.gsub(GameDataTransfer.GetSysString(str_system.STRING_TIPS_SKILLINFO1),"?",attackRate)))
    children.tx_skill_effect2:setString((string.gsub(GameDataTransfer.GetSysString(str_system.STRING_TIPS_SKILLINFO2),"?",attackAdd)))
    
    SkillScene.ReadyToUpgradeSkill(skillID,level+1)
end

function SkillScene.ReadyToUpgradeSkill(skillID,level)
    
    local nd_units=children.nd_skill_units
    local nd_balls=nd_units:getChildByTag(3)
    local nd_lines=nd_units:getChildByTag(2)
    
    local index=level%10 if index==0 then index=10 end
    local start=level-index
    for i=1,10 do
        local bt_ball=nd_balls:getChildByTag(i)
        bt_ball:getChildByTag(1):setVisible(false)
        GameAssistor.SetButtonBright(bt_ball,i>=index)
        bt_ball:setTouchEnabled(false)
        bt_ball:setTitleText(start+i)
        local nd_line=nd_lines:getChildByTag(i-1)
        if nd_line then
            nd_line:setVisible(i<index)
        end
    end
    
    local skillValue2=GameDataTransfer.GetActiveSkillValue(skillID,level)
    local isUpdateAble=skillValue2~=nil
    if isUpdateAble then
        --outputString("cost:"..skillValue2.GoldCost,"Unlock_Level:"..skillValue2.Unlock_Level)
        local bt_ball=nd_balls:getChildByTag(index)
        bt_ball:addTouchEventListener(
            function(sender,eventType)
                if eventType== ccui.TouchEventType.ended then
                    SkillScene.AddFireEffect(bt_ball)
                    return SkillScene.UpgradeSkill(skillID)
                end
            end)
        bt_ball:setTouchEnabled(true)
        local nd_upgrade=bt_ball:getChildByTag(1)
        nd_upgrade:getChildByTag(2):setString(skillValue2.GoldCost)
        local skillValue=GameDataTransfer.GetSkillValue(skillValue2.skillrefid,level)
        local attribute=GameDataTransfer.GetSysString(str_system.STRING_TIPS_DAMAGE).."+"..skillValue.Attack2 
        nd_upgrade:getChildByTag(3):setString(attribute)
        nd_upgrade:setVisible(true)
    end
end

function SkillScene.AddFireEffect(bt_ball)
    local effect,timeline = CommonAssistor.AddEffectLayer("e-ND.csb",false,35,35)
    bt_ball:addChild(effect)
    local function Remove()
        bt_ball:removeChild(effect)
    end
    local actionTo = cc.Sequence:create(cc.DelayTime:create(1),cc.CallFunc:create(Remove))
    bt_ball:runAction(actionTo)
end

function SkillScene.ChangeARoundUpgradeUnits()
    
    local nd_units=children.nd_skill_units

    local nd_balls=nd_units:getChildByTag(3)
    local nd_lines=nd_units:getChildByTag(2)
    for i=1,10 do
        local bt_ball=nd_balls:getChildByTag(i)
        bt_ball:getChildByTag(1):setVisible(false)
        GameAssistor.SetButtonBright(bt_ball,false)
        local nd_line=nd_lines:getChildByTag(i-1)
        if nd_line then nd_line:setVisible(true) end
    end

    local orbit1 = cc.OrbitCamera:create(0.5,1, 0, 0, 80, 0, 0)
    local orbit2 = cc.OrbitCamera:create(0.5,1, 0, 0, 1080, 0, 0)
    local function Finish()
        return SkillScene.Refresh()
    end
    local actionTo = cc.Sequence:create(orbit1,orbit2,cc.CallFunc:create(Finish))
    nd_units:runAction(actionTo)
end

function SkillScene.UpgradeSkill(skillID)
    
    skillID = skillID or select_skill_id
    local nowLevel=children.skillLvs[skillID] or 0
    local level=nowLevel+1
    local skillValue2=GameDataTransfer.GetActiveSkillValue(skillID,level)
    if not skillValue2 then return outputString("Max!") end

    if level>GameRecorder.GetBasicData("level") then
        return CommonAssistor.AddWarningBox(GameDataTransfer.GetSysString(str_system.STRING_TIPS_ERROR_UPGRADELV))
    end
    
    if GameRecorder.GetBasicData("level")<skillValue2.Unlock_Level then 
        return CommonAssistor.AddWarningBox(GameDataTransfer.GetSysString(str_system.STRING_TIPS_ERROR_ROLELV))
    end
    
    local cost=skillValue2.GoldCost
    if GameRecorder.GetBasicData("coin")<cost then 
        return GameMainScene.AddGoToConfirmBox(str_system.STRING_TIPS_COIN_NO_ENOUGH)
    end
    GameRecorder.AddQuantity("coin",-cost)
    
    children.skillLvs[skillID]=level
    local role=GameRecorder.GetRole(select_role_id)
    local skillList=GameRecorder.GetRoleSkills(select_role_id)
    for i=1,3 do
        role.skill_lvs[i]=children.skillLvs[skillList[i].ID] or 0
    end
    GameRecorder.SaveBasicData()
    GameMainScene.RefreshPlayerStateBar()
    GameRecorder.SaveBasicData()

    CombatPowerUnit.CheckRoleCombatPowers()
    
    if level%10==0 then
        return SkillScene.ChangeARoundUpgradeUnits()
    end
    
    return SkillScene.Refresh()
end



------Toutorial-----------------
function SkillScene.GetCanUpgradeButton()
    local nd_units=children.nd_skill_units
    for key, button in pairs(nd_units:getChildByTag(3):getChildren()) do
        if button:getChildByTag(1):isVisible() then return button end
    end
end
