StageScene = {}

StageScene = class("StageScene",function()
    return cc.Scene:create()
end)

local children={}
local select_fb_type=1
local select_fb_chapter
local select_fb_id=1

function StageScene:sceneEnter()
    outputString("StageScene:sceneEnter()")
end

function StageScene:sceneExit()
    StageScene.RemoveAllChildren(self)
end

function StageScene.RemoveAllChildren(scene)
    table.clear(children)
    GameAssistor.RemoveUnusedSceneResource(scene)
end

function StageScene.SetNodeTouchScale(sender,eventType)
    if eventType== ccui.TouchEventType.began then
        sender:setScale(0.95)
    elseif eventType== ccui.TouchEventType.ended or eventType== ccui.TouchEventType.canceled then
        sender:setScale(1)
    end
end

local function SetBlueOutline(text,space)
    text:enableOutline(cc.c4b(0,99,162,255),space or 1)
end
local function SetButtonBlueOutline(button,space)
    return SetBlueOutline(button:getTitleRenderer(),space)
end

function StageScene.create()

    local scene = StageScene.new()

    Sence_Init(scene)

    local layerFarm = cc.Layer:create()

    scene.getMainLayer=function()
        return layerFarm
    end
    children.mainLayerFarm = layerFarm

    local background = GameAssistor.CreateBackMaskLayout(180)
    layerFarm:addChild(background)

    scene.AddMainChildren()

    scene:addChild(layerFarm)

    return scene
end

function StageScene.Refresh()
end

function StageScene.RefreshChapterList(difficulty,isAction)

    select_fb_type=difficulty
    
    local nd_section=children.nd_section
    if nd_section then nd_section:setVisible(false) end
    
    local nd_button=children.nd_typeButtons
    if not nd_button then return end
    for type=1,3 do
        local button=nd_button:getChildByTag(type)
        GameAssistor.SetButtonBright(button,type~=difficulty)
        button:setTouchEnabled(StageScene.GetDifficultyListIsUnlock(type))
    end
    
    local gameChapterNames={str_system.STRING_GAME_CHAPTERNAME1,str_system.STRING_GAME_CHAPTERNAME2,str_system.STRING_GAME_CHAPTERNAME3}
    
    local sc_chapter=children.sc_chapter
    sc_chapter:removeAllChildren()
    for key,var in ipairs(GameDataTransfer.GetGameChapter(difficulty)) do
        local page_layer = cc.CSLoader:createNode("FB-1.csb")
        local button = page_layer:getChildByTag(1)
        button:addTouchEventListener(
            function(sender,eventType)
                StageScene.SetNodeTouchScale(sender,eventType)
                if eventType== ccui.TouchEventType.ended then
                    if StageScene.GetChapterIsUnlock(difficulty,key) then
                        return StageScene.ChooseChapter(key,true)
                    end
                    return outputString("The Chapter is Lock!")
                end
            end)
        button:removeFromParent()

        
        local tx_chapter=button:getChildByTag(1)
        SetBlueOutline(tx_chapter,2)
        tx_chapter:setString(GameDataTransfer.GetChineseNumber(key))
        local tx_name=button:getChildByTag(2)
        GameAssistor.SetBlackOutlineNoSpace(tx_name)
        tx_name:setString(GameDataTransfer.GetGameString(gameChapterNames[difficulty]+key))
        
        local nd_perfect=button:getChildByTag(3)
        button.check_perfect=function()
            local color=StageScene.GetChapterIsUnlock(difficulty,key) and res_color.FULL or res_color.BLACK2
            button:setColor(color)
            nd_perfect:setVisible(StageScene.GetChapterIsPerfact(difficulty,key))
        end
        button.check_perfect()
        
        sc_chapter:addChild(button)
    end
    
    if isAction then GameMainScene.SetProtectMaskTimer(1.8) end

    local w = 204
    local innerWidth = math.max(sc_chapter:getContentSize().width,math.floor(#sc_chapter:getChildren())*w+80)
    local innerHeight = sc_chapter:getContentSize().height
    sc_chapter:setInnerContainerSize(cc.size(innerWidth,innerHeight))
    local x,y = 130,210
    for key, child in pairs(sc_chapter:getChildren()) do
        local tx=x+(key-1)*w
        if isAction then
            child:setPosition(1300,y)
            local actionTo = cc.MoveTo:create(0.2,cc.p(tx,y))
            local actionTo = cc.Sequence:create(cc.DelayTime:create(key*0.2),cc.EaseBackOut:create(actionTo))
            child:runAction(actionTo)
        else
            child:setPosition(tx,y)
        end
    end
    sc_chapter:jumpToLeft()

    if select_fb_chapter and not isAction then
        if StageScene.GetChapterIsUnlock(difficulty,select_fb_chapter) then
            return StageScene.ChooseChapter(select_fb_chapter)
        end
    end
end

function StageScene.RefreshChapterPerfact()
    for key, button in pairs(children.sc_chapter:getChildren()) do
        button.check_perfect()
    end
end

function StageScene.AddMainChildren()

    local layerFarm = children.mainLayerFarm

    local page_layer = cc.CSLoader:createNode("FB.csb")
    layerFarm:addChild(page_layer)
    
    --FBChapter
    local nd_chapter=page_layer:getChildByTag(1)
    children.sc_chapter=nd_chapter:getChildByTag(1)

    local nd_button=page_layer:getChildByTag(3)
    children.nd_typeButtons=nd_button
    for type=1,3 do
        local button=nd_button:getChildByTag(type)
        button:addTouchEventListener(
            function(sender,eventType)
                if eventType==ccui.TouchEventType.ended then
                    if StageScene.GetDifficultyListIsUnlock(type) then
                        return StageScene.RefreshChapterList(type,true)
                    end
                    return outputString("The Type FB is Lock!")
                end
            end)
    end
    
    local nd_section=page_layer:getChildByTag(2)
    children.nd_section=nd_section
    local nd_section2=nd_section:getChildByTag(2)
    children.sc_section=nd_section2:getChildByTag(1)
    
    nd_section:addTouchEventListener(
        function(sender,eventType)
            if eventType==ccui.TouchEventType.ended then
                return outputString("nd_section!")
            end
        end)

    return StageScene.RefreshChapterList(select_fb_type)
end

function StageScene.GetChapterIsPerfact(type,chapter)
    local chapterList=GameDataTransfer.GetGameChapterSections(type,chapter)
    for key,var in ipairs(chapterList) do
        if GameRecorder.GetFBStar(var.fbid)<3 then return false end
    end
    return true
end

function StageScene.GetChapterIsUnlock(type,chapter)
    local chapterList=GameDataTransfer.GetGameChapterSections(type,chapter)
    if chapterList[1] then
        return GameRecorder.GetFBIsUnlock(chapterList[1].fbid)
    end
    return false
end

function StageScene.GetDifficultyListIsUnlock(type)
    local chapterList=GameDataTransfer.GetGameChapter(type)
    if chapterList[1] then
        return StageScene.GetChapterIsUnlock(type,1)
    end
    return false
end

function StageScene.ChooseChapter(chapter,isMove)

    select_fb_chapter=chapter

    local sc_chapter=children.sc_section
    sc_chapter:removeAllChildren()
    
    local fbList,starCount={},0
    for key,var in ipairs(GameDataTransfer.GetGameChapterSections(select_fb_type,chapter)) do
        if GameRecorder.GetFBIsUnlock(var.fbid) then
            var.index=key
            table.insert(fbList,1,var) starCount=starCount+GameRecorder.GetFBStar(var.fbid)
        end
    end
    
    if #fbList==0 then return end

    local nd_section=children.nd_section
    nd_section:setVisible(true)
    local nd_section2=nd_section:getChildByTag(2)

    local tx_stars=nd_section2:getChildByTag(2):getChildByTag(3)
    tx_stars:setString("x"..starCount)
    local nd_starAwards=nd_section2:getChildByTag(3)
    local lb_stars=nd_starAwards:getChildByTag(1)
    lb_stars:setPercent(100*starCount/30)
    
    children.nd_starAwards=nd_starAwards
    for i=1,3 do
        local bt_getSAward=nd_starAwards:getChildByTag(i+1)
        bt_getSAward.Refresh=function()
            local chapterAwards=GameDataTransfer.GetChapterAwards(select_fb_type,chapter)
            local starList=GameRecorder.GetFBChapterStars(chapterAwards.ID)
            local canGet,isGot=starCount>=i*10,starList[i] or false
            GameAssistor.SetButtonBright(bt_getSAward,not isGot)
            bt_getSAward:getChildByTag(2):setVisible(canGet and not isGot)
            bt_getSAward.canGet=canGet
        end
        bt_getSAward.Refresh()
        bt_getSAward:addTouchEventListener(
            function(sender,eventType)
                if eventType==ccui.TouchEventType.ended then
                    if bt_getSAward.canGet then
                        StageScene.GetChapterAward(select_fb_type,chapter,i)
                        return bt_getSAward.Refresh()
                    end
                end
            end)
        GameAssistor.SetBlackOutlineNoSpace(bt_getSAward:getChildByTag(1))
    end
    
    local filename="stage/chapter"..chapter
    cc.SpriteFrameCache:getInstance():addSpriteFrames(filename..".plist",filename..".png")
    
    local function Finish()
        for key,var in ipairs(fbList) do
            local page_layer = cc.CSLoader:createNode("FB-2.csb")
            
            local button = page_layer:getChildByTag(1)
            button:addTouchEventListener(
                function(sender,eventType)
                    StageScene.SetNodeTouchScale(sender,eventType)
                    if eventType== ccui.TouchEventType.ended then
                        return StageScene.ShowChallengePage(true,var.fbid,var.index)
                    end
                end)

            local nd_name=button:getChildByTag(4)
            nd_name:getChildByTag(1):setString(GameDataTransfer.GetGameString(var.name))
            nd_name:getChildByTag(2):setString(var.index)

            local star=GameRecorder.GetFBStar(var.fbid)
            local nd_stars=button:getChildByTag(5)
            for i=1,3 do
                local node=nd_stars:getChildByTag(i)
                node:setVisible(star>=i)
            end

            local button_img=button:getChildByTag(1)
            button_img:loadTexture(chapter.."_"..(11-var.index)..".png",ccui.TextureResType.plistType)  
            
            for _,node in pairs(button:getChildByTag(2):getChildren()) do
                node:setVisible(node:getTag()==select_fb_type)
            end
            
            button:removeFromParent()
            sc_chapter:addChild(button)
        end

        local w = 160
        local innerWidth = math.max(sc_chapter:getContentSize().width,math.floor(#sc_chapter:getChildren())*w+80)
        local innerHeight = sc_chapter:getContentSize().height
        sc_chapter:setInnerContainerSize(cc.size(innerWidth,innerHeight))
        local x,y = 130,210
        for key, child in pairs(sc_chapter:getChildren()) do
            local tx=x+(key-1)*w
            child:setPosition(1300,y)
            local actionTo = cc.MoveTo:create(0.2,cc.p(tx,y))
            local actionTo = cc.Sequence:create(cc.DelayTime:create(key*0.2),cc.EaseBackOut:create(actionTo))
            child:runAction(actionTo)
        end
        sc_chapter:jumpToLeft()
    end

    local bakX=-1280    
    local function Finish2()
        nd_section:setVisible(false)
    end
    local function Reverse()
        local actionTo = cc.MoveTo:create(0.3,cc.p(bakX,nd_section2:getPositionY()))
        nd_section2:runAction(cc.Sequence:create(actionTo,cc.CallFunc:create(Finish2)))
    end

    local bt_return=nd_section2:getChildByTag(4)
    bt_return:addTouchEventListener(
        function(sender,eventType)
            if eventType== ccui.TouchEventType.ended then
                return Reverse()
            end
        end)
        
    if not isMove then return Finish() end
    
    nd_section2:setPositionX(bakX)
    local actionTo = cc.MoveTo:create(0.3,cc.p(0,nd_section2:getPositionY()))
    nd_section2:runAction(cc.Sequence:create(actionTo,cc.CallFunc:create(Finish)))
end


function StageScene.AddChallengePage()

    local background,obstructLayout=GameMainScene.AddBasicSubPageBackgroundPage()    
    children.pg_challengePage=background
    
    local function HidePage()
        return StageScene.ShowChallengePage(false)
    end
    
    GameMainScene.SetObstructLayoutTouchEvent(obstructLayout,HidePage)
    
    local page_layer = cc.CSLoader:createNode("FB-start.csb")
    obstructLayout:addChild(page_layer)
    GameAssistor.SetToSceneCenter(page_layer)

    local nd_referrer = page_layer:getChildByTag(1)
    children.ic_challenge_role=nd_referrer:getChildByTag(1)
    children.tx_challenge_rolename=nd_referrer:getChildByTag(2)
    GameAssistor.SetBlackOutlineNoSpace(children.tx_challenge_rolename)

    --children.ic_challenge_role:loadTexture(GameDataTransfer.GetCardPath(1))

    local nd_infos = page_layer:getChildByTag(2)
    
    local nd_titles = nd_infos:getChildByTag(1)
    StageScene.SetChallengeAwardTitle=function(isSecondTime)
        nd_titles:getChildByTag(2):setVisible(isSecondTime)
        nd_titles:getChildByTag(3):setVisible(not isSecondTime)
    end

    local nd_conditions = nd_infos:getChildByTag(2)
    children.tx_conditionValue2=nd_conditions:getChildByTag(2):getChildByTag(2)
    children.tx_conditionValue3=nd_conditions:getChildByTag(3):getChildByTag(2)
    children.nd_conditionStars=nd_conditions
    children.nd_challenge_awards=page_layer:getChildByTag(3)
    children.tx_challenge_fbname=page_layer:getChildByTag(5)
    children.lingqu = page_layer:getChildByName("anniu_lingqu")

    local bt_close = page_layer:getChildByTag(6)

    children.lingqu:addTouchEventListener(
            function(sender,eventType)
                
                if eventType== ccui.TouchEventType.ended then
                    GameMainScene.HidePageScaleToAction(obstructLayout,function()
                        HidePage()
                        -- background:removeFromParent()
                       StoreScene.CharageRMB(GameData_List.Game_Shop[recommend_player - 1])
                        --StoreScene.QuickOpenTypeStore(res_store_type.ROLE,1)
                    end)
                    
                end
            end)

    CommonAssistor.AddSimpleTouchEvent(bt_close,function()
        GameMainScene.HidePageScaleToAction(obstructLayout,HidePage)
    end)

    local button = page_layer:getChildByTag(4)
    button:addTouchEventListener(
        function(sender,eventType)
            if eventType== ccui.TouchEventType.ended then
                if PINGTEST then return Ping_TestChallengeStage(select_fb_chapter,select_fb_id) end
                return GameManager.startFB(select_fb_id) 
            end
        end)
    button:setPressedActionEnabled(true)
    SetButtonBlueOutline(button,2)
    --GameAssistor.SetButtonTitleBlackOutline(button)
    children.bt_challenge=button
end

function StageScene.ShowChallengePage(show,ID,index)

    if not children.pg_challengePage then 
        if show then
            StageScene.AddChallengePage()
        else return
        end
    end

    local page=children.pg_challengePage  
    


    if show then
        select_fb_id=ID
        
        local gData=GameDataTransfer.GetGameFB(ID)
        recommend_player    = gData.recommend
        recommend_player_ex = gData.ex_power
        for i=1,3 do
            children.ic_challenge_role:getChildByTag(i):setVisible(recommend_player==i)
        end

        local data=GameRecorder.GetRole(recommend_player)
        if data.unlock > 0 then
            -- cclog("data.unlock  " .. data.unlock .. "  recommend_player " .. recommend_player)
             children.lingqu:setVisible(false)
        else
             children.lingqu:setVisible(true)
        end
        

        local rData=GameDataTransfer.GetRoleData(gData.recommend)
        children.tx_challenge_rolename:setString(GameDataTransfer.GetGameString(rData.RoleName))
        children.tx_challenge_fbname:setString(GameDataTransfer.GetGameString(gData.name)..index)
        
        local displayStr=GameDataTransfer.GetSysString(str_system.STRING_TIPS_FB_CONDITION2)
        displayStr = string.gsub(displayStr,"A",math.floor(gData.ConditionValue2/60))
        displayStr = string.gsub(displayStr,"B",math.floor(gData.ConditionValue2)%60)
        children.tx_conditionValue2:setString(displayStr)

        local displayStr=GameDataTransfer.GetSysString(str_system.STRING_TIPS_FB_CONDITION3)
        displayStr = string.gsub(displayStr,"A",gData.ConditionValue3)
        children.tx_conditionValue3:setString(displayStr)

        local stars=GameRecorder.GetFBStars(ID)
        local nd_conditions=children.nd_conditionStars
        for i=1,3 do
            local ndStar=nd_conditions:getChildByTag(i):getChildByTag(1)
            ndStar:setVisible(stars[i])
        end
        
        local nd_challenge_awards=children.nd_challenge_awards
        local isSecondTime=GameRecorder.GetFBStar(ID)>0
        StageScene.SetChallengeAwardTitle(isSecondTime)
        local awardList=string.split(isSecondTime and gData.Awards or gData.FirstAwards,"|")
        for i=1,3 do
            local ndItem=nd_challenge_awards:getChildByTag(i):getChildByTag(1)
            ndItem:setVisible(false)
            if awardList[i] then
                local awards=string.split(awardList[i],",")
                local iData=GameDataTransfer.GetItemData(tonumber(awards[1]))
                if iData then
                    RoleScene.SetEquipItem(nd_challenge_awards:getChildByTag(i),iData.Equip_Id)
                    --outputString("ID:"..iData.Equip_Id.." : ".."IconID:"..iData.IconID.." Quality:"..iData.Quality)
                    local txName=ndItem:getChildByTag(2)
                    local name=GameDataTransfer.GetGameString(iData.Equip_Name)
                    if not isSecondTime then name=(name.."x"..awards[3]) end
                    StageScene.SetQualityNameText(txName,name,iData.Quality)
                    ndItem:setVisible(true)
                end
            end
        end
        GameMainScene.PlayPageScaleToAction(page.obstructLayout)
    end
    page:setVisible(show)
end

function StageScene.SetQualityNameText(tx_name,name,quality)
    tx_name:setString(name) 
    if quality then tx_name:setColor(quality_color[quality]) end
end

function StageScene.GetTypeStorageTotal(Type)
    for ID,data in pairs(GameRecorder.GetStorageList()) do
        if GameDataTransfer.GetItemData(ID).Type==Type then
            return data.qty
        end
    end return 0
end

function StageScene.GetMedicineTotal()
    return StageScene.GetTypeStorageTotal(res_item_type.MEDICINE)
end

function StageScene.UsedMedicine()
    for ID,data in pairs(GameRecorder.GetStorageList()) do
        if GameDataTransfer.GetItemData(ID).Type==res_item_type.MEDICINE then
            if data.qty>0 then
                GameRecorder.StorageChange(ID,data.qty-1)
                return GameRecorder.SaveBasicData()
            end break
        end
    end return outputString("No Medicine")
end

function StageScene.GetChapterAward(type,chapter,index)
    local chapterAwards=GameDataTransfer.GetChapterAwards(type,chapter)
    local ID=chapterAwards.ID
    local starList=GameRecorder.GetFBChapterStars(ID)
    if starList[index] then return end
    StageScene.AddAwardStrings(chapterAwards["award"..index])
    GameRecorder.FBChapterChange(ID,index)
    return GameRecorder.SaveBasicData()
end

function StageScene.AddAwardStrings(awardString)
    local awardList=GameRecorder.ItemGainWithAwardString(awardString)
    local count=0
    for ID,data in pairs(awardList) do
        local gData=GameDataTransfer.GetItemData(data.ID)
        if gData then
            local stringValue=GameDataTransfer.GetGameString(gData.Equip_Name).."x"..data.qty
            GameAssistor.AddSceneGreenStrings(stringValue,nil,nil,nil,0.4*count)
            count=count+1
        end
    end
end

function StageScene.GetExpAndCheckLevel(GetExp)
    if not GetExp or GetExp<=0 then return end
    
    local player=GameRecorder.GetBasicDatas()
    local lvData=GameDataTransfer.GetRoleLevelData(player.level)
    --outputString("GetExp:"..GetExp,"Lv."..player.level,"myExp:"..player.exp.."/"..lvData.exp)
    if player.exp+GetExp>=lvData.exp then
        local newLevel=player.level+1
        local lvData2=GameDataTransfer.GetRoleLevelData(newLevel)
        if lvData2 then
            player.level=newLevel player.exp=0
            GameMainScene.RefreshPlayerStateBar()
            return StageScene.GetExpAndCheckLevel(player.exp+GetExp-lvData.exp)
        end
        player.exp=lvData.exp
    else
        player.exp=(player.exp+GetExp)
    end
end

function StageScene.ChallengeWinnerAwards(ID,GetExp,AwardList,StarList)

    if PINGTEST then return Ping_TestChallengeWinnerAwards(ID) end
    
    StageScene.GetExpAndCheckLevel(GetExp)
    
    GameRecorder.FBChangeStars(ID,StarList or {})
    
    for _,data in pairs(AwardList or {}) do
        if data.ID and data.ID>0 then
            GameRecorder.ItemGain(data.ID,data.qty)
        end
    end

    return GameRecorder.SaveBasicData()
end




------Toutorial-----------------
function StageScene.GetChapterButton(index)
    for key, button in pairs(children.sc_chapter:getChildren()) do
        if key==index then return button end
    end
end

function StageScene.GetSectiionNodeShowing()
    return CommonAssistor.GetPageVisible(children.nd_section)
end

function StageScene.GetSectiionButton(index)
    for key, button in pairs(children.sc_section:getChildren()) do
        if key==index then return button end
    end
end

function StageScene.GetStarAwardButton(index)
    local nd_starAwards=children.nd_starAwards
    local bt_getSAward=nd_starAwards:getChildByTag(index+1)
    return bt_getSAward
end

function StageScene.GetChallengePageShowing()
    return CommonAssistor.GetBackgroundObstructLayoutShowing(children.pg_challengePage)
end

function StageScene.GetChallengeButton()
    return children.bt_challenge
end

function StageScene.GetDifficultyButton(type)
    local nd_button=children.nd_typeButtons
    if not nd_button then return end
    return nd_button:getChildByTag(type)
end