StoreScene = {}

StoreScene = class("StoreScene",function()
    return cc.Scene:create()
end)

local children={}
local select_store_tag=1
local select_store_subpage

function StoreScene:sceneEnter()
    outputString("StoreScene:sceneEnter()")
end

function StoreScene:sceneExit()
    StoreScene.RemoveAllChildren(self)
end

function StoreScene.RemoveAllChildren(scene)
    table.clear(children)
    GameAssistor.RemoveUnusedSceneResource(scene)
end

function StoreScene.create()

    local scene = StoreScene.new()

    Sence_Init(scene)

    local layerFarm = cc.Layer:create()

    scene.getMainLayer=function()
        return layerFarm
    end
    children.mainLayerFarm = layerFarm

    local background = GameAssistor.CreateBackMaskLayout(180)
    layerFarm:addChild(background)

    scene.AddMainChildren()

    scene:addChild(layerFarm)

    return scene
end

function StoreScene.Refresh()
    --if PINGTEST then select_store_tag=4 end
    StoreScene.ChooseTypeTag(select_store_tag)    
    
    local function ChangeSubPage()
        local pageView=children.pv_pageStore
        if select_store_subpage and pageView:getCurPageIndex()~=select_store_subpage then 
            --pageView:scrollToPage(math.min(#pageView:getPages(),select_store_subpage)) 
            if pageView.setCurPageIndex then pageView:setCurPageIndex(select_store_subpage) end
            select_store_subpage=nil 
        end
    end
    try_catch(ChangeSubPage,function() outputString("?????") end) 
end

local storeTypes={res_store_type.ROLE,res_store_type.EQUIP_SET,res_store_type.COINS,res_store_type.VIP,res_store_type.OTHERS}

function StoreScene.QuickOpenTypeStore(type,value)
    local pageID=GameMainScene.pageZOrder.SUBPAGE_STORE
    local nextTag,subpage=1
    for tag,key in pairs(storeTypes) do
        if type==key then nextTag=tag break end
    end
    local isShowing=GameMainScene.CheckSubFuncPageShowing(pageID)
    if type==res_store_type.ROLE then subpage=value and value-2 or 0
    elseif type==res_store_type.EQUIP_SET then subpage=value and value-1 or 0 end
    if isShowing and subpage and select_store_tag==nextTag then return children.pv_pageStore:scrollToPage(subpage) end
    select_store_tag=nextTag select_store_subpage=subpage
    if isShowing then return StoreScene.Refresh() end
    return GameMainScene.AddSceneToSubFuncPage(pageID)
end

function StoreScene.AddMainChildren()

    local layerFarm = children.mainLayerFarm

    local page_layer = cc.CSLoader:createNode("SHOP.csb")
    layerFarm:addChild(page_layer)
    
    children.sc_storeList=page_layer:getChildByTag(1):getChildByTag(1)
    
    local nd_tags=page_layer:getChildByTag(2)
    for i=1,5 do
        local button=nd_tags:getChildByTag(i)
        button:addTouchEventListener(
            function(sender,eventType)
                if eventType== ccui.TouchEventType.ended then
                    return StoreScene.ChooseTypeTag(i)
                end
            end)
        GameAssistor.SetButtonTitleBlackOutline(button)
    end
    children.nd_typeTags=nd_tags

    children.nd_pageStore=page_layer:getChildByTag(3)
    children.pv_pageStore=children.nd_pageStore:getChildByTag(2)
end

function StoreScene.ChooseTypeTag(tag)

    for key,node in pairs(children.nd_typeTags:getChildren()) do
        GameAssistor.SetButtonBright(node,tag~=node:getTag())
    end
    select_store_tag=tag
    
    local sType=storeTypes[tag]
    local isPage=(sType==res_store_type.ROLE or sType==res_store_type.EQUIP_SET)
    local nd_pageStore,sc_storeList = children.nd_pageStore,children.sc_storeList
    nd_pageStore:setVisible(isPage) sc_storeList:setVisible(not isPage)
    
    local goodsList=GameDataTransfer.GetTypeStoreList(sType)
    --table.print(goodsList,"----ChooseTag goodsList--------")

    local pageView=children.pv_pageStore
    pageView:removeAllPages() pageView:removeAllChildren() 
    sc_storeList:removeAllChildren()
    
    if isPage then
        return StoreScene.RefreshPageView(sType,goodsList)
    end
    
    local filenames={"","","SHOP-coin.csb","SHOP-vip.csb","SHOP-dj.csb"}

    for index,data in pairs(goodsList) do
        local page_layer = cc.CSLoader:createNode(filenames[tag])
        local node = page_layer:getChildByTag(1)
        node:removeFromParent() node.ID=data.ID
        sc_storeList:addChild(node)
        
        local title=node:getChildByTag(3)
        if title then
            GameAssistor.SetBlackOutlineNoSpace(title) 
            title:setString(GameDataTransfer.GetGameString(data.name))
        end

        local nd_center=node:getChildByTag(1)
        local bt_change=node:getChildByTag(2)

        CommonAssistor.AddSimpleTouchEvent(bt_change,function()
            return StoreScene.ExchangeItems(data)
        end)
        
        local awardList=GameDataTransfer.GetAwardStringData(data.ResId)
        
        if sType==res_store_type.COINS then
            local icon=nd_center:getChildByTag(0):getChildByTag(1)
            --icon:loadTexture(GameDataTransfer.GetIconPath(data.Icon))
            for i,node in pairs(icon:getChildren()) do
                node:setVisible(index==node:getTag())
            end
            nd_center:getChildByTag(2):setString("x"..awardList[1].qty)
            bt_change:getChildByTag(1):setString(data.Rate)
            
        elseif sType==res_store_type.OTHERS then
            bt_change:getChildByTag(1):setString(data.Rate)
            bt_change:getChildByTag(2):setVisible(data.CurrencyType==res_currency.COINS)

            local ndItem=nd_center:getChildByTag(1)
            local iData=GameDataTransfer.GetItemData(awardList[1].ID)
            if ndItem and iData then
                RoleScene.SetEquipItem(ndItem,iData.Equip_Id)
                local nd_infos=node:getChildByTag(4)
                --nd_infos:getChildByTag(0):setString(GameDataTransfer.GetGameString(iData.Equip_Description))
                GameAssistor.SetSCRichText(GameDataTransfer.GetGameString(iData.Equip_Description),nd_infos:getChildByTag(1),res_color.GRAY)
            end
            
        elseif sType==res_store_type.VIP then
            local days=GameDataTransfer.GetSysString(str_system.STRING_TIPS_FOREVER)
            if data.time>0 then days=data.time..GameDataTransfer.GetSysString(str_system.STRING_TIPS_DAY) end
            nd_center:getChildByTag(3):setString(days)
            bt_change:getChildByTag(2):setString(data.Rate..GameDataTransfer.GetSysString(str_system.STRING_TIPS_YUAN))
            CommonAssistor.PlayTimelineEffect("e-AN-H.csb",bt_change:getChildByTag(3),true)

            local title=node:getChildByTag(6)
            for i,node in pairs(title:getChildren()) do
                node:setVisible(index==node:getTag())
            end
            
            local nd_awards=node:getChildByTag(4)
            for key,aData in pairs(awardList) do
                local ndItem=nd_awards:getChildByTag(key)
                local iData=GameDataTransfer.GetItemData(aData.ID)
                if ndItem and iData then
                    local ndItem=ndItem:getChildByTag(1)
                    RoleScene.SetEquipItem(ndItem:getChildByTag(1),aData.ID)
                    local txName=ndItem:getChildByTag(2)
                    --StageScene.SetQualityNameText(txName,GameDataTransfer.GetGameString(iData.Equip_Name),iData.Quality)
                    txName:setString(GameDataTransfer.GetGameString(iData.Equip_Name))
                    ndItem:getChildByTag(3):setString("x"..aData.qty)
                end
            end
            
            local bt_getAward=node:getChildByTag(5)
            local tx_days=bt_getAward:getChildByTag(2)
            node.Refresh=function(data)
                local isValidity,exchangedtime,myDays=StoreScene.GetVIPValidity(data.ID)
                if isValidity then
                    CommonAssistor.AddSimpleTouchEvent(bt_getAward,function()
                        if exchangedtime<=os.time() then
                            return StoreScene.GetVIPDailyAward(data)
                        end
                    end)
                    tx_days:setString(myDays)
                    GameAssistor.SetButtonBright(bt_getAward,exchangedtime<=os.time())
                end
                bt_change:setVisible(not isValidity) bt_getAward:setVisible(isValidity)
            end
            local isForvevr=data.time==0
            if isForvevr then bt_getAward:getChildByTag(1):setString(days) end
            tx_days:setVisible(not isForvevr)
            node.Refresh(data)
        end
    end
    
    local w = 250
    local innerWidth = math.max(sc_storeList:getContentSize().width,math.ceil(#sc_storeList:getChildren())*w+200)
    local innerHeight = sc_storeList:getContentSize().height
    sc_storeList:setInnerContainerSize(cc.size(innerWidth,innerHeight))
    local x,y = 60,0
    for key, child in pairs(sc_storeList:getChildren()) do
        local tx=x+(key-1)*w
        if PINGTEST1 then
            child:setPosition(1300,y)
            local actionTo = cc.MoveTo:create(0.2,cc.p(tx,y))
            local actionTo = cc.Sequence:create(cc.DelayTime:create(key*0.2),cc.EaseBackOut:create(actionTo))
            child:runAction(actionTo)
        else
            child:setPosition(tx,y)
        end
    end
    sc_storeList:jumpToLeft()
end

function StoreScene.RefreshPageView(sType,goodsList)
    
    local nd_pageStore=children.nd_pageStore
    local pageView=children.pv_pageStore
    
    local function AddPage(filename,data)
        --local page_layer = cc.CSLoader:createNode(filename)
        local page_layer = CommonAssistor.AddEffectLayer(filename,true)
        page_layer:setPosition(428,250)
        local layout = ccui.Layout:create()
        layout:setContentSize(pageView:getContentSize())
        layout:addChild(page_layer)
        pageView:addPage(layout)

        local nd_buy=page_layer:getChildByTag(1)
        nd_buy:getChildByTag(2):setString(data.Rate..GameDataTransfer.GetSysString(str_system.STRING_TIPS_YUAN))
        
        local button=nd_buy:getChildByTag(3)
        CommonAssistor.AddSimpleTouchEvent(button,function()
            return StoreScene.ExchangeItems(data)
        end)
        return page_layer,layout
    end
    
    local nd_leftRight=nd_pageStore:getChildByTag(1)
    local bt_left=nd_leftRight:getChildByTag(1)
    CommonAssistor.AddSimpleTouchEvent(bt_left,function()
        return pageView:scrollToPage(pageView:getCurPageIndex()-1)
    end)
    local bt_right=nd_leftRight:getChildByTag(2)
    CommonAssistor.AddSimpleTouchEvent(bt_right,function()
        return pageView:scrollToPage(pageView:getCurPageIndex()+1)
    end)

    if sType==res_store_type.ROLE then
        for index,data in pairs(goodsList) do
            local page_layer,layout = AddPage("SHOP-hero.csb",data)
            layout.Refresh=function()
                local roleID=tonumber(data.ResId)
                local icons=page_layer:getChildByTag(0)
                for key,node in pairs(icons:getChildren()) do
                    node:setVisible(roleID==node:getTag())
                end
                if StoreScene.CheckRoleUnlock(roleID) then
                    greyNodeWithCCUI(icons:getChildByTag(roleID),false,true)
                    page_layer:getChildByTag(1):setVisible(false)
                    page_layer:getChildByTag(2):setVisible(true)
                end
            end
            layout.Refresh()
        end
    elseif sType==res_store_type.EQUIP_SET then
        for index,data in pairs(goodsList) do
            local page_layer = AddPage("SHOP-shtz.csb",data)
            local awardList=GameDataTransfer.GetAwardStringData(data.ResId)
            local icons=page_layer:getChildByTag(0)
            for key,node in pairs(icons:getChildren()) do
                node:setVisible(index==node:getTag())
            end
            local nd_awards=page_layer:getChildByTag(2)
            for key,data in pairs(awardList) do
                local ndItem=nd_awards:getChildByTag(key)
                local iData=GameDataTransfer.GetItemData(data.ID)
                if ndItem and iData then
                    RoleScene.SetEquipItem(ndItem,data.ID)
                    local ndItem=ndItem:getChildByTag(1)
                    local txName=ndItem:getChildByTag(2)
                    StageScene.SetQualityNameText(txName,GameDataTransfer.GetGameString(iData.Equip_Name),iData.Quality)
                    --txName:setString(GameDataTransfer.GetGameString(iData.Equip_Name))
                    local value,skip=IntensifyScene.GetMainAttribute(iData)
                    local strType,strValue = GameDataTransfer.GetSysString(str_system.STRING_ATTRIBUTE+skip),value
                    ndItem:getChildByTag(3):setString(strType.."+"..value)
                end
            end
        end
    end
    nd_leftRight:setVisible(#pageView:getChildren()>0) 
end

isInCharge = false

function StoreScene.CharageRMB(data, callback, nd)

    local title = ""
    if data.Price - math.floor(data.Price / 10) * 10 > 0 then
        title = string.format("花费%.1f元购买道具：%s", data.Price / 10,  GameDataTransfer.GetGameStrings(data.name))
    else
        title = string.format("花费%.0f元购买道具：%s", data.Price / 10,  GameDataTransfer.GetGameStrings(data.name))
    end
    
	
 --if quick_debug  then
 --    local btnOk = CommonAssistor.AddConfirmBox(title, function() 
 --                StoreScene.ExchangeItemsSuccess(data.ID)
 --
 --                 if callback then
 --                    callback()
 --                end   
 --    end,999999)
 --    btnOk:setTitleText("购买")
 --    return
 --
 --end

    local btnOK = CommonAssistor.AddConfirmBox(title, function() 
            cc.PlatformAdapter:openPay(data.order_id,GameDataTransfer.GetGameStrings(data.name),string.format("%f", data.Price / 10),"",function(p1,p2,p3,p4) 
                -- isInCharge = false

                if GameDataTransfer.GetGameStrings(data.name) ~= "复活" or p1 == "false" then
                    CommonAssistor.AddWarningBox(p2) 
                end

                if p1 == "true" then
                    StoreScene.ExchangeItemsSuccess(data.ID)

                     if callback then
                        callback()
                    end   
                end
               
            end)
        end,999999)
	btnOK:setTitleText("购买")
end

function StoreScene.ExchangeItems(data)
    --table.print(data,"-----goods------")
    
    local sType,cType,cost = data.Type,data.CurrencyType,data.Rate
    
    if cType==res_currency.RMB then
        return StoreScene.CharageRMB(data)
    end
    
    if cType==res_currency.DIAMOND then
        if GameRecorder.GetBasicData("diamond")<cost then 
            return GameMainScene.AddGoToConfirmBox(str_system.STRING_TIPS_DIAMOND_NO_ENOUGH)
        end
        GameRecorder.AddQuantity("diamond",-cost)
    elseif cType==res_currency.COINS then
        if GameRecorder.GetBasicData("coin")<cost then 
            return GameMainScene.AddGoToConfirmBox(str_system.STRING_TIPS_COIN_NO_ENOUGH)
        end
        GameRecorder.AddQuantity("coin",-cost)
    end
    
    return StoreScene.ExchangeItemsSuccess(data.ID)
end

function StoreScene.CheckRoleUnlock(roleID,unlock)
    local role=GameRecorder.GetRole(roleID) 
    if unlock then 
        role.unlock=1 
        GameMainScene.RefreshPlayerRoleList() 
    end
    return role.unlock>0
end

function StoreScene.CharageRMBSuccess(ID)

    local data=GameDataTransfer.GetStoreData(ID)

    if data.CurrencyType~=res_currency.RMB then return end
    
    GameRecorder.AddQuantity("charge",data.Rate)
    
    return StoreScene.ExchangeItemsSuccess(data.ID)
end

function StoreScene.ExchangeItemsSuccess(ID)
    
    local data=GameDataTransfer.GetStoreData(ID)
	GameRecorder.SetBasicData("charge", GameRecorder.GetBasicData("charge") + data.Price)
    local sType,cType = data.Type,data.CurrencyType
    
    if sType==res_store_type.ROLE then
        StoreScene.CheckRoleUnlock(tonumber(data.ResId),true)
         --for _,layout in pairs(children.pv_pageStore:getPages()) do
        --     if layout.Refresh then layout.Refresh() end
        -- end        
    elseif sType==res_store_type.VIP then
        StoreScene.ExchangeVIP(data)
    else
        StageScene.AddAwardStrings(data.ResId)
    end

    GameMainScene.RefreshPlayerStateBar()
    GameRecorder.SaveBasicData()
    
     if sType==res_store_type.ROLE then
         for _,layout in pairs(children.pv_pageStore:getPages()) do
             if layout.Refresh then layout.Refresh() end
         end    
     end
end

function StoreScene.RefreshStoreList()
    for _,layout in pairs(children.sc_storeList:getChildren()) do
        local data=GameDataTransfer.GetStoreData(layout.ID)
        if layout.Refresh then layout.Refresh(data) end
    end
end

function StoreScene.GetVIPDailyAward(data)
    local vData=PlayerDataList.VIPs[data.ID]
    if not vData then return end
    vData.exchangedtime=StoreScene.GetNewDeadline(1)
    StageScene.AddAwardStrings(data.ResId)
    StoreScene.RefreshStoreList()
    GameMainScene.RefreshPlayerStateBar(true)
    return GameRecorder.SaveBasicData()
end

function StoreScene.GetNewDeadline(days,lasttime)
    local exchangedtime=lasttime or os.time()
    local temp = os.date("*t",exchangedtime)
    local deadline=os.time{year=temp.year, month=temp.month, day=temp.day+days, hour=0}
    return deadline
end

function StoreScene.ExchangeVIP(data)
    local exchangedtime=os.time()
    local deadline=StoreScene.GetNewDeadline(data.time,exchangedtime)
    --outputString(os.date("deadline: %c",deadline))
    table.replace(PlayerDataList.VIPs,data.ID,{ID=data.ID,exchangedtime=exchangedtime,deadline=deadline})
    StoreScene.RefreshStoreList() StoreScene.RefreshVIPID()
    return GameRecorder.SaveBasicData()
end

function StoreScene.GetVIPValidity(ID)
    local vData=PlayerDataList.VIPs[ID]
    if not vData then return false,outputString("vip nil") end
    local data=GameDataTransfer.GetStoreData(ID)
    if data.time==0 then return true,vData.exchangedtime end
    local nowtime=os.time()
    if nowtime>vData.deadline then return false,outputString("deadline") end
    local second=os.difftime(vData.deadline,math.max(vData.exchangedtime,nowtime))
    local days=math.ceil(second/86400)--floor,ceil
    return second>0,vData.exchangedtime,days
end

function StoreScene.RefreshVIPID()
    local vip=0
    for ID,data in pairs(PlayerDataList.VIPs) do
        if ID>vip and StoreScene.GetVIPValidity(ID) then vip=ID end
    end
    GameRecorder.SetBasicData("vip",vip)
    return vip
end

