
GameAssistor={}

function GameAssistor.GetRunningScene()
	return cc.Director:getInstance():getRunningScene()
end

function GameAssistor.SceneIsRunning(sceneName)
	local nowScene = GameAssistor.GetRunningScene()
	return nowScene.__cname == sceneName
end

function GameAssistor.SetToSceneCenter(page_layer)
	local screenSize = cc.Director:getInstance():getWinSize()
	page_layer:setPosition(screenSize.width/2,screenSize.height/2)
end

function GameAssistor.IsWindowsPlatform()
    local targetPlatform = cc.Application:getInstance():getTargetPlatform()
    return cc.PLATFORM_OS_WINDOWS==targetPlatform
end

function GameAssistor.IsIOSPlatform()
    local targetPlatform = cc.Application:getInstance():getTargetPlatform()
    return cc.PLATFORM_OS_IPHONE==targetPlatform or cc.PLATFORM_OS_IPAD==targetPlatform or cc.PLATFORM_OS_MAC==targetPlatform
end

function GameAssistor.IsAndroidPlatform()
    local targetPlatform = cc.Application:getInstance():getTargetPlatform()
    return cc.PLATFORM_OS_ANDROID==targetPlatform
end

function GameAssistor.EndToLua()
    if GameAssistor.IsIOSPlatform() and App_Exit then
		return App_Exit()
	end
	cc.Director:getInstance():endToLua()
end

function GameAssistor.CheckNetworkState()
	
    if GameAssistor.IsIOSPlatform() and App_CheckNetworkState then
		return App_CheckNetworkState()
	end
	
    if GameAssistor.IsAndroidPlatform() and LuaJavaBridgeSDK then
		return LuaJavaBridgeSDK.isNetworkConnected()
	end
	
	return true
end

function GameAssistor.CheckKeywordInputWarning(str)
    for _,key in pairs({"'","<f>","<br>"}) do
        local pos = string.find(str,key)
        if pos then
            GameAssistor.AddWarningBox(LoadingScene.GetSysString(str_systring.STRING_INPUT_KEYWORD_ERROR))
            return true
        end
    end
    return false
end

function GameAssistor.GetCurrentDevice()
    if SDK_GetCurrentDevice then
        return SDK_GetCurrentDevice()
    end
    local targetPlatform = cc.Application:getInstance():getTargetPlatform()
    if cc.PLATFORM_OS_ANDROID==targetPlatform then
        return "Android"
    elseif cc.PLATFORM_OS_IPHONE==targetPlatform or cc.PLATFORM_OS_IPAD==targetPlatform then
        return "iOS"
    end
    return "Unknown"
end

function GameAssistor.RemoveUnusedSceneResource(scene)
	scene:removeAllChildren()
	GameAssistor.RemoveUnusedResource()
end

function GameAssistor.RemoveUnusedResource()
	cc.SpriteFrameCache:getInstance():removeUnusedSpriteFrames()
	cc.Director:getInstance():getTextureCache():removeUnusedTextures()
end

function GameAssistor.CleanScrollView(scrollView)
	GameAssistor.RemoveUnusedSceneResource(scrollView)
end

function GameAssistor.CreateMaskLayout(color,opacity)

	local layout = ccui.Layout:create()
	layout:setTouchEnabled(true)
	layout:setContentSize(cc.Director:getInstance():getVisibleSize())
	layout:setBackGroundColor(color)
	layout:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid)
	layout:setOpacity(opacity)

	return layout
end

function GameAssistor.CreateBackMaskLayout(opacity)
	return GameAssistor.CreateMaskLayout(cc.c3b(0,0,0),opacity or 0)
end

function GameAssistor.AddSubPageMaskLayout(nowScene,layerFarm,isMaskEvent,page_layer)

	local layout = GameAssistor.CreateBackMaskLayout(180)

	if isMaskEvent then
		GameAssistor.RemovePageEvent(layout,layerFarm,page_layer,nil)
	end

	layerFarm:addChild(layout)
end

function GameAssistor.RemoveLayout(layerFarm)
	GameAssistor.GetRunningScene():removeChild(layerFarm,true)
end

function GameAssistor.RemovePageEvent(sender,layerFarm,page_layer,btFunction)

	local function RemovePage()
		--page_layer:setVisible(false)
		GameAssistor.RemoveLayout(layerFarm)
		if btFunction then btFunction() end
	end

	sender:addTouchEventListener(
		function(sender,eventType)
			if eventType== ccui.TouchEventType.ended then 
				local action_hide = cc.Sequence:create(cc.EaseBackIn:create(cc.ScaleTo:create(0.3,0)),cc.CallFunc:create(RemovePage))
				page_layer:runAction(action_hide)
				
				GameSound.PlayEffect(SoundEffect.UI_PAGE_CLOSE)
			end
		end)
end

function GameAssistor.AddWarningBox(tips)

	GameAssistor.CreateOneButtonMessageLayout(tips,true)
	
	GameSound.PlayEffect(SoundEffect.UI_PAGE_OPEN)
end

function GameAssistor.AddConfirmBox(tips,btFunction)
	
	local layerFarm,bt_ok = GameAssistor.CreateTwoButtonMessageLayout(tips,false,nil,btFunction)
    
	GameSound.PlayEffect(SoundEffect.UI_PAGE_OPEN)
	
    return layerFarm,bt_ok
end

function GameAssistor.AddWarningConfirmBox(tips,errorData)
    
    local function btFunction()
        return GameMainScene.PageGoTo(errorData.Styleid)
    end
    
    local layerFarm,bt_ok,bt_cancel = GameAssistor.CreateTwoButtonMessageLayout(tips,false,nil,btFunction)
    
    GameSound.PlayEffect(SoundEffect.UI_PAGE_OPEN)
    
    local btStrings=string.split(errorData.StyleString,",")
    bt_ok:setTitleText(btStrings[2] or "ok")
    bt_cancel:setTitleText(btStrings[1] or "cancel")
end

function GameAssistor.AddReconnectMessageBox()

	SCREEN_RECONNECT_MESBOX_TAG=2222

	if GameAssistor.GetRunningScene():getChildByTag(SCREEN_RECONNECT_MESBOX_TAG) then return end
	
	local tips,btString
	if GameDataTransfer then
		tips=GameDataTransfer.GetSysString(str_system.STRING_TIPS_RECONNECT)
		btString=GameDataTransfer.GetSysString(str_system.STRING_BT_RECONNECT)
	else
		tips=LoadingScene.GetSysString(str_systring.STRING_SERVER_NO_CONNECT)
	end

	local layerFarm = GameAssistor.CreateOneButtonMessageLayout(tips,false,btString,GameNetwork.Game_Reconnect)

	layerFarm:setTag(SCREEN_RECONNECT_MESBOX_TAG)

	GameAssistor.GetRunningScene():reorderChild(layerFarm,SCREEN_RECONNECT_MESBOX_TAG)

	GameSound.PlayEffect(SoundEffect.UI_PAGE_OPEN)
end

function GameAssistor.RemoveReconnectMessageBox()

	if not SCREEN_RECONNECT_MESBOX_TAG then return end

	local layerFarm = GameAssistor.GetRunningScene():getChildByTag(SCREEN_RECONNECT_MESBOX_TAG)

	if not layerFarm then return end

	GameAssistor.RemoveLayout(layerFarm)
end

function GameAssistor.AddChargingMessageBox()

    SCREEN_CHARGING_MESBOX_TAG=1222

    if GameAssistor.GetRunningScene():getChildByTag(SCREEN_CHARGING_MESBOX_TAG) then return end

    local tips,btString
    tips=GameDataTransfer.GetSysString(str_system.STRING_TIPS_CHARGING_WAITING)
    btString=GameDataTransfer.GetSysString(str_system.STRING_TIPS_CHARGING_REFRESH)
    
    local function Refresh()
        --GameAssistor.RemoveChargingMessageBox()
        return GameNetwork.Game_Request(request_num.NETWORK_REQUEST_VIP_REFRESH)
    end

    local layerFarm = GameAssistor.CreateOneButtonMessageLayout(tips,false,btString,Refresh)

    layerFarm:setTag(SCREEN_CHARGING_MESBOX_TAG)
    
    GameAssistor.GetRunningScene():reorderChild(layerFarm,SCREEN_CHARGING_MESBOX_TAG)

    GameSound.PlayEffect(SoundEffect.UI_PAGE_OPEN)
end

function GameAssistor.RemoveChargingMessageBox()

    if not SCREEN_CHARGING_MESBOX_TAG then return end

    local layerFarm = GameAssistor.GetRunningScene():getChildByTag(SCREEN_CHARGING_MESBOX_TAG)

    if not layerFarm then return end

    GameAssistor.RemoveLayout(layerFarm)
end

function GameAssistor.AddEffectLayer(csb_path,x,y,loop)

	local effect = cc.CSLoader:createNode(csb_path)
	local timeline = cc.CSLoader:createTimeline(csb_path)
	effect:runAction(timeline)
	effect:setPosition(x,y)
	timeline:gotoFrameAndPlay(0,loop or false)
	effect.timeline=timeline
	return effect,timeline
end

function GameAssistor.AddDelayEffectLayer(node,csb_path,x,y,delay)

	local function PlayAction()
		local effect,timeline = GameAssistor.AddEffectLayer(csb_path,x,y)
		node:addChild(effect)
	end
	local actionTo = cc.Sequence:create(cc.DelayTime:create(delay),cc.CallFunc:create(PlayAction))
	node:runAction(actionTo)
end

function GameAssistor.AddConnectingLayer(isNow)

	SCREEN_CONNECTING_ACTION_TAG=2221
	
	if not GameAssistor.GetRunningScene() then return end

	if GameAssistor.GetRunningScene():getChildByTag(SCREEN_CONNECTING_ACTION_TAG) then return end

	local nowScene = GameAssistor.GetRunningScene()

	local layerFarm = cc.Layer:create()
    
	local layout = GameAssistor.CreateBackMaskLayout()
	layerFarm:addChild(layout)

	local function Show()

		GameAssistor.AddSubPageMaskLayout(nowScene,layerFarm)

		local screenSize = cc.Director:getInstance():getWinSize()
		local page_layer = GameAssistor.AddEffectLayer("guide/loading.csb",screenSize.width/2,screenSize.height/2,true)
		layerFarm:addChild(page_layer)
	end
	
	if isNow then
		Show()
	else
		local actionTo = cc.Sequence:create(cc.DelayTime:create(0.6),cc.CallFunc:create(Show))
		layerFarm:runAction(actionTo)
	end

	layerFarm:setTag(SCREEN_CONNECTING_ACTION_TAG)

	nowScene:addChild(layerFarm,SCREEN_CONNECTING_ACTION_TAG)
end

function GameAssistor.RemoveConnectingLayer()

	if not SCREEN_CONNECTING_ACTION_TAG then return end

	local layerFarm = GameAssistor.GetRunningScene():getChildByTag(SCREEN_CONNECTING_ACTION_TAG)

	if not layerFarm then return end
	
	GameAssistor.RemoveLayout(layerFarm)
end

function GameAssistor.AddWarningTips(tips,point)

    if PINGTEST then return outputString(tips) end

	SCREEN_WARNING_TIPS_TAG=2224

	local nowScene = GameAssistor.GetRunningScene()

	local page_layer = cc.CSLoader:createNode("Warnning.csb")
	nowScene:addChild(page_layer,SCREEN_WARNING_TIPS_TAG)
	--page_layer:setScale(1.2)
	
	if point then
		page_layer:setPosition(point)
	else
		GameAssistor.SetToSceneCenter(page_layer)
	end

	local textLabel = page_layer:getChildByTag(2)

	textLabel:setString(tips)

	local textBg = page_layer:getChildByTag(1)
	local size=textBg:getContentSize()
	textBg:setContentSize(textLabel:getContentSize().width+50,size.height)

	local function HidePage()
		nowScene:removeChild(page_layer)
	end

	local action_hide = cc.Sequence:create(cc.DelayTime:create(1),cc.FadeOut:create(1),cc.CallFunc:create(HidePage))
	page_layer:runAction(action_hide)
end

function GameAssistor.AddBasicAwardNoEnoughTips(awardID,point)
	local str=GameDataTransfer.GetAwardName(awardID)..GameDataTransfer.GetSysString(str_system.STRING_TIPS_NO_ENOUGH)
	return GameAssistor.AddWarningTips(str,point)
end

function GameAssistor.AddBasicAwardNoEnoughBox(awardID)
	local str=GameDataTransfer.GetAwardName(awardID)..GameDataTransfer.GetSysString(str_system.STRING_TIPS_NO_ENOUGH)
	return GameAssistor.AddWarningBox(str)
end

function GameAssistor.SetAutoFadeOutAction(tips)
	
	if tips:isRunning() then tips:stopAllActions() end

	tips:setOpacity(255)

	local function HidePage()
		tips:setVisible(false)
	end

	local action_hide = cc.Sequence:create(cc.DelayTime:create(1),cc.FadeOut:create(1),cc.CallFunc:create(HidePage))
	tips:runAction(action_hide)
end

function GameAssistor.CreateCommonTipsLayout(isMaskEvent)

	SCREEN_WARNING_BOX_TAG=2225

	local nowScene = GameAssistor.GetRunningScene()

	local screenSize = cc.Director:getInstance():getWinSize()

	local layerFarm = cc.Layer:create()

	local obstructLayout = GameAssistor.CreateBackMaskLayout()
	obstructLayout:setAnchorPoint(cc.p(0.5,0.5))
	obstructLayout:setPosition(screenSize.width/2,screenSize.height/2)
	obstructLayout:setTouchEnabled(false)
    
	local page_layer = cc.CSLoader:createNode("tp-1.csb")

	GameAssistor.AddSubPageMaskLayout(nowScene,layerFarm,isMaskEvent,obstructLayout)
	
    layerFarm.obstructLayout=obstructLayout
	obstructLayout:addChild(page_layer)
	layerFarm:addChild(obstructLayout)

	local img_bg = page_layer:getChildByTag(1):getChildByTag(1)
	--img_bg:setScale(0.48,0.4)
	local bgSize=img_bg:getContentSize()
	img_bg:setContentSize(bgSize.width,442)

	page_layer:setPosition(screenSize.width/2,screenSize.height/2)

    obstructLayout:setScale(0)
    obstructLayout.isDone=false
    local ease = cc.EaseBackOut:create(cc.ScaleTo:create(0.3,1))
    local actionTo = cc.Sequence:create(ease,cc.CallFunc:create(function()
        obstructLayout.isDone=true
    end))
    obstructLayout:runAction(actionTo)

	nowScene:addChild(layerFarm,SCREEN_WARNING_BOX_TAG,SCREEN_WARNING_BOX_TAG)

	return page_layer,layerFarm
end

function GameAssistor.ShowServerMaintainMessage(message)

	local page_layer,layerFarm = GameAssistor.CreateCommonTipsLayout()

	local button = GameAssistor.CreateCommonButtonMessageLayout(page_layer,message)
	
	--[[
	local nd_messageBox = page_layer:getChildByTag(2)
	nd_messageBox:setVisible(true)

	local titleLabel = nd_messageBox:getChildByTag(1):getChildByTag(2)
	GameAssistor.SetBlackOutline(titleLabel)

	local textLabel = nd_messageBox:getChildByTag(2)
	GameAssistor.SetBlackOutline(textLabel)
	textLabel:setString(message)
	
	local button = nd_messageBox:getChildByTag(3)
	GameAssistor.SetButtonTitleBlackOutline(button)--]]
	button:addTouchEventListener(
		function(sender,eventType)
			if eventType== ccui.TouchEventType.ended then 
				GameAssistor.EndToLua()
			end
		end)
end

function GameAssistor.GetStringLabelNumLines(text)
	local vLabel=text:getVirtualRenderer()
	local lineWidth=text:getContentSize().width
	vLabel:setMaxLineWidth(lineWidth)
	return vLabel:getStringNumLines()
end

function GameAssistor.GetStringLabelHeight(text)
	local vLabel=text:getVirtualRenderer()
	return (GameAssistor.GetStringLabelNumLines(text) * vLabel:getLineHeight())
end

function GameAssistor.SetStringLabelHeight(text)
	local vLabel=text:getVirtualRenderer()
	local lineWidth=text:getContentSize().width
	local height=GameAssistor.GetStringLabelHeight(text)
	text:setContentSize(lineWidth,height)
	return height
end

function GameAssistor.CreateCommonButtonMessageLayout(page_layer,tips,btString)

	local nd_messageBox = page_layer:getChildByTag(2)
	nd_messageBox:setVisible(true)

	local nd_messageInfo = nd_messageBox:getChildByTag(1)
	local titleLabel = nd_messageInfo:getChildByTag(1):getChildByTag(2)
	GameAssistor.SetBlackOutline(titleLabel)

	local textLabel = nd_messageInfo:getChildByTag(2)
	--GameAssistor.SetBlackOutline(textLabel)
	textLabel:setString(tips)
	
	if GameAssistor.GetStringLabelNumLines(textLabel)<2 then
		local img_bg = page_layer:getChildByTag(1):getChildByTag(1)
		img_bg:setContentSize(img_bg:getContentSize().width,280)
		nd_messageInfo:setPositionY(-60)
	end

	local nd_buttons = nd_messageBox:getChildByTag(2)
	local bt_ok = nd_buttons:getChildByTag(1)
	local bt_cancel = nd_buttons:getChildByTag(2)

	--if not btString then btString=GameDataTransfer.GetSysString(str_system.STRING_BT_OK) end
	if btString then
		bt_ok:setTitleText(btString)
	end
	GameAssistor.SetButtonTitlesBlackOutline{bt_ok,bt_cancel}
	
	return bt_ok,bt_cancel
end

function GameAssistor.CreateOneButtonMessageLayout(tips,isMaskEvent,btString,btFunction)

	local page_layer,layerFarm = GameAssistor.CreateCommonTipsLayout(isMaskEvent)

	local button = GameAssistor.CreateCommonButtonMessageLayout(page_layer,tips,btString)
	
	GameAssistor.RemovePageEvent(button,layerFarm,page_layer,btFunction)
	
	return layerFarm
end

function GameAssistor.CreateTwoButtonMessageLayout(tips,isMaskEvent,btString,btFunction)

	local page_layer,layerFarm = GameAssistor.CreateCommonTipsLayout(isMaskEvent)
	
	local bt_ok,bt_cancel = GameAssistor.CreateCommonButtonMessageLayout(page_layer,tips,btString)
	bt_cancel:setVisible(true)

	GameAssistor.RemovePageEvent(bt_ok,layerFarm,page_layer,btFunction)
	GameAssistor.RemovePageEvent(bt_cancel,layerFarm,page_layer)
	
    return layerFarm,bt_ok,bt_cancel
end

function GameAssistor.SetTextSpace(text,space)
	text:getVirtualRenderer():setAdditionalKerning(space)
end

function GameAssistor.SetTextsSpace(children,space)
	for key, var in ipairs(children) do
		GameAssistor.SetTextSpace(var,space)
	end
end

local outlineDefault=1
function GameAssistor.SetBlackOutline(child,space)
	space = space or outlineDefault
	child:enableOutline(cc.c4b(0,0,0,255),space)
end

function GameAssistor.SetBlackOutlineNoSpace(child,space)
	space = space or outlineDefault
	if not child.outlineSize then child.outlineSize=child:getFontSize()-outlineDefault end
	child:setFontSize(child.outlineSize)
	GameAssistor.SetBlackOutline(child)
	GameAssistor.SetTextSpace(child,-space)
end

function GameAssistor.SetBlackOutlineNoSpaces(children)
	for key, var in ipairs(children) do
		GameAssistor.SetBlackOutlineNoSpace(var)
	end
end

function GameAssistor.SetBlackOutlines(children)
	for key, var in ipairs(children) do
		GameAssistor.SetBlackOutline(var)
	end
end

function GameAssistor.SetButtonTitleBlackOutline(button)
	local title=button:getTitleRenderer()
	GameAssistor.SetBlackOutline(title)
end

function GameAssistor.SetButtonTitlesBlackOutline(children)
	for key, var in ipairs(children) do
		GameAssistor.SetButtonTitleBlackOutline(var)
	end
end

function GameAssistor.SetLabelFollowLabel(label1,label2,space)
	local tx_len=label1:getContentSize().width
	label2:setPositionX(label1:getPositionX()+tx_len+space)
end

function GameAssistor.SetButtonsPressedAction(children)
	for key, button in ipairs(children) do
		button:setPressedActionEnabled(true)
	end
end

function GameAssistor.SetButtonBright(button,show)
	button:setEnabled(show)
	button:setBright(show)
end

function GameAssistor.SetColorText(text,string,color)
	text:setString(string)
	text:setColor(color)
end

function GameAssistor.GetWorldPosition(node)
	return node:convertToWorldSpaceAR(cc.p(node:getPosition()))
end

function GameAssistor.AddSceneGreenStrings(stringValue,nowScene,x,y,delay)

	if not nowScene then nowScene=GameAssistor.GetRunningScene() end
    local tx_attribute= cc.Label:createWithTTF(res_font.attribute,stringValue)
	tx_attribute:setPosition(x or 640,y or 360)
	nowScene:addChild(tx_attribute,999)
    
	local function HidePage()
		nowScene:removeChild(tx_attribute)
	end
	tx_attribute:setOpacity(0)
	tx_attribute:setTextColor(cc.c4b(0,255,0,255))
	local action_show = cc.Spawn:create(cc.MoveBy:create(2,cc.p(0,180)),cc.Sequence:create(cc.FadeIn:create(0.7),cc.DelayTime:create(0.6),cc.FadeOut:create(0.3)))
    local action_hide = cc.Sequence:create(cc.DelayTime:create(delay or 0),action_show,cc.CallFunc:create(HidePage))
	tx_attribute:runAction(action_hide)
	
    return tx_attribute
end

function GameAssistor.AddExchangeSuccessStrings(awardID,awardQty)
    local name,quality = GameDataTransfer.GetAwardName(awardID)
    local stringDisplay=GameDataTransfer.GetSysString(str_system.STRING_TIPS_EXCHANGE_SUCCESS)
    stringDisplay = stringDisplay.."\""..name.."\""
    if awardQty and awardQty>1 then
        stringDisplay = stringDisplay.."x"..awardQty
    end
    GameAssistor.AddSceneGreenStrings(stringDisplay)
end

GameAssistor.splitRichText = function(str,color)      
    local sub_str_tab = {}
    local split_char="%[(.-)%]<(.-)>"
    while (string.len(str)>0) do
        local pos,_end,value1,value2 = string.find(str, split_char) 
        if (not pos) then
            table.insert(sub_str_tab,{color=color,string=str}) break
        end
        local sub_str = string.sub(str, 1, pos - 1)
        table.insert(sub_str_tab,{color=color,string=sub_str})
        table.insert(sub_str_tab,{color=tonumber(value1),string=value2})
        str = string.sub(str, _end+1, -1)
    end      
    return sub_str_tab
end 

function GameAssistor.SetToolsRichText(strContent,scrollView,color,offset,fontSize)
    scrollView:setLayoutType(ccui.LayoutType.VERTICAL)
    getRichText(strContent,color or res_color.FULL,scrollView:getContentSize().width,res_font.default,fontSize or 20,scrollView,"\r\n",offset)
end

function GameAssistor.SetSCRichText(strContent,scrollView,color)
    scrollView:removeAllChildren()
    return GameAssistor.SetToolsRichText(strContent,scrollView,color)
end

