
GameEventManager={}

local GameEnemyQueue={}
local GameEventQueue={}
local mRandom = Random

conditionIDs = enum{
    "CONDITION_WAVE=1",
	"CONDITION_LASTAREA",
    
    "CONDITION_MOVE=1",
    "CONDITION_BLOON",
    "CONDITION_START",
    "CONDITION_QUEUEEND",

    "CONDITION_ADDEMENYS=1",
    "CONDITION_ADDEMENYTIMES",
    "CONDITION_AREA",
	"CONDITION_LOSER",
	"CONDITION_ADDEMENYLOOPS",
	"CONDITION_REMOVEEMENYS",
	"CONDITION_BOSS_READY",
}

local EventDefault={
    ID=0,
    area=1,
    delay=0,
    position="0,0",
    condition_ID=0,
    condition_value=0,
    isComplete=false,
    isDone=false,
    start_time=0,
	last_wave=0,
}

local EnemyQueueDefault={
    ID=0,
    relation=0,
    pos_type=0,
    enemys={
        --{ID=0,position="0,0",x=0,y=0,AI=0,speed=1,view=1}
    },
    isAppear=false,
}

local FBDataDefault={
    ID=0,
    position="0,0",
    begin_x=0,
    begin_y=0,
    time=0,
    areas={
        --{ID=0,sx=0,ex=0}
    },
    condition_ID=0,
    --condition_value="1",
    condition_objs={},
}

local now_fb_id = 0
----------------------------------------------------------------------

local function SetDataToGameManager(FBData)
	GameManager.setHeroPosition(FBData.begin_x,FBData.begin_y)
	GameManager.setWaveInfo(FBData.WinBoCi)
	GameManager.setTime(FBData.showCountDown,FBData.time)
end

local function GetHeroHP()
    return GameManager.getHeroHpPercent()
end

local function GetHeroPos()
    return GameManager.getHeroPosition()
end

local function GetPositionXY(position)
    local xy=string.split(position,",")
    return tonumber(xy[1]),tonumber(xy[2])
end

local function GetPositionRect(position)
    local xy=string.split(position,",")
    return tonumber(xy[1]),tonumber(xy[2]),tonumber(xy[3]),tonumber(xy[4])
end

local now_playing_area=1
local max_playing_area=1
local unlock_playing_area=1
local event_is_started=false
local event_is_finish=false
local boss_wave_finish=false
local event_is_pause=false

local function UpdateHeroArea()

    for key, area in pairs(FBDataDefault.areas) do
		if area.ID>now_playing_area and area.ID<=unlock_playing_area then  --支持矩形区域
		    local herox,heroy = GetHeroPos()
            if herox>area.sx and herox< (area.sx+area.width) and heroy>area.sy and heroy <(area.sy+area.height) then
                now_playing_area=area.ID
                break
            end
        end
    end
    return now_playing_area
end

local function GetAreaPos(areaID)

	local area = FBDataDefault.areas[areaID]
	
	if not area then return 0,0,0,0 end
	
    return area.sx,area.sy,area.width,area.height
end

local function GetMaxArea()
    local max_area=1
    for key, area in pairs(FBDataDefault.areas) do
        if area.ID>max_area then
            max_area=area.ID
        end
    end
    return max_area
end

local function SetAreaPos(areaID)
	unlock_playing_area=areaID
    GameManager.setArean(GetAreaPos(areaID))
end

local function GetRunningTime()
    return GameManager.getTime()
end

local function GameOver(isWin,isTimeOver)
	event_is_finish=true
	if isWin then
		GameSound.PlayMusic(SoundType.BG_WIN)  
	else
		GameSound.PlayMusic(SoundType.BG_LOST)  
	end
    GameManager.gameFinish(isWin,isTimeOver or false)
	return true
end


local event_runningTime=0
local event_runningStart=Time_GetCurrentTime()

local function GetCurrentTime()
    return event_runningTime
end

local function GetDelayTime(start)
    return (event_runningTime-start)
end

local function UpdateCurrentTime()
    local delay=Time_GetDelayTime(event_runningStart)
    if delay>0 then
        event_runningTime = event_runningTime+delay
        event_runningStart=Time_GetCurrentTime()
    end
end

----------------------------------------------------------------------

function GameEventManager.RunEventFile(filename)

    --filename="Events/"..filename
    cclog("GameEventManager.RunEventFile  %s", filename)
    GameEventManager.ResetDatas()
    
    local contentBytes=GameDataReader.readFromLocalFile(filename)

    if not contentBytes then return end
    
    GameEventManager.LoadFileDatas(contentBytes)

    GameEventManager.ConfigEvent()
    
    GameEventManager.Run()
end

function GameEventManager.LoadEventFile(m_fb_id)

    if GameAssistor.SceneIsRunning("GameScene") then
        if GameManager.FB_Is_Running() then return end
    end
    local filename="Data/FB/"..m_fb_id..".data"
    cclog("fbid:" .. m_fb_id)
    
    now_fb_id = m_fb_id

    GameEventManager.ResetDatas()

    local contentBytes=GameDataReader.readDataFromAppFile(filename)

    if not contentBytes then return end
    
    

    GameEventManager.LoadFileDatas(contentBytes)
    

end

function GameEventManager.LoadNetworkEventBytes(contentBytes)
    
    if GameAssistor.SceneIsRunning("GameScene") then
        if GameManager.FB_Is_Running() then return end
    end

    GameEventManager.ResetDatas()

    if not contentBytes then return end
    
    if PINGTEST then
		--contentBytes=GameDataReader.readFromLocalFile("test.dt")
    end

    GameEventManager.LoadFileDatas(contentBytes)
end

function GameEventManager.ConfigEvent()

    now_playing_area=1
    
    max_playing_area=nil
    
	event_is_finish=false
	boss_wave_finish=false
	event_is_started=false

    SetAreaPos(now_playing_area)
    
	SetDataToGameManager(FBDataDefault)
    
	PreloadGameEnemyRes()
end

function GameEventManager.Resume()
	event_is_finish=false
end

function GameEventManager.Start()
	event_is_started=true
end

function GameEventManager.Pause(isPause)
	event_is_pause=isPause
end

function GameEventManager.Shutdown()
    --outputString("GameEventManager.Shutdown()")

    GameEventManager.Stop()
    GameEventManager.ResetDatas()
end

function GameEventManager.Run()

    GameEventManager.Stop()
    
    local function tick()
        GameEventManager.UpdateProcess()
    end
    GameEventManager.schedulerID = cc.Director:getInstance():getScheduler():scheduleScriptFunc(tick, 0, false)
end

function GameEventManager.Stop()

    if GameEventManager.schedulerID then
        cc.Director:getInstance():getScheduler():unscheduleScriptEntry(GameEventManager.schedulerID)
    end
end

function GameEventManager.UpdateProcess()
    --outputString("GameEventManager.UpdateProcess()")
    
    if not event_is_started then return end
    
	if event_is_finish then GameManager.updateFinish() return end
	
	if GameEventManager.CheckFBComplete() then
        return GameOver(true)
	elseif GetRunningTime()<=0 or GameManager.HeroIsDead() then 
        return GameOver(false,GetRunningTime()<=0)
	end

	if event_is_pause then 
        event_runningStart=Time_GetCurrentTime()
	return end
	
    UpdateCurrentTime()
    
    GameEventManager.CheckEventQueues()
end

function GameEventManager.ResetDatas()
    outputString("GameEventManager.ResetDatas")
    table.clear(GameEnemyQueue)
    table.clear(GameEventQueue)
end

function GameEventManager.LoadFileDatas(contentBytes)

    local skip,count
    
    skip = GameEventManager.LoadFBDatas(contentBytes,skip)
    
    skip = GameEventManager.LoadEnemyQueueDatas(contentBytes,skip)
    
    skip = GameEventManager.LoadEventQueueDatas(contentBytes,skip)
    
    event_runningTime=0
    event_runningStart=Time_GetCurrentTime()
    
    for key, event in pairs(GameEventQueue) do
        for key, event in pairs(event.gameEvents) do
            event.start_time=GetCurrentTime()
        end
        event.start_time=GetCurrentTime()
    end
    
    local fbData = GameDataTransfer.GetGameFB(now_fb_id)
    FBDataDefault.time = fbData.TimeLimit
    
    return true
end

function GameEventManager.LoadFBDatas(contentBytes,skip)

    skip,FBDataDefault.Type = GameDataReader.readInt(contentBytes,skip)
    skip,FBDataDefault.ID = GameDataReader.readInt(contentBytes,skip)
    skip,FBDataDefault.condition_ID = GameDataReader.readInt(contentBytes,skip)
    
    ---[[
    --skip,FBDataDefault.condition_value = GameDataReader.readString(contentBytes,skip)
    table.clear(FBDataDefault.condition_objs)
    local count,ID
    skip, count = GameDataReader.readShort(contentBytes,skip)
    outputString("LoadFBDatas condition_objs count:",count)
    for i=1,count do
        skip,ID = GameDataReader.readInt(contentBytes,skip)
        table.insert(FBDataDefault.condition_objs,ID)
    end
    --]]
    
    skip,FBDataDefault.position = GameDataReader.readString(contentBytes,skip)
	skip,FBDataDefault.time = GameDataReader.readInt(contentBytes,skip)
	skip,FBDataDefault.WinBoCi = GameDataReader.readInt(contentBytes,skip)
    skip,FBDataDefault.showCountDown = GameDataReader.readInt(contentBytes,skip)
    
    FBDataDefault.begin_x,FBDataDefault.begin_y=GetPositionXY(FBDataDefault.position)

    table.clear(FBDataDefault.areas)
    local count
    skip, count = GameDataReader.readShort(contentBytes,skip)
    outputString("LoadFBDatas areas count:",count)
    for i=1,count do
        local tab={}
        skip,tab.ID = GameDataReader.readInt(contentBytes,skip)
        skip,tab.position = GameDataReader.readString(contentBytes,skip)
        tab.sx,tab.sy,tab.width,tab.height=GetPositionRect(tab.position)
        table.replace(FBDataDefault.areas,tab.ID,tab)
    end

	

	--table.print(FBDataDefault,"---------------FBDataDefault----------------")
	
    return skip
end

function GameEventManager.LoadEnemyQueueDatas(contentBytes,skip)

    local count
    skip, count = GameDataReader.readShort(contentBytes,skip)
    outputString("LoadEnemyQueueDatas data count:",count)
    
    for i=1,count do
        local table_data={}

        table.copy(EnemyQueueDefault,table_data)

        skip,table_data.ID = GameDataReader.readInt(contentBytes,skip)
		skip,table_data.relation = GameDataReader.readInt(contentBytes,skip)
		skip,table_data.pos_type = GameDataReader.readInt(contentBytes,skip)

        table_data.enemys={}
        local count
        skip, count = GameDataReader.readShort(contentBytes,skip)
        for i=1,count do
            local tab={}
            skip,tab.ID = GameDataReader.readInt(contentBytes,skip)
            skip,tab.position = GameDataReader.readString(contentBytes,skip)
            tab.x,tab.y=GetPositionXY(tab.position)
            skip,tab.AI = GameDataReader.readInt(contentBytes,skip)
			skip,tab.speed = GameDataReader.readInt(contentBytes,skip)
			skip,tab.view = GameDataReader.readInt(contentBytes,skip)
			skip,tab.hpLayer = GameDataReader.readInt(contentBytes,skip)
			skip,tab.appearStyle = GameDataReader.readInt(contentBytes,skip)
			skip,tab.pos_direction = GameDataReader.readInt(contentBytes,skip)
			skip,tab.delayTime = GameDataReader.readInt(contentBytes,skip)
            table.insert(table_data.enemys,tab)
        end
        table.replace(GameEnemyQueue,table_data.ID,table_data)
    end
    --table.print(GameEnemyQueue,"---------------GameEnemyQueue----------------")
    return skip
end

function GameEventManager.LoadEventQueueDatas(contentBytes,skip)
    
    local count
    skip, count = GameDataReader.readShort(contentBytes,skip)
    outputString("LoadEventQueueDatas data count:",count)

	for i=1,count do
        local table_data={}

        table.copy(EventDefault,table_data)

        skip,table_data.ID = GameDataReader.readInt(contentBytes,skip)
        skip,table_data.area = GameDataReader.readInt(contentBytes,skip)
        skip,table_data.condition_ID = GameDataReader.readInt(contentBytes,skip)
        skip,table_data.delay = GameDataReader.readInt(contentBytes,skip)
        
        --unconfirmed
        skip,table_data.condition_value = GameDataReader.readString(contentBytes,skip)
        
        table_data.gameEvents={}
        local count
        skip, count = GameDataReader.readShort(contentBytes,skip)
        for i=1,count do
            local tab={}
            
            table.copy(EventDefault,tab)
            
            skip,tab.ID = GameDataReader.readInt(contentBytes,skip)
            skip,tab.condition_ID = GameDataReader.readInt(contentBytes,skip)
            skip,tab.condition_value = GameDataReader.readInt(contentBytes,skip)
            
            --unconfirmed
            
            skip,tab.condition_value2 = GameDataReader.readString(contentBytes,skip)
            skip,tab.condition_value3 = GameDataReader.readInt(contentBytes,skip)
            skip,tab.condition_value4 = GameDataReader.readInt(contentBytes,skip)

            table.replace(table_data.gameEvents,tab.ID,tab)
        end
        
        table.replace(GameEventQueue,table_data.ID,table_data)
    end
    table.print(GameEventQueue,"---------------GameEventQueue----------------")
    return skip
end

function GameEventManager.GetEnemyIDList()
	local result={}
	for key, var in pairs(GameEnemyQueue) do
		for key, var in pairs(var.enemys) do
			table.replace(result,var.ID,var.ID)
		end
	end
	table.print(result,"---------------result----------------")
	return result
end

function GameEventManager.SetBossWaveComplete()
	boss_wave_finish=true
end

function GameEventManager.CheckBossWaveComplete(checkHp)

    local isComplete=false
    if conditionIDs.CONDITION_WAVE==FBDataDefault.condition_ID then
        for key,qid in pairs(FBDataDefault.condition_objs) do
            if not GameEnemyQueue[qid].isAppear then return false end
			if not GameEventManager.CheckPosEnemys(qid,checkHp) then return false end
        end
        isComplete=true
	end
	return isComplete
end

function GameEventManager.CheckFBComplete()

    local isComplete=false
    
    if conditionIDs.CONDITION_WAVE==FBDataDefault.condition_ID then
    
		isComplete=GameEventManager.CheckBossWaveComplete(false) or boss_wave_finish
        
    elseif conditionIDs.CONDITION_LASTAREA==FBDataDefault.condition_ID then
    
        if not max_playing_area then
            max_playing_area=GetMaxArea()
        end
        isComplete = (UpdateHeroArea()==max_playing_area)
    end
    
    return isComplete
end

function GameEventManager.ClearAreaEventQueues()

    for key, event in pairs(GameEventQueue) do
        if event.area<now_playing_area then
            table.clear(GameEventQueue[key]) GameEventQueue[key]=nil
        end
    end
end

function GameEventManager.CheckEventQueues()
    
    GameEventManager.ClearAreaEventQueues()
    
    for key, event in pairs(GameEventQueue) do
        if event.area==now_playing_area then
            if event.isDone then
            --remove?
            elseif event.isComplete then
                if GetDelayTime(event.start_time)>=event.delay then
                    event.isDone = GameEventManager.CheckEventList(event.gameEvents)==0
                end
            else
                local isComplete=false
                if conditionIDs.CONDITION_MOVE==event.condition_ID then
                    local hx,hy = GetHeroPos()
                    local tx,ty = GetPositionXY(event.condition_value)
                    if hx > tx then
                        isComplete=true
                    end
                elseif conditionIDs.CONDITION_BLOON==event.condition_ID then
                    local hp = tonumber(event.condition_value)
                    if GetHeroHP()<hp then
                        isComplete=true
                    end
                elseif conditionIDs.CONDITION_START==event.condition_ID then
                    isComplete=true
                elseif conditionIDs.CONDITION_QUEUEEND==event.condition_ID then
                    if (not GameEventQueue[event.condition_value]) then
                    	--isComplete=true
                    elseif GameEventQueue[event.condition_value].isDone then
                        isComplete=true
                    end
                end
                if isComplete then
                    event.isComplete=true
                    event.start_time=GetCurrentTime()
                end
            end
        end
    end
end

function GameEventManager.CheckEventList(eventList)

    if not eventList then return 0 end

    local event = eventList[1]

    if not event then return 0 end
    
    if event.isComplete then
        table.remove(eventList,1) event = eventList[1]
        if not event then return 0 end
    end
    
    GameEventManager.CompelEvent(event)

    if conditionIDs.CONDITION_ADDEMENYS==event.condition_ID then
        if event.isDone then
            event.isComplete=GameEventManager.CheckPosEnemys(event.condition_value,false)
        end
	elseif conditionIDs.CONDITION_ADDEMENYTIMES==event.condition_ID then
		if event.isDone then
			local isComplete=true
			local condition_objs=event.condition_objs or string.split(event.condition_value2,",")
			for key, var in pairs(condition_objs) do
				if not GameEventManager.CheckPosEnemys(tonumber(var),false) then
					isComplete=false
				end
			end
			event.isComplete=isComplete
		end
	elseif conditionIDs.CONDITION_ADDEMENYLOOPS==event.condition_ID then
		--No Complete
    elseif conditionIDs.CONDITION_AREA==event.condition_ID then
        if UpdateHeroArea() >= event.condition_value then
            event.isComplete=true
        end
    elseif conditionIDs.CONDITION_LOSER==event.condition_ID then
		event.isDone=GameOver(false)
	elseif conditionIDs.CONDITION_REMOVEEMENYS==event.condition_ID then
		event.isComplete=event.isDone
    elseif conditionIDs.CONDITION_BOSS_READY==event.condition_ID then
		event.isComplete=event.isDone
    end
    
    return #eventList
end

function GameEventManager.CompelEvent(event)

    if event.isDone then return end

    if conditionIDs.CONDITION_ADDEMENYS==event.condition_ID then
        if GameManager.FB_Value.is_infinite == false then
		    event.isDone=GameEventManager.AddPosEnemys(event.condition_value,event.condition_ID)
		else
            if GetDelayTime(event.start_time)>= 5 then
                
                GameEventManager.AddPosEnemys(event.condition_value,event.condition_ID)
                
                event.start_time=GetCurrentTime()
                
            end
		end
    elseif conditionIDs.CONDITION_ADDEMENYTIMES==event.condition_ID then
        if GetDelayTime(event.start_time)>=event.condition_value then
            local condition_objs=event.condition_objs
            if not condition_objs then
				condition_objs=string.split(event.condition_value2,",")
				event.condition_objs=condition_objs
            end
			local wave_id=tonumber(condition_objs[event.area])
			GameEventManager.AddPosEnemys(wave_id,event.condition_ID)
            if event.area==#condition_objs then event.area=1
            else event.area=event.area+1 end
            event.start_time=GetCurrentTime()
            event.condition_value3=event.condition_value3-1
        end
		event.isDone=(event.condition_value3<=0)
	elseif conditionIDs.CONDITION_ADDEMENYLOOPS==event.condition_ID then
		local condition_objs=event.condition_objs
		if not condition_objs then
			condition_objs=string.split(event.condition_value2,",")
			event.condition_objs=condition_objs
		end
		local wave_id=tonumber(condition_objs[math.max(event.last_wave or 1)])
		event.isEmpty=GameEventManager.CheckPosEnemys(wave_id,false)
		if event.isEmpty then
			if event.last_wave==#condition_objs then event.last_wave=1
			else event.last_wave=event.last_wave+1 end
			wave_id=tonumber(condition_objs[event.last_wave])
			GameEventManager.AddPosEnemys(wave_id,event.condition_ID)
		end
    elseif conditionIDs.CONDITION_AREA==event.condition_ID then
        event.condition_value=now_playing_area+1
        SetAreaPos(event.condition_value)
		event.isDone=true
	elseif conditionIDs.CONDITION_REMOVEEMENYS==event.condition_ID then
		GameManager.delMonster_with_waveid(event.condition_value)
		event.isDone=true
    elseif conditionIDs.CONDITION_BOSS_READY==event.condition_ID then
        GameManager.addBossTips(event.condition_value)
		event.isDone=true
    end
end

function GameEventManager.AddPosEnemys(qid,type)

	if boss_wave_finish then return false end

    if not GameEnemyQueue[qid] then return false end
    
    
    
    if GameManager.FB_Value.is_infinite == false then
    
        local enemyQueue = GameEnemyQueue[qid]

        outputString("enemyQueue----------",enemyQueue.ID)
        GameManager.addMonster(enemyQueue,type)

        enemyQueue.isAppear=true
        
        return true
    else
        local enemyQueue = GameEnemyQueue[qid]

        outputString("enemyQueue----------",enemyQueue.ID)
        GameManager.addMonster(enemyQueue,type)

        enemyQueue.isAppear=true
        
        GameManager.FB_Value.infinite_level = GameManager.FB_Value.infinite_level + 1
        
        return false
    end
end

function GameEventManager.CheckPosEnemys(qid,checkHp)
    
    local enemyList=GameManager.getEnemyList()
    
    if not enemyList then return false end
    
    for i=1,List.getSize(enemyList) do
        if enemyList[i]._wave_id == qid then
        	if checkHp then
				if enemyList[i]._hp>0 then return false end
			else
				return false
        	end
        end
    end

    return true
end

