
GameRecorder={}

function GameRecorder.DefaultSystemSetting()

    local cUserDefault=cc.UserDefault:getInstance()

    local isSetting = cUserDefault:getBoolForKey("isSetting")

    if isSetting then
        local SoundOn = cUserDefault:getBoolForKey("SoundOn")
        GameSound.SetEnable(SoundOn,SoundOn)
    else
        cUserDefault:setBoolForKey("isSetting",true)
        cUserDefault:setBoolForKey("SoundOn",true)
    end
end

res_record=enum{
    "TESTDATA=0",
    "BASICBACKUP",
    "BASICDATA",
    
    "VERSION=2016071400",
}
function GameRecorder.GetRecordPath(key)  
    return GameDataReader.getWritablePath("rcd"..key..".rc")
end

PlayerDataList={}

function GameRecorder.DefaultBasicData(isSave,isBackup)

    table.clear(PlayerDataList)

    PlayerDataList.Basic={
        coin=0, diamond=0, vip=1, level=1, exp=0, 
        charge=0, first=0, tutorial1=0, tutorial2=0,
    }
    
    local User_Role={}
    for roleID=1,3 do
        local data=GameDataTransfer.GetRoleData(roleID)
        
        local tab={}
        tab.role_id = data.id
        tab.unlock  = data.unlock 
        tab.star    = 0
        tab.petlv   = 0
        tab.petexp  = 0
        
        tab.equip_ids,tab.equip_lvs,tab.skill_lvs = {0,0,0},{0,0,0},{1,1,1}
        
        table.insert(User_Role,tab)
    end
    PlayerDataList.Roles=User_Role

    PlayerDataList.FBs={}
    PlayerDataList.FBChapters={}
    PlayerDataList.Equips={}
    PlayerDataList.Storages={}
    PlayerDataList.VIPs={}
    PlayerDataList.Tasks={}
    
    PlayerDataList.Explorers={
        nextRefreshTime=StoreScene.GetNewDeadline(1),
        used1=0, used2=0, used3=0, playNum=1, maxNum=0, 
    }
    
    PlayerDataList.Arenas={
        rank=3000,rankMax=3000,lastTime=os.time()
    }
    
    if isBackup then GameRecorder.LoadBasicData(res_record.BASICBACKUP) end
    
    if isSave then GameRecorder.SaveBasicData() end
end

function GameRecorder.SaveBasicData(recordKey)

    local file_path = GameRecorder.GetRecordPath(recordKey or res_record.BASICDATA)
    local file=io.open(file_path,"wb")
    local function WriteData(format,...)
        file:write(string.pack(format,...))
    end
    WriteData("L",res_record.VERSION)
    local function WriteTableCount(tableList)
        local count=0
        for key,tab in pairs(tableList) do count=count+1 end
        WriteData("h",count) return count
    end
    
    local Basic=PlayerDataList.Basic
    WriteData("LLbii",Basic.coin,Basic.diamond,Basic.vip,Basic.level,Basic.exp)
    WriteData("ibii",Basic.charge,Basic.first,Basic.tutorial1,Basic.tutorial2)
    
    WriteTableCount(PlayerDataList.Roles)
    for key,tab in ipairs(PlayerDataList.Roles) do
        WriteData("bbbii",tab.role_id,tab.unlock,tab.star,tab.petlv,tab.petexp)
        for i=1,WriteTableCount(tab.equip_ids) do
            WriteData("iii",tab.equip_ids[i],tab.equip_lvs[i],tab.skill_lvs[i])
        end
    end
    
    WriteTableCount(PlayerDataList.FBs)
    for key,tab in pairs(PlayerDataList.FBs) do
        WriteData("ibi",tab.ID,tab.star,tab.stars)
    end
    WriteTableCount(PlayerDataList.FBChapters)
    for key,tab in pairs(PlayerDataList.FBChapters) do
        WriteData("ii",tab.ID,tab.stars)
    end
    WriteTableCount(PlayerDataList.Storages)
    for key,tab in pairs(PlayerDataList.Storages) do
        WriteData("ii",tab.ID,tab.qty)
    end
    WriteTableCount(PlayerDataList.Equips)
    for key,tab in pairs(PlayerDataList.Equips) do
        WriteData("ii",tab.ID,tab.equipID)
    end

    WriteTableCount(PlayerDataList.VIPs)
    for key,tab in pairs(PlayerDataList.VIPs) do
        WriteData("iLL",tab.ID,tab.exchangedtime,tab.deadline)
    end

    local Data=PlayerDataList.Explorers
    WriteData("Liiiii",Data.nextRefreshTime,Data.used1,Data.used2,Data.used3,Data.playNum,Data.maxNum)
    
    local Data=PlayerDataList.Arenas
    WriteData("Lii",Data.lastTime,Data.rank,Data.rankMax)
    
    WriteTableCount(PlayerDataList.Tasks)
    for key,tab in pairs(PlayerDataList.Tasks) do
        WriteData("iL",tab.ID,tab.datetime)
    end
    
    for i=1,10 do
        WriteData("h",0)--backup
    end
    
    file:flush() file:close()
end

function GameRecorder.LoadUserBasicData(bytes,skip)

    local Basic=PlayerDataList.Basic
    skip,Basic.coin,Basic.diamond,Basic.vip,Basic.level,Basic.exp=string.unpack(bytes,"LLbii",skip)
    skip,Basic.charge,Basic.first,Basic.tutorial1,Basic.tutorial2=string.unpack(bytes,"ibii",skip)

    local count,count2
    skip, count = GameDataReader.readShort(bytes,skip)
    for roleID=1, count do
        local tab=PlayerDataList.Roles[roleID]
        skip,tab.role_id,tab.unlock,tab.star,tab.petlv,tab.petexp=string.unpack(bytes,"bbbii",skip)
        skip, count2 = GameDataReader.readShort(bytes,skip)
        for i=1,count2 do
            skip,tab.equip_ids[i],tab.equip_lvs[i],tab.skill_lvs[i]=string.unpack(bytes,"iii",skip)
        end
    end
    skip, count = GameDataReader.readShort(bytes,skip)
    for i=1, count do
        local tab={}
        skip,tab.ID,tab.star,tab.stars=string.unpack(bytes,"ibi",skip)
        table.replace(PlayerDataList.FBs,tab.ID,tab)
    end
    skip, count = GameDataReader.readShort(bytes,skip)
    for i=1, count do
        local tab={}
        skip,tab.ID,tab.stars=string.unpack(bytes,"ii",skip)
        table.replace(PlayerDataList.FBChapters,tab.ID,tab)
    end
    skip, count = GameDataReader.readShort(bytes,skip)
    for i=1, count do
        local tab={}
        skip,tab.ID,tab.qty=string.unpack(bytes,"ii",skip)
        table.replace(PlayerDataList.Storages,tab.ID,tab)
    end
    skip, count = GameDataReader.readShort(bytes,skip)
    for i=1, count do
        local tab={}
        skip,tab.ID,tab.equipID=string.unpack(bytes,"ii",skip)
        table.replace(PlayerDataList.Equips,tab.ID,tab)
    end
    skip, count = GameDataReader.readShort(bytes,skip)
    for i=1, count do
        local tab={}
        skip,tab.ID,tab.exchangedtime,tab.deadline=string.unpack(bytes,"iLL",skip)
        table.replace(PlayerDataList.VIPs,tab.ID,tab)
    end

    local Data=PlayerDataList.Explorers
    skip,Data.nextRefreshTime,Data.used1,Data.used2,Data.used3,Data.playNum,Data.maxNum=string.unpack(bytes,"Liiiii",skip)
    
    local Data=PlayerDataList.Arenas
    skip,Data.lastTime,Data.rank,Data.rankMax=string.unpack(bytes,"Lii",skip)

    skip, count = GameDataReader.readShort(bytes,skip)
    for i=1, count do
        local tab={}
        skip,tab.ID,tab.datetime=string.unpack(bytes,"iL",skip)
        table.replace(PlayerDataList.Tasks,tab.ID,tab)
    end
    
    return true
end

function GameRecorder.LoadBasicData(recordKey)

    local file_path = GameRecorder.GetRecordPath(recordKey or res_record.BASICDATA)

    if not cc.FileUtils:getInstance():isFileExist(file_path) then return GameRecorder.DefaultBasicData(true) end
    
    local file=io.open(file_path,"rb")
    local bytes=file:read("*a")
    
    local skip,version=string.unpack(bytes,"L",1)
    outputString("load version:",version)

    local function saveDefault(isVersion)
        if PINGTEST then outputString("Load Record Error!") end
        local isBackup=(not isVersion) and (not recordKey)
        return GameRecorder.DefaultBasicData(true,isBackup)
    end
    if res_record.VERSION~=version then return saveDefault(true) end
    
    GameRecorder.DefaultBasicData()
    
    local isBackup=false
    try_catch(function() isBackup=GameRecorder.LoadUserBasicData(bytes,skip) end,saveDefault)    
    
    if isBackup and not recordKey then GameRecorder.SaveBasicData(res_record.BASICBACKUP) end
    
    file:close()
end

function GameRecorder.LoadRecords()
    GameRecorder.DefaultSystemSetting()    
    GameRecorder.LoadBasicData()
end


---------------------------------------------------------------------------------------------------


function GameRecorder.GetBasicDatas()
    return PlayerDataList.Basic
end

function GameRecorder.GetBasicData(key)
    return PlayerDataList.Basic[key] or 0
end

function GameRecorder.SetBasicData(key,value)
    PlayerDataList.Basic[key]=value
end

function GameRecorder.AddQuantity(key,value)
    GameRecorder.SetBasicData(key,GameRecorder.GetBasicData(key)+value)
end

function GameRecorder.GetRole(role_id)
    return PlayerDataList.Roles[role_id]
end

function GameRecorder.GetFBStar(ID)
    if PlayerDataList.FBs[ID] then
        return PlayerDataList.FBs[ID].star or 0
    end return 0
end

local svaule={1,10,100}
function GameRecorder.GetFBStars(ID)
    local data=PlayerDataList.FBs[ID]
    if data then 
        if data.star>=#svaule then return {true,true,true} end
        return {(data.stars%svaule[2])>=svaule[1],(data.stars%svaule[3])>=svaule[2],data.stars>=svaule[3]}
    end
    return {false,false,false}
end

function GameRecorder.GetFBChapterStars(ID)
    local data=PlayerDataList.FBChapters[ID]
    if data then 
        return {(data.stars%svaule[2])>=svaule[1],(data.stars%svaule[3])>=svaule[2],data.stars>=svaule[3]}
    end
    return {false,false,false}
end

function GameRecorder.FBChapterChange(ID,index)
    local starList=GameRecorder.GetFBChapterStars(ID)
    local stars=0
    for idx=1,#svaule do
        if starList[idx] or index==idx then stars=stars+svaule[idx] end
    end
    if PlayerDataList.FBChapters[ID] then
        PlayerDataList.FBChapters[ID].stars=stars or 0
    else
        table.replace(PlayerDataList.FBChapters,ID,{ID=ID,stars=stars or 0})
    end
end

function GameRecorder.FBChange(ID,star,stars)
    if PlayerDataList.FBs[ID] then
        PlayerDataList.FBs[ID].star=star PlayerDataList.FBs[ID].stars=stars or 0
    else
        table.replace(PlayerDataList.FBs,ID,{ID=ID,star=star,stars=stars or 0})
    end
end

function GameRecorder.FBChangeStars(ID,starList)
    local star,stars=0,0
    local starsNow=GameRecorder.GetFBStars(ID)
    for index=1,#svaule do
        if starList[index] or starsNow[index] then
            star=star+1 stars=stars+svaule[index]
        end
    end
    return GameRecorder.FBChange(ID,star,stars)
end

function GameRecorder.GetFBIsUnlock(ID)
    local gData=GameDataTransfer.GetGameFB(ID)
    if not gData then errorString("GameRecorder.GetFBIsUnlock Data Error!") return false end
    local fbID=gData.Unlock_fb_id
    if fbID>0 then
        return GameRecorder.GetFBStar(fbID)>0
    end
    return true
end

function GameRecorder.GetStorageList()
    return PlayerDataList.Storages or {}
end

function GameRecorder.StorageChange(ID,qty)
    if PlayerDataList.Storages[ID] then
        PlayerDataList.Storages[ID].qty=qty
    else
        table.replace(PlayerDataList.Storages,ID,{ID=ID,qty=qty})
    end
end

function GameRecorder.GetItemQuantity(ID)
    if PlayerDataList.Storages[ID] then
        return PlayerDataList.Storages[ID].qty or 0
    end return 0
end

function GameRecorder.ItemGain(ID,qty)
    local gData=GameDataTransfer.GetItemData(ID)
    if not gData then return end
    if gData.Type==res_item_type.COINS then
        return GameRecorder.AddQuantity("coin",qty)
    end
    if gData.Type==res_item_type.DIAMOND then
        return GameRecorder.AddQuantity("diamond",qty)
    end
    if gData.pile>0 then
        return GameRecorder.StorageChange(ID,qty+GameRecorder.GetItemQuantity(ID))
    end
    for i=1,qty or 1 do
        GameRecorder.EquipGain(ID)
    end
end

function GameRecorder.ItemGainWithAwardString(awardString)
    --outputString("GameRecorder.ItemGainWithAwardString:"..awardString)
    local awardList=GameDataTransfer.GetAwardStringData(awardString)
    for ID,data in pairs(awardList) do
        GameRecorder.ItemGain(data.ID,data.qty)
    end
    return awardList
end

function GameRecorder.GetEquipList()
    return PlayerDataList.Equips or {}
end

function GameRecorder.GetEquip(ID)
    return PlayerDataList.Equips[ID]
end

function GameRecorder.GetEquipItemID(ID)
    return PlayerDataList.Equips[ID] and PlayerDataList.Equips[ID].equipID or 0
end

function GameRecorder.RemoveEquip(ID)
    PlayerDataList.Equips[ID]=nil
end

function GameRecorder.EquipsChange(ID,equipID)
    if equipID and equipID>0 then
        return table.replace(PlayerDataList.Equips,ID,{ID=ID,equipID=equipID})
    end
    GameRecorder.RemoveEquip(ID)
end

function GameRecorder.GetEquipNewID()
    for i=1,1000 do
        if not PlayerDataList.Equips[i] then return i end
    end
end

function GameRecorder.EquipGain(equipID)
    local ID=GameRecorder.GetEquipNewID()
    if ID then GameRecorder.EquipsChange(ID,equipID) end
end

function GameRecorder.GetRoleAttributes(roleID)
    local data=GameDataTransfer.GetRoleData(roleID)
    local Attack,Defence,Blood,Crit = data.Attack,data.Defence,data.life,data.cri_rate
    local function AddAttribute(data)
        if not data then return end
        Attack,Defence,Blood,Crit = (Attack+data.power),(Defence+data.defence),(Blood+data.blood),(Crit+data.crit)
    end
    local level=GameRecorder.GetBasicData("level")
    AddAttribute(GameDataTransfer.GetRoleLevelData(level))
    
    local role=GameRecorder.GetRole(roleID)
    for i=1,3 do
        local equipID=role.equip_ids[i]
        if equipID>0 then
            AddAttribute(GameDataTransfer.GetItemData(equipID))
            AddAttribute(GameDataTransfer.GetItemLevelData(equipID,role.equip_lvs[i]))
        end
        local skillID=role.skill_lvs[i]
    end
    if role.petlv>0 then
        AddAttribute(GameDataTransfer.GetPetLevelData(role.petlv))
    end
    if role.star>0 then
        AddAttribute(GameDataTransfer.GetPetStarData(roleID,role.star))
    end
    return Attack,Defence,Blood,Crit
end

function GameRecorder.GetAttributeCombatPower(Attack,Defence,Blood)
    return Attack+math.floor((Defence+Blood)*0.5)
end

function GameRecorder.GetRoleCombatPower(roleID)
    local Attack,Defence,Blood = GameRecorder.GetRoleAttributes(roleID)
    local combatPower=GameRecorder.GetAttributeCombatPower(Attack,Defence,Blood)
    for key,data in ipairs(GameRecorder.GetRoleSkills(roleID)) do
        local lvData=GameDataTransfer.GetActiveSkillValue(data.ID,data.lv)
        if lvData then combatPower=(combatPower+lvData.combat_power) end
    end
    return combatPower
end

function GameRecorder.GetAmountCombatPower()
    local combatPower=0
    for roleID=1,3 do
        local data=GameDataTransfer.GetRoleData(roleID)
        if data.unlock>0 then
            combatPower=combatPower+GameRecorder.GetRoleCombatPower(roleID)
        end
    end
    return combatPower
end

function GameRecorder.GetRoleSkills(roleID,sLv)
    local data=GameDataTransfer.GetRoleData(roleID)
    local role=GameRecorder.GetRole(roleID)
    local result={}
    local ids=string.split(data.SkilID,",")
    for key,var in ipairs(ids) do
        local skillID,level=tonumber(var),sLv or role.skill_lvs[key]
        local activeSkillValue=GameDataTransfer.GetActiveSkillValue(skillID,math.max(1,level))
        if activeSkillValue then
            table.insert(result,{ID=skillID,lv=level,skillID=activeSkillValue.skillrefid})
        end
    end
    return result
end

function GameRecorder.GetRoleEquipments(roleID)
    local role=GameRecorder.GetRole(roleID)
    local result={}
    for i=1,3 do
        table.insert(result,{ID=role.equip_ids[i],lv=role.equip_lvs[i]})
    end
    return result
end

function GameRecorder.GetTaskData(ID)
    return PlayerDataList.Tasks[ID]
end

function GameRecorder.SetTaskData(ID)
    PlayerDataList.Tasks[ID]={ID=ID,datetime=os.time()}
end









