
require "tools.SoundMg" 

GameSound = {}

SoundType = {
	--BGM
	BG_MAIN=100,
	BG_FIGHT=101,
	BG_WIN=102,
	BG_LOST=103,
	BG_LOGIN=104,
	BG_BOSS=312,
	BG_FIGHT1=401,
	BG_FIGHT2=402,
	BG_FIGHT3=403,
    BG_FIGHT4=405,
    BOSS_FIGHT=406,
    BOSS_FIGHT2=407,
	
	EF_ROLL=10503,
}

SoundEffect = {
    UI_CLICK = "anniu",
    EF_STEP1 = "shlpt1",
    EF_STEP2 = "",
    EF_STEP3 = "",
    EF_ROLL  = "10503",
    EF_DEFEN = "10513",
    EF_HURT  = "",
    EF_COMBOSKILL="10519",
    EF_BOOM  = "20000",
    EF_SHOUT = "wxl4",
    EF_QTE1  = "30000",
    EF_QTE2  = "",
    EF_QTE3  = "",
}

local PreloadEffectList={}
local PreloadInGameEffectList={}
local tempEffect = {}
local Playing_SoundID = nil
local SoundMap = {}
local musicEnable,effectEnable=true,true

function GameSound.GetEnable()
	return musicEnable,effectEnable
end

function GameSound.SetEnable(music,effect)

	musicEnable,effectEnable=music,effect
	
	if musicEnable then
		--SoundMG_ResumeMusic()
		GameSound.PlayMusic(SoundType.BG_MAIN)
	else
		--SoundMG_PauseMusic()
		SoundMG_StopMusic(false)
	end

	if effectEnable then
	else
		SoundMG_StopAllEffects()
	end
end

function GameSound.setupSoundMap()   --设置基础的音效设置表
    for key,value in pairs(GameData_List.Game_Sounds) do
        local tempSound = {}
        tempSound.one = Skill.StrA2NumberList(string.split(value.Attack_Sounds,","))
        tempSound.sec = Skill.StrA2NumberList(string.split(value.Hurt_Sounds,","))
        SoundMap[value.id] = tempSound
    end
end

function GameSound.playGameEffectAttack(map_id)
    --if map_id == 0 or map_id == nil then return end
    if map_id == 0  then return end
    local nowPro = SoundMap[map_id]
    
    if nowPro == nil then
        if map_id then
            cclog("can't find sound with sound_id = "..map_id)
        else
            cclog("no sound init and call it")
        end
        return
    end
    
    if nowPro.one.last > 1 then
        local allweight = nowPro.one.Last * 30
        local nowWeight = Random(0,allweight)
        local nowIndex = math.floor(nowWeight/30)
        if nowWeight%30 ~= 0 then nowIndex = nowIndex + 1 end
        if nowIndex == 0  then nowIndex = 1 end
        GameSound.PlayGameEffect(nowPro.one[nowIndex])
    elseif nowPro.one.last > 0 then
        GameSound.PlayGameEffect(nowPro.one[1])
    end
end

function GameSound.playGameEffectHurt(map_id)
    if map_id == 0 or map_id == nil then return end
    local nowPro = SoundMap[map_id]
    if nowPro == nil then
        -- cclog("SoundMap[%d] can't find",map_id)
        return
    end
    if nowPro.sec.last > 1 then
        local allweight = nowPro.sec.Last * 30
        local nowWeight = Random(0,allweight) 
        local nowIndex = math.floor(nowWeight/30)
        if nowWeight%30 ~= 0 then nowIndex = nowIndex + 1 end
        if nowIndex == 0  then nowIndex = 1 end
        GameSound.PlayGameEffect(nowPro.sec[nowIndex])
    elseif nowPro.sec.last > 0 then
        GameSound.PlayGameEffect(nowPro.sec[1])
    end
end

function GameSound.GetMusic(soundID)
	return "Sounds/BGM/"..soundID..".mp3"
end

function GameSound.GetEffect(soundID)
	return "Sounds/Effects/"..soundID..".mp3"
end

function GameSound.PlayMusic(soundID)
	if not musicEnable then return end
	if SoundMG_IsMusicPlaying() then
		if Playing_SoundID==soundID then return end
		GameSound.StopMusic()
	end
	Playing_SoundID=soundID
	local loop = (soundID~=SoundType.BG_WIN and soundID~=SoundType.BG_LOST)
    SoundMG_PlayMusic(GameSound.GetMusic(soundID),loop)
end

function GameSound.StopMusic()
	Playing_SoundID=nil
	SoundMG_StopMusic(true)
end

function GameSound.PlayEffect(soundID,loop)
    if not effectEnable or not soundID then return end
    if not cc.FileUtils:getInstance():isFileExist(GameSound.GetEffect(soundID)) then return end
	if not PreloadEffectList[soundID] then
		GameSound.PreloadEffect(soundID)
	end
    return SoundMG_PlayEffect(PreloadEffectList[soundID],loop)
end

function GameSound.PreloadEffect(soundID)
	local filename=GameSound.GetEffect(soundID)
	PreloadEffectList[soundID]=filename
	SoundMG_PreloadEffect(filename)
end

function GameSound.UnloadEffect(soundID)
	if not PreloadEffectList[soundID] then return end
	SoundMG_UnloadEffect(PreloadEffectList[soundID])
	PreloadEffectList[soundID]=nil
end

function GameSound.UnloadEffects(filelist)
	for key, var in pairs(filelist) do
		if var then
			GameSound.UnloadEffect(key)
		end
	end
end



function GameSound.PreloadAllEffects()
	--[[local SoundList={
		200,201,202,300,301,302,303,304,305,306,307,308,309,310,311,350,351,400
	}--]]
    for key, var in pairs(SoundEffect) do
        GameSound.PreloadEffect(var)
	end
end


function GameSound.UnloadAllEffect()
	GameSound.UnloadEffects(PreloadEffectList)
end

-------------------------预加载战斗中音效---------------------------------

function GameSound.PreloadGameEffect(soundID)
    local filename=GameSound.GetEffect(soundID)
    PreloadInGameEffectList[soundID]=filename
    SoundMG_PreloadEffect(filename)
end

function GameSound.UnloadGameEffect(soundID)
    if not PreloadInGameEffectList[soundID] then return end
    SoundMG_UnloadEffect(PreloadInGameEffectList[soundID])
    PreloadInGameEffectList[soundID]=nil
end

function GameSound.PreloadAllGameEffects() --载入游戏中音效ID表 并且清空暂时表

    for key, var in pairs(tempEffect) do
        GameSound.PreloadGameEffect(key)
    end

    table.clean(tempEffect)
end

function GameSound.UnloadGameEffects(filelist)
    for key, var in pairs(filelist) do
        if var then
            GameSound.UnloadGameEffect(key)
        end
    end
end

function GameSound.UnloadAllGameEffect()
    GameSound.UnloadGameEffects(PreloadInGameEffectList)
end



function GameSound.initGamesoundWithMapId(map_id) --根据音效MAPid 来生成暂时的音效ID表
    local nowSound = SoundMap[map_id]
    if nowSound ~= nil then
        local as = nowSound.one
        local hs = nowSound.sec

        for i=1,List.getSize(as) do
            tempEffect[as[i]] = 1
        end

        for i=1,List.getSize(hs) do
            tempEffect[hs[i]] = 1
        end
    end
end

function GameSound.initGamesoundWithSkill(m_skill) --根据音效MAPid 来生成暂时的音效ID表
    if m_skill == nil then return end
    for i=1,List.getSize(m_skill.sound_list) do
        if m_skill.sound_list[i] ~= 0 then
            GameSound.initGamesoundWithMapId(m_skill.sound_list[i])
        end
    end
end

function GameSound.initGamesoundWithSound_ids(Sound_ids) --根据音效MAPid 来生成暂时的音效ID表
    local Ids = string.split(Sound_ids,",")
    
    for i=1,#Ids do
        local s_index = tonumber(Ids[i])
        if s_index ~=nil and s_index ~= 0 then
            GameSound.initGamesoundWithMapId(s_index)
        end
    end
end

function GameSound.initGamesoundWithSkill_IDS(m_skill_ids) --根据技能音效ID 来生成暂时的音效ID表
    local Ids = string.split(m_skill_ids,",")
    
    for i=1,#Ids do
        local s_index = tonumber(Ids[i])
        local tempSkill = GameData_List.skill[s_index]
        if tempSkill~= nil and tempSkill~= 0 then
            GameSound.initGamesoundWithSound_ids(tempSkill.sounds)
        end
        
    end
end

function GameSound.PlayGameEffect(soundID)
    if not effectEnable then return end
    if not PreloadInGameEffectList[soundID] then
        GameSound.PreloadGameEffect(soundID)
    end
    SoundMG_PlayEffect(PreloadInGameEffectList[soundID])
end
------------------------------------------------------------------------------------

function GameSound.PauseAllSounds()
	SoundMG_PauseMusic()
	SoundMG_PauseAllEffects()
end

function GameSound.ResumeAllSounds()
	SoundMG_ResumeMusic()
	SoundMG_ResumeAllEffects()
end

function GameSound.StopAllSounds()
	if SoundMG_IsMusicPlaying() then
		SoundMG_StopMusic(true)
	end
	SoundMG_StopAllEffects()
end
