
table.copy = function(t_add,t_to)
    for key, value in pairs(t_add) do  
		if type(value) == "table" then
			t_to[key]={}
			table.copy(value,t_to[key])
		else
			t_to[key]=value
        end
    end
    return t_to
end

table.enum = function(table, index)
    local enum_table = {} 
    local enum_index = index or 0 
    for i, v in ipairs(table) do 
        enum_table[v] = enum_index + (i-1) 
    end 
    return enum_table 
end

table.print = function(tb,display_string)
    
    if display_string then
        print(display_string)
    end
    
    for i,v in pairs(tb) do
        if type(v) == "function" then
            print("function:",i)
        elseif type(v) == "table" then
            print(i,"{")
            table.print(v)
            print("--------------}")
        elseif type(v) == "boolean" then
            print(i.."=",v)
        else
            print(i.."="..v)
        end
    end
end

table.clean = function(tb)
	if (not tb) then return end
	local key=next(tb)
	while key do
		tb[key]=nil
		key=next(tb)
	end
end

table.clear = function(tb)

    if (not tb) then return end
	
    local key=next(tb)
	while key do
		if type(tb[key]) == "table" then
			table.clear(tb[key])
		end
		tb[key]=nil
        key=next(tb)
    end
end

table.delete = function(tb)

	if (not tb) then return end
	
	table.clear(tb)
	
	tb=nil
end

table.reverse = function(tb)
	local revTable={}
	for key,value in pairs(tb) do
		revTable[value]=key
	end
	return revTable
end

table.replace = function(tb,key,var)
    tb[key]=var
end

table.weak = function(tb,mode)
	setmetatable(tb,{__mode=mode})
end

string.split = function(str, split_char)      
    local sub_str_tab = {};  

    while (true) do          
        local pos = string.find(str, split_char);    
        if (not pos) then              
            local size_t = table.getn(sub_str_tab)  
            table.insert(sub_str_tab,size_t+1,str);  
            break;    
        end  

        local sub_str = string.sub(str, 1, pos - 1);                
        local size_t = table.getn(sub_str_tab)  
        table.insert(sub_str_tab,size_t+1,sub_str);  
        local t = string.len(str);  
        str = string.sub(str, pos + string.len(split_char), t);     
    end      
    return sub_str_tab;  
end 

enum = function(table)
    local enum_table = {}
    local enum_index = 0
    for i, str in ipairs(table) do
        local pos = string.find(str,"=")
        if pos then
            local sub_str = string.sub(str, 1, pos - 1)
            local t = string.len(str);  
            str = string.sub(str, pos + 1, t)
            enum_table[sub_str]=tonumber(str)
            enum_index=enum_table[sub_str]
        else
            enum_table[str]=enum_index+1
            enum_index=enum_table[str]
        end
    end
    return enum_table 
end

Time_GetCurrentTime = function()
	return os.time()
end
Time_GetDelayTime = function(start)
	return os.difftime(Time_GetCurrentTime(),start)
end

Time_GetCPUTime = function()
	return os.clock()
end
Time_GetCPUDelayTime = function(start)
	return Time_GetCPUTime()-start
end

local mathrandomseed =math.randomseed
local mathrandom = math.random

mathrandomseed(os.time())
mathrandom()

Random = function(m,n)
   return mathrandom(m,n)
end

local osClock = os.clock
setRandomSeed = function()
	mathrandomseed(osClock())
end

FormatTime = function(second)   --with format hh:mm:ss
    local fixTime = (24 - os.date("*t",0).hour) *3600
    second = math.max(second,0)
    return os.date("%X",second + fixTime )
end
