--require "cocos.cocos2d.Cocos2d"

--global var
camera =nil
logicSpeed = 1

CC_CONTENT_SCALE_FACTOR = function()
    return cc.Director:getInstance():getContentScaleFactor()
end


CC_POINT_PIXELS_TO_POINTS = function(pixels)
    return cc.p(pixels.x/CC_CONTENT_SCALE_FACTOR(), pixels.y/CC_CONTENT_SCALE_FACTOR())
end

CC_POINT_POINTS_TO_PIXELS = function(points)
    return cc.p(points.x*CC_CONTENT_SCALE_FACTOR(), points.y*CC_CONTENT_SCALE_FACTOR())
end

function  setMotionSpeed(speed) --设置快慢动作
    logicSpeed=speed
    cc.Director:getInstance():getScheduler():setTimeScale(speed) 
end

function  resetMotionSpeed() --动作速度恢复
    logicSpeed=1
    cc.Director:getInstance():getScheduler():setTimeScale(1) 
end

local pIsLineIntersect = cc.pIsLineIntersect

function pGetIntersectXY(pt1,pt2,pt3,pt4)
    local s,t, ret = 0,0,false
    ret,s,t = pIsLineIntersect(pt1,pt2,pt3,pt4,s,t)
    
    
    if ret and  s >= 0.0 and s <= 1.0 and t >= 0.0 and t <= 1.0 then
        return (pt1.x + s * (pt2.x - pt1.x)), (pt1.y + s * (pt2.y - pt1.y))
    else
        return 0,0
    end
end
--print table
function printTab(tab)
    for i,v in pairs(tab) do
        if type(v) == "table" then
            print("table",i,"{")
            printTab(v)
            print("}")
        else
            print(v)
        end
    end
end



function print_lua_table (lua_table, indent)

    --print(debug.traceback())
    indent = indent or 0
    for k, v in pairs(lua_table) do
        if type(k) == "string" then
            k = string.format("%q", k)
        end
        local szSuffix = ""
        if type(v) == "table" then
            szSuffix = "{"
        end
        local szPrefix = string.rep("    ", indent)
        formatting = szPrefix.."["..k.."]".." = "..szSuffix
        if type(v) == "table" then
            cclog(formatting)
            print_lua_table(v, indent + 1)
            cclog(szPrefix.."},")
        else
            local szValue = ""
            if type(v) == "string" then
                szValue = string.format("%q", v)
            else
                szValue = tostring(v)
            end
            cclog(formatting..szValue..",")
        end
    end
end


--radiansNormalizer
function radNormalize(rad)
    local pi2 = 2*math.pi
    rad = rad % pi2
    rad = (rad + pi2)%pi2
    if rad > math.pi then
        rad = rad - math.pi
    end
    return rad
end

-- 获取镜头的X,Y点
--curPos = 当前镜头坐标 cc.P
--targetPos = 目标坐标 cc.P
--broder = 边界XY cc.P
--speed = 移动速度
--min = 最小移动速度

function getCamNextXY(curPos,targetPos,broder,speed,min)
    local resultPos = cc.p(curPos.x,curPos.y)
    if speed<min then speed=min end
    
    if curPos.x>targetPos.x then
        resultPos.x=curPos.x-speed
        if resultPos.x<targetPos.x then resultPos.x=targetPos.x end
        if resultPos.x<broder.x then resultPos.x=broder.x end
    else 
        resultPos.x=curPos.x+speed
        if resultPos.x>targetPos.x then resultPos.x=targetPos.x end
        if resultPos.x>broder.x then resultPos.x=broder.x end
    end
    
    if curPos.y>targetPos.y then
        resultPos.y=curPos.y-speed
        broder.y = 0
        if resultPos.y<targetPos.y then resultPos.y=targetPos.y end
        if resultPos.y<broder.y then resultPos.y=broder.y end
    else 
        resultPos.y=curPos.y+speed
        broder.y = 1280-720
        if resultPos.y>targetPos.y then resultPos.y=targetPos.y end
        if resultPos.y>broder.y then resultPos.y=broder.y end
    end
    return {x= resultPos.x,y=resultPos.y} 
end

-- getpostable
function getPosTable(obj)
    local posX,posY = obj:getPosition()
    return {x= posX,y=posY} 
end

--getnextpos
function getNextStepPos(curPos, targetPos, speed, dt)
    local angel = math.atan2(targetPos.y-curPos.y,targetPos.x-curPos.x)
    return {x = curPos.x+math.cos(angel)*speed*dt, y = curPos.y+math.sin(angel)*speed*dt}
end

--createAnimationStruct
function createAnimationStruct(var1, var2, var3)
    local timerange = {begin = var1, ended = var2, speed=var3}
    return timerange
end
function createAnimation(file, begin, finish, speed)
    --TODO: we don't need to create the same Animation3D all the time
    local animation3d = cc.Animation3D:create(file)
    local animate3d = cc.Animate3D:create(animation3d, begin/30,(finish-begin)/30)
    animate3d:setSpeed(speed)
    animate3d:retain()
    return animate3d
end

--createKnockedMessageStruct
function createKnockedMsgStruct(object)
    local msgStruct = {attacker = object, target = object._target}
    return msgStruct
end

-- cclog
cclog = function(...)
    print(string.format(...))
end

-- change table to enum type
function CreateEnumTable(tbl, index)
    local enumTable = {}
    local enumIndex = index or -1
    for i, v in ipairs(tbl) do
        enumTable[v] = enumIndex + i
    end
    return enumTable
end

function removeAll(table)
    while true do
        local k =next(table)
        if not k then break end
        table[k] = nil
    end
end

List = {}
function List.new()
    return {first = 1, last = 0}
end

function List.pushfirst(list, value)
    local first = list.first - 1
    list.first = first
    list[first] = value
end

function List.pushlast(list, value)
    local last = list.last + 1
    list.last = last
    list[last] = value
end

function List.popfirst(list)
    local first = list.first
    if first > list.last then return nil end
    local value = list[first]
    list[first] = nil
    list.first = first + 1
    return value
end

function List.poplast(list)
    local last = list.last
    if list.first > last then return nil end
    local value = list[last]
    list[last] = nil
    list.last = last - 1
    return value    
end

function List.removeAll(list)
    removeAll(list)
    list.first = 1
    list.last = 0
end

function List.getSize(list)
    return list.last - list.first + 1
end

function List.first(list)
    local value = nil
    if list.first <= list.last then
        value = list[first]
    end

    return value
end

function List.remove(list, index)
    if index < list.first or index > list.last then return end

    while index <= list.last do
        list[index] = nil
        list[index] = list[index+1]
        index = index + 1
    end

    list.last = list.last -1
end

function List.removeObj(list, obj)
    if obj == nil or List.getSize(list) == 0 then return end

    for index=list.first, List.getSize(list) do
        if list[index] == obj then
            List.remove(list,index)
            break
        end
    end    
end

function copyTable(t1, t2)
    table.copy(t1, t2)
--    for key, var in pairs(t1) do
--        t2[key] = var
--    end
end

function delayExecute(target, func, delay)
    local wait = cc.DelayTime:create(delay)
    target:runAction(cc.Sequence:create(wait, cc.CallFunc:create(func)))
end

function DEGREES_TO_RADIANS(__ANGLE__) 
    return __ANGLE__ * 0.01745329252
end
function RADIANS_TO_DEGREES(__ANGLE__)
    return __ANGLE__ * 57.29577951
end

-- 获取基类的某个方法
-- table C++类或者lua table
-- methodName 函数名，也可以是成员变量名
-- return 基类的函数或成员变量值（如果methodName为变量名）
--          nil 表示找不到
function getSuperMethod(table, methodName)
    local mt = getmetatable(table)
    local method = nil
    while mt and not method do
        method = mt[methodName]
        if not method then
            local index = mt.__index
            if index and type(index) == "function" then
                method = index(mt, methodName)
            elseif index and type(index) == "table" then
                method = index[methodName]
            end
        end
        mt = getmetatable(mt)
    end
    return method
end



vertDefaultSource = "\n"..
    "attribute vec4 a_position;\n" ..
    "attribute vec2 a_texCoord;\n" ..
    "attribute vec4 a_color;\n" ..
    "#ifdef GL_ES\n" .. 
    "varying lowp vec4 v_fragmentColor;\n" ..
    "varying mediump vec2 v_texCoord;\n" ..
    "#else\n" ..
    "varying vec4 v_fragmentColor;\n" ..
    "varying vec2 v_texCoord;\n" ..
    "#endif\n" ..
    "void main()\n" ..
    "{\n" .. 
    "gl_Position = (CC_PMatrix * CC_MVMatrix) * a_position;\n"..
    "v_fragmentColor = a_color;\n"..
    "v_texCoord = a_texCoord;\n" ..
    "}"
    
vertDefaultSource_CCUI = "\n"..
    "attribute vec4 a_position;\n" ..
    "attribute vec2 a_texCoord;\n" ..
    "attribute vec4 a_color;\n" ..
    "#ifdef GL_ES\n" .. 
    "varying lowp vec4 v_fragmentColor;\n" ..
    "varying mediump vec2 v_texCoord;\n" ..
    "#else\n" ..
    "varying vec4 v_fragmentColor;\n" ..
    "varying vec2 v_texCoord;\n" ..
    "#endif\n" ..
    "void main()\n" ..
    "{\n" .. 
    "gl_Position = CC_PMatrix  * a_position;\n"..
    "v_fragmentColor = a_color;\n"..
    "v_texCoord = a_texCoord;\n" ..
    "}"

vertDefaultSourceJPG = "\n"..
    "attribute vec4 a_position;\n" ..
    "attribute vec2 a_texCoord;\n" ..
    "attribute vec3 a_color;\n" ..
    "#ifdef GL_ES\n" .. 
    "varying lowp vec3 v_fragmentColor;\n" ..
    "varying mediump vec2 v_texCoord;\n" ..
    "#else\n" ..
    "varying vec3 v_fragmentColor;\n" ..
    "varying vec2 v_texCoord;\n" ..
    "#endif\n" ..
    "void main()\n" ..
    "{\n" .. 
    "gl_Position = (CC_PMatrix * CC_MVMatrix) * a_position;\n"..
    "v_fragmentColor = a_color;\n"..
    "v_texCoord = a_texCoord;\n" ..
    "}"

vertDefaultSource_CCUI_JPG = "\n"..
    "attribute vec4 a_position;\n" ..
    "attribute vec2 a_texCoord;\n" ..
    "attribute vec3 a_color;\n" ..
    "#ifdef GL_ES\n" .. 
    "varying lowp vec3 v_fragmentColor;\n" ..
    "varying mediump vec2 v_texCoord;\n" ..
    "#else\n" ..
    "varying vec3 v_fragmentColor;\n" ..
    "varying vec2 v_texCoord;\n" ..
    "#endif\n" ..
    "void main()\n" ..
    "{\n" .. 
    "gl_Position = CC_PMatrix  * a_position;\n"..
    "v_fragmentColor = a_color;\n"..
    "v_texCoord = a_texCoord;\n" ..
    "}"

function setProgramPrp(pProgram)
    pProgram:bindAttribLocation(cc.ATTRIBUTE_NAME_POSITION,cc.VERTEX_ATTRIB_POSITION)
    pProgram:bindAttribLocation(cc.ATTRIBUTE_NAME_COLOR,cc.VERTEX_ATTRIB_COLOR)
    pProgram:bindAttribLocation(cc.ATTRIBUTE_NAME_TEX_COORD,cc.VERTEX_ATTRIB_FLAG_TEX_COORDS)
    pProgram:link()
    pProgram:updateUniforms()
end

function preLoadResource()

    local JPG_goldScale = cc.FileUtils:getInstance():getStringFromFile("shader/JPG_goldScale.fsh")
    local PNG_goldScale = cc.FileUtils:getInstance():getStringFromFile("shader/PNG_goldScale.fsh")

    local pProgramJPG =  cc.GLProgram:createWithByteArrays(vertDefaultSourceJPG,JPG_goldScale) --jpg
    local pProgramPNG =  cc.GLProgram:createWithByteArrays(vertDefaultSource,PNG_goldScale) --png
    setProgramPrp(pProgramJPG)
    setProgramPrp(pProgramPNG)
    cc.GLProgramCache:getInstance():addGLProgram(pProgramJPG,"JPGgoldScaleProgram")
    cc.GLProgramCache:getInstance():addGLProgram(pProgramPNG,"PNGgoldScaleProgram")
-----------------------------------------------------------------------------------------------------------------    
    local JPG_LightScale = cc.FileUtils:getInstance():getStringFromFile("shader/JPG_LightScale.fsh")
    local PNG_LightScale = cc.FileUtils:getInstance():getStringFromFile("shader/PNG_LightScale.fsh")

    local pProgramJPG =  cc.GLProgram:createWithByteArrays(vertDefaultSourceJPG,JPG_LightScale) --jpg
    local pProgramPNG =  cc.GLProgram:createWithByteArrays(vertDefaultSource,PNG_LightScale) --png
    setProgramPrp(pProgramJPG)
    setProgramPrp(pProgramPNG)
    cc.GLProgramCache:getInstance():addGLProgram(pProgramJPG,"JPGLightScaleProgram")
    cc.GLProgramCache:getInstance():addGLProgram(pProgramPNG,"PNGLightScaleProgram")

    local pProgramJPG_CCUI =  cc.GLProgram:createWithByteArrays(vertDefaultSource_CCUI_JPG,JPG_LightScale) --jpg
    local pProgramPNG_CCUI =  cc.GLProgram:createWithByteArrays(vertDefaultSource_CCUI,PNG_LightScale) --png
    setProgramPrp(pProgramJPG_CCUI)
    setProgramPrp(pProgramPNG_CCUI)
    cc.GLProgramCache:getInstance():addGLProgram(pProgramJPG_CCUI,"JPGLightScaleProgram_CCUI")
    cc.GLProgramCache:getInstance():addGLProgram(pProgramPNG_CCUI,"PNGLightScaleProgram_CCUI")
    
---------------------------------------------------------------------------------------------------------
    local JPG_default = cc.FileUtils:getInstance():getStringFromFile("shader/JPG_default.fsh")
    local PNG_default = cc.FileUtils:getInstance():getStringFromFile("shader/PNG_default.fsh")

    local pProgramJPG =  cc.GLProgram:createWithByteArrays(vertDefaultSourceJPG,JPG_default) --jpg
    local pProgramPNG =  cc.GLProgram:createWithByteArrays(vertDefaultSource,PNG_default) --png
    setProgramPrp(pProgramJPG)
    setProgramPrp(pProgramPNG)
    cc.GLProgramCache:getInstance():addGLProgram(pProgramJPG,"JPGdefaultProgram")
    cc.GLProgramCache:getInstance():addGLProgram(pProgramPNG,"PNGdefaultProgram")
    
    --CCUI
    local pProgramJPG_CCUI =  cc.GLProgram:createWithByteArrays(vertDefaultSource_CCUI_JPG,JPG_default) --jpg
    local pProgramPNG_CCUI =  cc.GLProgram:createWithByteArrays(vertDefaultSource_CCUI,PNG_default) --png
    setProgramPrp(pProgramJPG_CCUI)
    setProgramPrp(pProgramPNG_CCUI)
    cc.GLProgramCache:getInstance():addGLProgram(pProgramJPG_CCUI,"JPGdefaultProgram_CCUI")
    cc.GLProgramCache:getInstance():addGLProgram(pProgramPNG_CCUI,"PNGdefaultProgram_CCUI")
    
 ----------------------------------------------------------------------------------------------

    local JPG_greyScale = cc.FileUtils:getInstance():getStringFromFile("shader/JPG_greyScale.fsh")
    local PNG_greyScale = cc.FileUtils:getInstance():getStringFromFile("shader/PNG_greyScale.fsh")
    
    local pProgramJPG =  cc.GLProgram:createWithByteArrays(vertDefaultSourceJPG,JPG_greyScale) --jpg
    local pProgramPNG =  cc.GLProgram:createWithByteArrays(vertDefaultSource,PNG_greyScale) --png
    setProgramPrp(pProgramJPG)
    setProgramPrp(pProgramPNG)
    cc.GLProgramCache:getInstance():addGLProgram(pProgramJPG,"JPGgreyScaleProgram")
    cc.GLProgramCache:getInstance():addGLProgram(pProgramPNG,"PNGgreyScaleProgram")
    
    --CCUI
    local pProgramJPG_CCUI =  cc.GLProgram:createWithByteArrays(vertDefaultSource_CCUI_JPG,JPG_greyScale) --jpg
    local pProgramPNG_CCUI =  cc.GLProgram:createWithByteArrays(vertDefaultSource_CCUI,PNG_greyScale) --png
    setProgramPrp(pProgramJPG_CCUI)
    setProgramPrp(pProgramPNG_CCUI)
    cc.GLProgramCache:getInstance():addGLProgram(pProgramJPG_CCUI,"JPGgreyScaleProgram_CCUI")
    cc.GLProgramCache:getInstance():addGLProgram(pProgramPNG_CCUI,"PNGgreyScaleProgram_CCUI")
    
-----------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------
    local vertSource = cc.FileUtils:getInstance():getStringFromFile("shader/example_ColorBars.vsh")
    local fragSource = cc.FileUtils:getInstance():getStringFromFile("shader/example_ColorBars.fsh")
    local pProgramColorBars =  cc.GLProgram:createWithByteArrays(vertSource,fragSource)
    setProgramPrp(pProgramColorBars)
    cc.GLProgramCache:getInstance():addGLProgram(pProgramColorBars,"exampleColorBarsProgram")
end

function colorNode(node)
    local pProgram = cc.GLProgramCache:getInstance():getGLProgram("exampleColorBarsProgram")
    if nil ~= pProgram then
        node:setGLProgram(pProgram)
    end
end

function setProgremNodeWithCCUI(node,pProgram,isSprite)
	if isSprite then
		node:setGLProgram(pProgram)
	else
		node:getVirtualRenderer():getSprite():setGLProgram(pProgram)
	end
end

function greyNodeWithCCUI(node,isJpg,isSprite)
    local pProgram = nil
    if isJpg == true then
        pProgram = cc.GLProgramCache:getInstance():getGLProgram("JPGgreyScaleProgram_CCUI")
    else  
        pProgram = cc.GLProgramCache:getInstance():getGLProgram("PNGgreyScaleProgram_CCUI")
    end
	if nil ~= pProgram then
		setProgremNodeWithCCUI(node,pProgram,isSprite)
    end
end
function greyNode(node,isJpg)

    local pProgram = nil
    if isJpg == true then 
        pProgram = cc.GLProgramCache:getInstance():getGLProgram("JPGgreyScaleProgram")
    else 
        pProgram = cc.GLProgramCache:getInstance():getGLProgram("PNGgreyScaleProgram")
    end
    if nil ~= pProgram then
        node:setGLProgram(pProgram)
    end

end

function lightNodeWithCCUI(node,isJpg,isSprite)
    local pProgram = nil
    if isJpg == true then 
        pProgram = cc.GLProgramCache:getInstance():getGLProgram("JPGLightScaleProgram_CCUI")
    else  
        pProgram = cc.GLProgramCache:getInstance():getGLProgram("PNGLightScaleProgram_CCUI")
    end
    if nil ~= pProgram then
		setProgremNodeWithCCUI(node,pProgram,isSprite)
    end
end

function lightNode(node,isJpg)
    local pProgram = nil
    if isJpg == true then 
        pProgram = cc.GLProgramCache:getInstance():getGLProgram("JPGLightScaleProgram")
    else  
        pProgram = cc.GLProgramCache:getInstance():getGLProgram("PNGLightScaleProgram")
    end
    if nil ~= pProgram then
        node:setGLProgram(pProgram)
    end
end

function goldNode(node,isJpg)
    local pProgram = nil
    if isJpg == true then 
        pProgram = cc.GLProgramCache:getInstance():getGLProgram("JPGgoldScaleProgram")
    else  
        pProgram = cc.GLProgramCache:getInstance():getGLProgram("PNGgoldScaleProgram")
    end
    if nil ~= pProgram then
        node:setGLProgram(pProgram)
    end
end

function defaultNode(node,isJpg)
    local pProgram = nil
    
    if isJpg == true then
        pProgram = cc.GLProgramCache:getInstance():getGLProgram("JPGdefaultProgram")
    else 
        pProgram =cc.GLProgramCache:getInstance():getGLProgram(cc.SHADER_POSITION_TEXTURE_COLOR)
        --pProgram = cc.GLProgramCache:getInstance():getGLProgram("PNGdefaultProgram")
    end
    if nil ~= pProgram then
        node:setGLProgram(pProgram)
    end
end

function defaultNodeWithCCUI(node,isJpg,isSprite)
    local pProgram = nil
    if isJpg == true then 
        pProgram = cc.GLProgramCache:getInstance():getGLProgram("JPGdefaultProgram_CCUI")
    else  
        pProgram = cc.GLProgramCache:getInstance():getGLProgram("PNGdefaultProgram_CCUI")
    end
	if nil ~= pProgram then
		setProgremNodeWithCCUI(node,pProgram,isSprite)
    end
end

function greyWithCCButton(button_node,isJpg)
    
    local button_normal = button_node:getVirtualRenderer():getSprite()
    button_node:setBrightStyle(ccui.BrightStyle.highlight)
    local button_Hight = button_node:getVirtualRenderer():getSprite()
    button_node:setBrightStyle(ccui.BrightStyle.normal)
    
    greyNodeWithCCUI(button_normal,isJpg,true)
	greyNodeWithCCUI(button_Hight,isJpg,true)
    
end

function lightWithCCButton(button_node,isJpg)
   
    local button_normal = button_node:getVirtualRenderer():getSprite()
    button_node:setBrightStyle(ccui.BrightStyle.highlight)
    local button_Hight = button_node:getVirtualRenderer():getSprite()
    button_node:setBrightStyle(ccui.BrightStyle.normal)

    lightNodeWithCCUI(button_normal,isJpg)
    lightNodeWithCCUI(button_Hight,isJpg)

end

function lightWithCCButton_canTouch(button_node,isJpg,isSprite)

    local button_normal = button_node:getVirtualRenderer():getSprite()
    
	lightNodeWithCCUI(button_normal,isJpg,isSprite)

end

function defaultWithCCButton(button_node,isJpg,isSprite)

    local button_normal = button_node:getVirtualRenderer():getSprite()
    button_node:setBrightStyle(ccui.BrightStyle.highlight)
    local button_Hight = button_node:getVirtualRenderer():getSprite()
    button_node:setBrightStyle(ccui.BrightStyle.normal)

	defaultNodeWithCCUI(button_normal,isJpg,isSprite)
	defaultNodeWithCCUI(button_Hight,isJpg,isSprite)

end
local findPro
local function length(str)
    local _, count = string.gsub(str, "[^\128-\193]", "")
    return count
end

local function getHyperLinkText(str,color,fontName,fontSize,lineHeight)
    local tempTxT = ccui.Text:create(str,fontName,fontSize)
    tempTxT:setColor(color)
    --tempTxT:enableOutline(res_color.BLACK,1)
    local nowHeight = tempTxT:getContentSize().height
    local nowWidth = tempTxT:getContentSize().width
    local tempNode = cc.DrawNode:create()
    tempTxT:addChild(tempNode)
    tempNode:clear()
    local r = color.r /255
    local g = color.g /255
    local b = color.b /255
    tempNode:drawSegment(cc.p(0,0),cc.p(nowWidth,0),lineHeight/2,cc.c4f(r,g,b,1))
    tempNode:setPositionY(-lineHeight/2)
    tempTxT:setContentSize(cc.size(nowWidth,nowHeight + lineHeight))
    return tempTxT,nowHeight + lineHeight
end

local function splitStringWithLineWidth(str1,fontName,fontSize,lineWidth,firstLineWidth)
    if lineWidth < fontSize then return nil end
    local lenInByte = #str1
    local width = 0
    local tempStr = ""
    local allStr ={}
    local fixI = 1
    local needFix = false
    if firstLineWidth and firstLineWidth ~= 0 then needFix = true end

    for i=1 ,lenInByte do
        local curByte = string.byte(str1, i)
        local byteCount = 1;
        if curByte>0 and curByte<=127 then
            byteCount = 1
        elseif curByte>=192 and curByte<=223 then
            byteCount = 2
        elseif curByte>=224 and curByte<=239 then
            byteCount = 3
        elseif curByte>=240 and curByte<=247 then
            byteCount = 4
        end
        
        if i == fixI or i == lenInByte then 
            if i <= fixI then
                if byteCount == 1 then
                    width = width + fontSize * 0.5
                else
                    width = width + fontSize
                end
            end
            local nowLineWidth = lineWidth
            if needFix == true then nowLineWidth = lineWidth - firstLineWidth end
            if width > nowLineWidth then
                needFix = false
                local fixWidth = fontSize
                if byteCount == 1 then
                    width = width - fontSize * 0.5
                    fixWidth = fontSize * 0.5
                else
                    width = width - fontSize
                end
                local tempString = {str = tempStr,str_width = width}
                table.insert(allStr,tempString)
                
                local char = string.sub(str1, i, i+byteCount-1)
                fixI = i + byteCount
                tempStr = ""..char
                width = fixWidth
            else
                if i == fixI then
                    local char = string.sub(str1, i, i+byteCount-1)
                    fixI = i + byteCount
                    tempStr = tempStr..char
                end
                
                if i == lenInByte then
                    local tempString = {str = tempStr,str_width = width}
                    table.insert(allStr,tempString)
                end
                
            end
        end
    end
    return allStr
end

local function checkMaxLineTypeSignal(str1,value2Type,tempFixListLine,defaultColor,fontName,fontSize,lineWidth,isNewLine)
    local allFixListLine = List.getSize(tempFixListLine)
   
    local firstLineWidth = 0
    if allFixListLine > 0 then
        local inLineCount = #tempFixListLine[allFixListLine]

        for i=1,inLineCount do
            firstLineWidth = firstLineWidth + tempFixListLine[allFixListLine][i].txtWidth
        end
        if lineWidth - firstLineWidth < fontSize* 0.5 or inLineCount == 0 then
            firstLineWidth = 0
        end 
    end
    
    if isNewLine == true then firstLineWidth = 0 end

    local allLine = splitStringWithLineWidth(str1,fontName,fontSize,lineWidth,firstLineWidth)
    if #allLine > 1 then
        
        for i=1,#allLine do

            local textA = { str = allLine[i].str, type = value2Type , txtColor = defaultColor ,txtWidth =  allLine[i].str_width }

            if firstLineWidth ~= 0 and i == 1 then
                table.insert(tempFixListLine[allFixListLine],textA)

            else
                local tempLine = {}
                table.insert(tempLine,textA)
                List.pushlast(tempFixListLine,tempLine)
            end


        end
    elseif #allLine == 1 then
        local textA = { str = allLine[1].str, type = value2Type , txtColor = defaultColor ,txtWidth =  allLine[1].str_width }

        if firstLineWidth ~= 0  then
            table.insert(tempFixListLine[allFixListLine],textA)

        else
            local tempLine = {}
            table.insert(tempLine,textA)
            List.pushlast(tempFixListLine,tempLine)
        end

    end
end

local function checkMaxLineTypeMix(str1,str2,value2Type,tempFixListLine,defaultColor,str2Color,fontName,fontSize,lineWidth,isNewLine)
    local allFixListLine = List.getSize(tempFixListLine)
   
    local firstLineWidth = 0
    if allFixListLine > 0 then
        local inLineCount = #tempFixListLine[allFixListLine]

        for i=1,inLineCount do
            firstLineWidth = firstLineWidth + tempFixListLine[allFixListLine][i].txtWidth
        end
        if lineWidth - firstLineWidth < fontSize* 0.5 or inLineCount == 0 then
            firstLineWidth = 0
        end 
    end
    if isNewLine == true then firstLineWidth = 0 end

    local allLine = splitStringWithLineWidth(str1,fontName,fontSize,lineWidth,firstLineWidth)
    if #allLine > 1 then
        
        for i=1,#allLine do

            local textA = { str = allLine[i].str, type = 1 , txtColor = defaultColor ,txtWidth =  allLine[i].str_width }

            if firstLineWidth ~= 0 and i == 1 then
                table.insert(tempFixListLine[allFixListLine],textA)

            else
                local tempLine = {}
                table.insert(tempLine,textA)
                List.pushlast(tempFixListLine,tempLine)
            end


        end
        
        local fixWidth = 0
        local leftListCount = List.getSize(tempFixListLine)

        if leftListCount > 0 then
            local inLineCount = #tempFixListLine[leftListCount]
            for i=1,inLineCount do
                fixWidth = fixWidth + tempFixListLine[leftListCount][i].txtWidth
            end
            if lineWidth - fixWidth < fontSize* 0.5 or inLineCount == 0 then
                fixWidth = 0
            end 
        end

        local str2Line = splitStringWithLineWidth(str2,fontName,fontSize,lineWidth,fixWidth)
        if #str2Line > 1 then
            for i=1,#allLine do
                local textA = { str = str2Line[i].str, type = value2Type , txtColor = str2Color ,txtWidth =  str2Line[i].str_width}
                if fixWidth ~= 0 and i == 1 then
                    table.insert(tempFixListLine[leftListCount],textA)

                else
                    local tempLine = {}
                    table.insert(tempLine,textA)
                    List.pushlast(tempFixListLine,tempLine)
                end
            end
        elseif #str2Line == 1 then
            local textA = { str = str2Line[1].str, type = value2Type , txtColor = str2Color ,txtWidth =  str2Line[1].str_width}
            if fixWidth ~= 0  then
                table.insert(tempFixListLine[leftListCount],textA)

            else
                local tempLine = {}
                table.insert(tempLine,textA)
                List.pushlast(tempFixListLine,tempLine)
            end
        end

    elseif #allLine == 1 then
        local textA = { str = allLine[1].str, type = 1 , txtColor = defaultColor ,txtWidth =  allLine[1].str_width }

        if firstLineWidth ~= 0  then
            table.insert(tempFixListLine[allFixListLine],textA)

        else
            local tempLine = {}
            table.insert(tempLine,textA)
            List.pushlast(tempFixListLine,tempLine)
        end
        
        local fixWidth = 0
        local leftListCount = List.getSize(tempFixListLine)

        if leftListCount > 0 then
            local inLineCount = #tempFixListLine[leftListCount]
            for i=1,inLineCount do
                fixWidth = fixWidth + tempFixListLine[leftListCount][i].txtWidth
            end
            if lineWidth - fixWidth < fontSize* 0.5 or inLineCount == 0 then
                fixWidth = 0
            end 
        end

        local str2Line = splitStringWithLineWidth(str2,fontName,fontSize,lineWidth,fixWidth)
        if #str2Line > 1 then
            for i=1,#allLine do
                local textA = { str = str2Line[i].str, type = value2Type , txtColor = str2Color ,txtWidth =  str2Line[i].str_width}
                if fixWidth ~= 0 and i == 1 then
                    table.insert(tempFixListLine[leftListCount],textA)

                else
                    local tempLine = {}
                    table.insert(tempLine,textA)
                    List.pushlast(tempFixListLine,tempLine)
                end
            end
        elseif #str2Line == 1 then
            
                local textA = { str = str2Line[1].str, type = value2Type , txtColor = str2Color ,txtWidth =  str2Line[1].str_width}
                if fixWidth ~= 0  then
                    table.insert(tempFixListLine[leftListCount],textA)

                else
                    local tempLine = {}
                    table.insert(tempLine,textA)
                    List.pushlast(tempFixListLine,tempLine)
                end
                
            
            
        end
        
    end
    
end

function findPro(strInput,defaultColor,fontName,fontSize,lineWidth,tempFixListLine,isNewLine)
    local s_s,s_end,value1,value2 = string.find(strInput,"%[(.-)%]<(.-)>")

    local newStr
    
    if value1 ~= nil then
    
        newStr = string.sub(strInput,s_end+1,-1)
        local colorIndex = tonumber(value1)
        local value2Type = 1 
        if colorIndex >100 then
            colorIndex = colorIndex - 100
            value2Type = 2
        end
        
        if s_s ~= 1 then
            local strBefore = string.sub(strInput,1,s_s-1)
            checkMaxLineTypeMix(strBefore,value2,value2Type,tempFixListLine,defaultColor,RichText_color[colorIndex],fontName,fontSize,lineWidth,isNewLine)
--            --checkMaxLineTypeMix(strBefore,value2,list,defaultColor,RichText_color[colorIndex],fontName,fontSize,lineWidth,tempFixListLine)
--            local textA = { str = strBefore, type = 1 , txtColor = defaultColor }
--            List.pushlast(list,textA)
--            local textB = { str = value2, type = 1 , txtColor = RichText_color[colorIndex] }
--            List.pushlast(list,textB)
        elseif s_s == 1 then
            checkMaxLineTypeSignal(value2,value2Type,tempFixListLine,RichText_color[colorIndex],fontName,fontSize,lineWidth,isNewLine)
            --checkMaxLineTypeMix(value2,value2,list,defaultColor,RichText_color[colorIndex],fontName,fontSize,lineWidth,tempFixListLine)
--            local textC = { str = value2, type = 1 , txtColor = RichText_color[colorIndex] }
--            List.pushlast(list,textC)
        end
        return true,newStr
    else
        checkMaxLineTypeSignal(strInput,1,tempFixListLine,defaultColor,fontName,fontSize,lineWidth,isNewLine)
        --checkMaxLineTypeMix(strInput,strInput,list,defaultColor,RichText_color[colorIndex],fontName,fontSize,lineWidth,tempFixListLine)
--        local textA = { str = strInput, type = 1 , txtColor = defaultColor }
--        List.pushlast(list,textA)
        return false,strInput
    end
end

function getRichText(str,defaultColor,lineWidth,fontName,fontSize,m_scrollview,suffix,lineSpace,spaceHieght)
    local s_s,s_end,value1,value2 = string.find(str,"(%\r%\n)")
    local nowsuffix = "\\n"
    if s_s ~= nil then
        nowsuffix = "\r\n"
    end
    
    local nowSpace = 0
    local nowLineSpace = 0
    if spaceHieght and spaceHieght ~= 0 then nowSpace = spaceHieght end
    if lineSpace and lineSpace ~= 0 then nowLineSpace = lineSpace end
    
    local allLine = string.split(str,nowsuffix)
   
    local LineCount = #allLine
    local allscrollviewChild = m_scrollview:getChildren()
    
    local pixelHeight = 0
    
    if #allscrollviewChild > 1 then
        for i=1,#allscrollviewChild do
            if allscrollviewChild[i].lineHeight ~= nil then
                pixelHeight = pixelHeight + allscrollviewChild[i].lineHeight
            end
        end
    end
    
    local tempFixListLine = List.new()
    

    
    
    for i=1,LineCount do
        local tempStr =  allLine[i] 
        if tempStr ~= "" then
            local shouldCheck = true
            local newLine = true
    
            while shouldCheck == true do
                shouldCheck,tempStr = findPro(tempStr,defaultColor,fontName,fontSize,lineWidth,tempFixListLine,newLine)
                newLine = false
            end
        else
            List.pushlast(tempFixListLine,{})
        end
    end
    
    for i=1,List.getSize(tempFixListLine) do
        local _richText = ccui.RichText:create()
        _richText:ignoreContentAdaptWithSize(false)
        _richText:setContentSize(cc.size(lineWidth, (fontSize + nowSpace)))
        
        local needFix = 0
        if #tempFixListLine[i] == 0 then
            local re = ccui.RichElementText:create(0, defaultColor, 255, " ", fontName, fontSize)
            _richText:pushBackElement(re)
        else
            for j=1,#tempFixListLine[i] do
                
                if tempFixListLine[i][j].type == 1 then
                    local re = ccui.RichElementText:create(0, tempFixListLine[i][j].txtColor, 255, tempFixListLine[i][j].str, fontName, fontSize)
                    _richText:pushBackElement(re)
                elseif tempFixListLine[i][j].type == 2 then
                    local nowTxT,fixHeight = getHyperLinkText(tempFixListLine[i][j].str,tempFixListLine[i][j].txtColor,fontName,fontSize,2)
                    if fixHeight > 0 then 
                        needFix = fixHeight
                        _richText:setContentSize(cc.size(lineWidth, (fixHeight + nowSpace)))
                    end
                    local re = ccui.RichElementCustomNode:create(0,tempFixListLine[i][j].txtColor,255,nowTxT)
                    _richText:pushBackElement(re)
                end
                
                                      
            end
        end
        
        local nowParameter = ccui.LinearLayoutParameter:create()
        nowParameter:setGravity(ccui.LinearGravity.left)
        _richText:setLayoutParameter(nowParameter)

        m_scrollview:addChild(_richText)
        if needFix > 0 then
            _richText.lineHeight = needFix + nowSpace
            pixelHeight = pixelHeight + (needFix + nowSpace)
        else
            _richText.lineHeight = fontSize + nowSpace
            pixelHeight = pixelHeight + (fontSize + nowSpace)
        end
        m_scrollview:forceDoLayout()
        
    end
    
    if nowLineSpace > 0 then
        local _richText = ccui.RichText:create()
        _richText:ignoreContentAdaptWithSize(false)
        _richText:setContentSize(cc.size(lineWidth, nowLineSpace))
        
        local re = ccui.RichElementText:create(0, defaultColor, 255, " ", fontName, nowLineSpace)
        _richText:pushBackElement(re)
        
        local nowParameter = ccui.LinearLayoutParameter:create()
        nowParameter:setGravity(ccui.LinearGravity.left)
        _richText:setLayoutParameter(nowParameter)
        
        
        
        m_scrollview:addChild(_richText)
        _richText.lineHeight = nowLineSpace
        pixelHeight = pixelHeight + nowLineSpace
        m_scrollview:forceDoLayout()
    end
    
    

    if m_scrollview:getInnerContainerSize().height < pixelHeight then
        m_scrollview:setInnerContainerSize(cc.size(m_scrollview:getInnerContainerSize().width,pixelHeight))
    end
    m_scrollview:forceDoLayout()
    m_scrollview:setSwallowTouches(false)
    List.removeAll(tempFixListLine)
    tempFixListLine = nil
end

