

--͵
local _class = {}

function BaseClass(super)

	-- һ
	local class_type = {}
	-- ڴʱԶ
	class_type.__init = false
	class_type.__delete = false
	class_type.super = super

	class_type.New = function(...)
		-- һ
		local obj = {}
		obj._class_type = class_type

		-- ڳʼ֮ǰע෽
		setmetatable(obj, { __index = _class[class_type] })

		-- óʼ
		do
			local create 
			create = function(c, ...)
				if c.super then
					create(c.super, ...)
				end
				
				if c.__init then
					c.__init(obj, ...)
				end
			end

			create(class_type, ...)
		end

		-- עһdelete
		obj.DeleteMe = function(self)
			local now_super = self._class_type 
			while now_super ~= nil do
				
				if now_super.__delete then
					now_super.__delete(self)
				end
				now_super = now_super.super
			end
		end

		return obj
	end

	local vtbl = {}
	_class[class_type] = vtbl	

	setmetatable(class_type, {__newindex =
		function(t,k,v)
			vtbl[k] = v
		end
		, 
		__index = vtbl, --For call parent method
	})

	if super then
		setmetatable(vtbl, {__index =
			function(t,k)
				local ret = _class[super][k]
				--do not do accept, make hot update work right!
				--vtbl[k] = ret
				return ret
			end
		})
	end

	return class_type
end



TimerQuest = TimerQuest or BaseClass()

---------------------------------------------------------------------------------------------------
TimerQuestItem = TimerQuestItem or BaseClass()

function TimerQuestItem:__init(id, agent, quest_func, is_not_clear_in_reconnect)
	self.id = id
	self.agent = agent
	self.quest_func = quest_func
	self.run_cnt = 0
	self.next_run_time = -1
	self.run_period = 0
	self.is_always_run = false
	self.lost_time = -1
	self.period_growth = math.huge
	self.is_not_clear_in_reconnect = is_not_clear_in_reconnect or false

	-- local func_info = debug.getinfo(quest_func)
	-- self.msg = string.gsub(func_info.source, "^.*/", "")..":"..func_info.linedefined
end

function TimerQuestItem:__delete()
	self.quest_func = nil
	self.agent = nil
end

function TimerQuestItem:Run()
	if not self.agent then
		return true
	end
	if not self.agent:IsQuestAlive( self.id ) then
		return true
	end
	
	if self.quest_func then
		-- self.quest_func()
		local  ret, error_msg = pcall( self.quest_func )
		if not ret then 
			logerror(error_msg)
		end
	end
	self.run_cnt = self.run_cnt +1

	if self.period_growth ~= math.huge then
		self.run_period = self.run_period + self.period_growth
	end

	self.next_run_time = self.next_run_time + self.run_period

	if (not self.is_always_run) and ( self.next_run_time > self.lost_time ) then
		return true
	end
	return false
end



---------------------------------------------------------------------------------------------------------


function TimerQuest:Instance()
	if not TimerQuest.instance then
		TimerQuest.New()
	end
	return TimerQuest.instance
end

function TimerQuest:__init()
	if TimerQuest.instance then 
		error( "TimerQuest instance is existed" )
	end

	TimerQuest.instance = self
	self.id_cnt = 0 
	self.now_time = Time_GetCurrentTime()
	self.timer_quest_map = {}
	self.keep_id_set = {}
end

function TimerQuest:__delete()
	self.timer_quest_map = {}
	self.keep_id_set = {}
	TimerQuest.instance = nil
end


--[[ʱִһ]]
function TimerQuest:AddDelayQuest(quest_func, delay_time, is_not_clear_in_reconnect)
	if delay_time < 0 then
		logerror("delay_time must bigger than 0 ", debug.traceback() )
		return
	end
	self.id_cnt = self.id_cnt + 1
	local quest = TimerQuestItem.New(self.id_cnt, self,  quest_func, is_not_clear_in_reconnect)

	quest.is_always_run = false	

	quest.next_run_time = self.now_time + delay_time 
	if not self.timer_quest_map[quest.next_run_time] then
		self.timer_quest_map[quest.next_run_time] = {}
	end

	table.insert(self.timer_quest_map[quest.next_run_time], quest )
	self.keep_id_set[quest.id] = true
	return quest.id
end

--[[ִһ]]
function TimerQuest:AddPeriodQuest(quest_func, period, last_time, is_not_clear_in_reconnect)
	self.id_cnt = self.id_cnt + 1
	local quest = TimerQuestItem.New( self.id_cnt, self, quest_func, is_not_clear_in_reconnect)
	quest.is_always_run = (last_time < 0.0)					
	quest.next_run_time = self.now_time + period			
	quest.run_period = period								
	quest.lost_time = self.now_time + last_time				

	if not self.timer_quest_map[quest.next_run_time] then
		self.timer_quest_map[quest.next_run_time] = {}
	end
	table.insert(self.timer_quest_map[quest.next_run_time], quest )
	self.keep_id_set[quest.id] = true
	return quest.id
end


--[[ִָ]]
function TimerQuest:AddRunTimesQuest( quest_func, period, run_count)
	self.id_cnt = self.id_cnt + 1
	local quest = TimerQuestItem.New( self.id_cnt, self, quest_func )
	
	quest.is_always_run = (run_count < 0)								
	quest.next_run_time = self.now_time + period						
	quest.run_period = period											
	quest.lost_time = self.now_time + period*run_count + 0.1*period 	

	if not self.timer_quest_map[quest.next_run_time] then
		self.timer_quest_map[quest.next_run_time] = {}
	end
	table.insert(self.timer_quest_map[quest.next_run_time], quest )
	self.keep_id_set[quest.id] = true

	return quest.id
end

--[[   ʱ ]]
-- !!! Ϊһֱִ
function TimerQuest:AddPeriodGrowthQuest( quest_func, period_start, period_growth, last_time )
	self.id_cnt =  self.id_cnt + 1
	local quest = TimerQuestItem.New( self.id_cnt, self, quest_func )

	quest.is_always_run = (last_time < 0.0)	
 	quest.next_run_time =  self.now_time + period_start    
	quest.run_period = period_start
	quest.lost_time = self.now_time + last_time
	quest.period_growth = period_growth

	if not self.timer_quest_map[ quest.next_run_time ] then
		self.timer_quest_map[quest.next_run_time] = {}
	end
	table.insert( self.timer_quest_map[quest.next_run_time], quest)
	self.keep_id_set[quest.id] = true

	return quest.id
end

function  TimerQuest:CancelQuest( quest_id )
	if quest_id == nil then 
		return 
	end
	self.keep_id_set[quest_id] = nil
end


function TimerQuest:Stop()
	self.timer_quest_map = {}
	self.keep_id_set = {}
end

function TimerQuest:ClearInReconnect()
	for _, questSet in pairs(self.timer_quest_map) do
		if questSet ~= nil then
			for i=#questSet,1,-1 do
				local quest = questSet[i]
				if quest and not quest.is_not_clear_in_reconnect then 
					self:CancelQuest( quest.id )
					quest:DeleteMe()
					quest = nil
					table.remove(questSet, i)
				end
			end
		end 
	end
end

function TimerQuest:IsQuestAlive( quest_id )
	if self.keep_id_set[quest_id] then
		return true
	end
	return false
end

function TimerQuest:GetExecutableQuest(now_time)
	local questKeyMap = {}
	for key, value in pairs(self.timer_quest_map) do
		if key <= now_time then
			table.insert(questKeyMap, key)
		end
	end
	return questKeyMap 
end

function  TimerQuest:Update(now_time)
	self.now_time = now_time
	local questKeyMap = self:GetExecutableQuest(now_time)

	for _, value in ipairs(questKeyMap) do
		local questSet = self.timer_quest_map[value]
		if questSet == nil then
			questSet = {}
		end
		for index, quest in ipairs(questSet) do
			-- BeginSample(quest.msg)
			local need_del = quest:Run()
			-- EndSample()
			if not need_del then
				if not self.timer_quest_map[quest.next_run_time] then
					self.timer_quest_map[quest.next_run_time] = {}
				end
				table.insert( self.timer_quest_map[quest.next_run_time], quest)
			else
				self:CancelQuest( quest.id )
				quest:DeleteMe()
				quest = nil
			end
		end

		self.timer_quest_map[value] = nil
	end
end